﻿//$reference parts\core\gamePlay.dll
using System;
using System.IO;
using maddox.game;
using maddox.game.world;
using maddox.GP;
using System.Collections.Generic;
using System.Globalization;

public class Mission : AMission
{
    private bool odenDebug = false;
    const string vFile = "DynMis40f"; // version filename
    const string vName = "DynMis R6"; // version caption
    private int ATK = 0;
    const int RAF = 1;
    const int LW = 2;
    const string fmDungeness = "FrontMarker13";
    const string fmDover = "FrontMarker14";
    const string fmRamsgate = "FrontMarker17";
    const string fmCanterbury = "FrontMarker18";
    const string fmRochester = "FrontMarker19";
    const string fmLondon = "FrontMarker20";
    const int idxDungeness = 13;
    const int idxDover = 14;
    const int idxRamsgate = 17;
    const int idxCanterbury = 18;
    const int idxRochester = 19;
    const int idxLondon = 20;

    // mission frontline is used to simulate Observer Corps when Chain Home Radar is out (2 minute delay)
    internal class MissionAirbase
    {   // Squadron or Gruppe Airbase at Airfield, there may be multiple "airbases" per airfield
        internal string name;
        internal int army;
        internal double x;
        internal double y;
        internal int startAlt;
        internal int maxPlanes;
        internal bool parked;
        internal bool parachute;
        internal string country;
        internal string type;
        internal string sqn;
        internal string sector;
        internal string aircraftType;
        internal AiAirport ap;
        internal string airportName;
        internal Point3d pt3d;

        public Point3d Pos() { return pt3d; }
        public void MoveSquadronToAirport(AiAirport apNew)
        {
            this.x = apNew.Pos().x;
            this.y = apNew.Pos().y;
            this.airportName = apNew.Name();
            this.pt3d = apNew.Pos();
        }

        internal MissionAirbase(string name
            , int army
            , double x
            , double y
            , int startAlt
            , int maxPlanes
            , bool parked
            , bool parachute
            , string country
            , string type
            , string sqn
            , string sector
            , string aircraftType
            , AiAirport ap
            , string airportName
            )
        {
            this.name = name;
            this.army = army;
            this.x = x;
            this.y = y;
            this.startAlt = startAlt;
            this.maxPlanes = maxPlanes;
            this.parked = parked;
            this.parachute = parachute;
            this.country = country;
            this.type = type;
            this.sqn = sqn;
            this.sector = sector;
            this.aircraftType = aircraftType;
            this.ap = ap;
            this.airportName = airportName;
            this.pt3d = new Point3d(x, y, 0);
        }
    }

    internal class DynCampEvent
    {
        internal DateTime dateTime;
        internal string action; // add/remove/move
        internal string type; // AirGroup
        internal string typeValue; // BoB_RAF_F_92Sqn
        internal string airportName; // Tangmere
        internal string param1; // 12
        internal string param2; // Aircraft.SpitfireMkIa_100oct
        internal string param3; // n/a
        internal DynCampEvent(DateTime dt, string action, string type, string typeValue, string airportName, string param1, string param2, string param3)
        {
            this.dateTime = dt;
            this.action = action;
            this.type = type;
            this.typeValue = typeValue;
            this.airportName = airportName;
            this.param1 = param1;
            this.param2 = param2;
            this.param3 = param3;
        }
    }

    internal class MissionMarker
    {
        internal double x;
        internal double y;
        internal int army;
        internal string key;
        internal MissionMarker(double x, double y, int army, string key) { this.x = x; this.y = y; this.army = army; this.key = key; }
    }

    internal class GroundColumn
    {
        internal string name;
        internal Point2d fr2d;
        internal Point2d to2d;
        internal AiGroundGroup group;
        internal IRecalcPathParams pathParams;
        internal GroundColumn(string name, Point2d fr2d, Point2d to2d, IRecalcPathParams pathParams) { this.name = name; this.fr2d = fr2d; this.to2d = to2d; this.group = null; this.pathParams = pathParams; }
    }

    internal class PlayerData
    {
        internal Player player;
        internal AiAirport homeplate;

    }

    private int GameOver = 0; // 1=RAF, 2=LW
    Random rnd = new Random();
    private int CAPStaffelIndex = 0, CAPFlightIndex = 0;
    private int BMBStaffelIndex = 0, BMBFlightIndex = 0, BMBFlightAlt = 0, BMBFlightSpeed = 0;
    private int misConvoyKills = 0, misRAFDeadF = 0, misLWDeadB = 0, misLWDeadF = 0, misRAFDeadB = 0, misLWDeadS = 0;
    private int addDays = 0;
    private int gameWon = 0; // RAF, LW
    private int columnUnits = 0, destroyedUnits = 0;
    private MissionAirbase BMBFlightAB = null;
    private int NextMissionNum = 0; // sub missions
    private double bombsOnTarget = 0;
    private bool gameIsOn = false;
    private bool playerWasAirborne = false;
    private bool bombersAtTarget = false;
    private bool detectedByObsCorps = false;
    private bool playerIsBomber = false;
    private bool firstMessage = true;
    private string missionCaption;
    private string localGroupLeaderSector = "Init";
    private string DoverFolder, CampInitFile, CampMisFile, CampBriFile;
    private string oldSqn = "";
    private string defSquadron, defPlayer, defClass, defCallSign, defSkin, defSerial, defTail; // from mis file
    private string defSatellites = "";
    private string tailNoSerialNo = "(verify tailnumber 'D' and serialno. 'R6708' and";
    private string groundAssaultFrom = "";
    private List<PlayerData> PlayersData = new List<PlayerData>();
    private List<string> mainStoryList = new List<string>();
    private List<string> strDynCampList = new List<string>();
    private List<string> AirGroupNames = new List<string>();
    private List<string> GndStatDead = new List<string>();
    private List<string> GndStatRestore = new List<string>();
    private List<string> GndGrpDead = new List<string>();
    private List<string> AirportsDamaged = new List<string>();
    private List<string> CHRadarDamaged = new List<string>();
    private List<string> PortDamaged = new List<string>();
    private List<string> FactoryDamaged = new List<string>();
    private List<string> Satellites = new List<string>();
    private List<string> activatedGroups = new List<string>();
    private List<string> activatedAirports = new List<string>();
    private List<string> sweepGroups = new List<string>();
    private List<int> AirGroupIDs = new List<int>();
    private List<GroundStationary> ChainHomeRadars = new List<GroundStationary>();
    private List<GroundStationary> Industries = new List<GroundStationary>();
    private List<GroundStationary> PortCranes = new List<GroundStationary>();
    private List<AiGroundGroup> ConvoyShips = new List<AiGroundGroup>();
    private List<MissionMarker> MissionMarkers = new List<MissionMarker>();
    private List<MissionAirbase> MissionAirbases = new List<MissionAirbase>();
    private List<DynCampEvent> DynCampEvents = new List<DynCampEvent>();
    private List<GroundColumn> GroundColumns = new List<GroundColumn>();
    private List<AiAirGroup> AiAirGroupsCAP = new List<AiAirGroup>();
    private List<AiAirGroup> AiAirGroupsStdby = new List<AiAirGroup>();
    private List<AiAirport> PossibleAirports = new List<AiAirport>();
    private AiAirGroup TargetGroup = null;
    private AiAirGroup EscortGroup = null;
    private MissionAirbase homeBase = null;
    private AiAirport homePlate;
    private AiAirport targetAirport = null;
    private AiAirport playerLandedAtAirport = null;
    private Point3d homePlate3d;
    private Point2d target2d = new Point2d(0, 0);
    private Point3d target3d = new Point3d(0, 0, 0);
    private string targetName = "";
    private AiAircraft FighterCommand = null;
    private GroundStationary mainGSTarget = null;
    private AiActor convoyTarget = null;
    private AiWayPoint oldWP;
    private int idxTarget;
    private IRecalcPathParams pathParams = null;
    private GroundColumn gc = null;
    private AiGroundGroup GroundTarget = null;
    private DateTime gameDateTime = new DateTime(1940, 7, 10, 5, 0, 0);
    private DateTime highOctaneDate = new DateTime(1940, 7, 10, 0, 0, 0);
    private DateTime ReinforcementDate = new DateTime(1940, 7, 13, 0, 0, 0);
    private DateTime timeoutRadio = DateTime.Now;
    private DateTime BoBDateTime = new DateTime(1940, 7, 10, 0, 0, 0);
    private DateTime dtCoastalBattles = new DateTime(1940, 8, 8, 0, 0, 0);
    private DateTime dtAdlerTag = new DateTime(1940, 8, 24, 0, 0, 0);
    private DateTime dtBlitz = new DateTime(1940, 9, 7, 0, 0, 0);
    private DateTime dtFighterBomber = new DateTime(1940, 10, 1, 0, 0, 0);



    public override void OnBattleStarted()
    {
        base.OnBattleStarted();

        MissionNumberListener = -1;
        NextMissionNum = GamePlay.gpNextMissionNumber();
        if (odenDebug) GamePlay.gpLogServer(null, "OnBattleStarted: '{0}' {1}", new object[] { "readMission", NextMissionNum });
        if (GamePlay is GameServerDef)
        {
            GameServerDef gameServer = (GameServerDef)GamePlay;
            ISectionFile missFile = gameServer.game.FirstMissionFile();
            ReadMissionFile(missFile);
        };
        if (GamePlay is GameSingleDef)
        {
            GameSingleDef gameServer = (GameSingleDef)GamePlay;
            ISectionFile missFile = gameServer.game.FirstMissionFile();
            ReadMissionFile(missFile);
        };
        if (odenDebug) GamePlay.gpLogServer(null, "OnBattleStarted: '{0}' {1}", new object[] { "CheckFiles", GamePlay.gpPlayer().Name() });
        CheckFiles();

        MissionNumberListener = -1;
        NextMissionNum = GamePlay.gpNextMissionNumber();

        if (odenDebug) GamePlay.gpLogServer(null, "OnBattleStarted: '{0}' {1}", new object[] { "ReadInit", GamePlay.gpPlayer().Name() });
        ReadInitFile();

        ISectionFile f = GamePlay.gpCreateSectionFile();
        // Fighter Command Control Voice, triggers
        f.add("AirGroups", "gb01.01", "");
        f.add("gb01.01", "Flight0", "1");
        f.add("gb01.01", "Class", "Aircraft.AnsonMkI");
        f.add("gb01.01", "Formation", "LINEABREAST");
        f.add("gb01.01", "CallSign", "0");
        f.add("gb01.01", "Fuel", "0");
        f.add("gb01.01", "Weapons", "0 0");
        f.add("gb01.01", "SetOnPark", "1");
        f.add("gb01.01", "Idle", "1");
        f.add("gb01.01", "Skill", "0.54 0.09 0.43 0.54 0.54 0.59 0.43 0.54");
        //f.add("gb01.01", "Person0_0", "1 0.54 0.09 0.43 0.54 0.54 0.59 0.43 0.54"); // gb only have 2 actors (de has 4)
        f.add("gb01.01_Way", "TAKEOFF", "135077.50 284460.35 0 0");

        f.add("AirGroups", "g06.01", "");
        f.add("g06.01", "Flight0", "1");
        f.add("g06.01", "Class", "Aircraft.Bf-108B-2");
        f.add("g06.01", "Formation", "LINEABREAST");
        f.add("g06.01", "CallSign", "26");
        f.add("g06.01", "Fuel", "0");
        f.add("g06.01", "Weapons", "0 0");
        f.add("g06.01", "SetOnPark", "1");
        f.add("g06.01", "Idle", "1");
        f.add("g06.01", "Skill", "0.54 0.09 0.43 0.54 0.54 0.59 0.43 0.54");
        //f.add("g06.01", "Person0_0", "1 0.54 0.09 0.43 0.54 0.54 0.59 0.43 0.54"); // gb only have 2 actors (de has 4)
        f.add("g06.01_Way", "TAKEOFF", "352655.02 14633.14 0 0");
        GamePlay.gpPostMissionLoad(f);
        NextMissionNum++;

        if (odenDebug) GamePlay.gpLogServer(null, "OnBattleStarted: '{0}' {1} {2}", new object[] { "StartMission", GamePlay.gpPlayer().Name(), GamePlay.gpPlayer().Place().Pos() });
        StartMission();

        if (GamePlay.gpIsServerSingle())
        {
            SetOrderMissionMenuOden(GamePlay.gpPlayer());
            PlayerData pd = new PlayerData { player = GamePlay.gpPlayer(), homeplate = FindAirport(GamePlay.gpPlayer().Army(), GamePlay.gpPlayer().Place().Pos(), 0) };
            PlayersData.Add(pd);
        };
        homePlate = FindAirport(GamePlay.gpPlayer().Army(), GamePlay.gpPlayer().Place().Pos(), 0);
    }

    public override void OnBattleStoped()
    {
        /* makes no difference in log file
        // clean up all spawned
        foreach (int i in GamePlay.gpArmies())
            if (GamePlay.gpAirGroups(i) != null)
                foreach (AiAirGroup ag in GamePlay.gpAirGroups(i))
                    foreach (AiActor a in ag.GetItems())
                        if (a is AiAircraft)
                            (a as AiAircraft).Destroy();
        */
        if (playerLandedAtAirport != null)
            if (Satellites.Contains(playerLandedAtAirport.Name()))
                homeBase.MoveSquadronToAirport(playerLandedAtAirport);

        if (odenDebug)
        {
            GamePlay.gpLogServer(null, "OnBattleStoped: '{0}' ATK={1}", new object[] { homeBase.name, ATK });
            //GameOver = true;
            //playerWasAirborne = true;
            //homeBase.maxPlanes -= 3;

            if (TargetGroup == null)
            {   // find possible target group
                AiAirGroup[] airGroups = GamePlay.gpAirGroups(ATK);
                foreach (AiAirGroup ag in airGroups)
                    if (ag.hasBombs() || ag.hasTorpedos() || ag.hasBombsInInternalBays())
                    {
                        GamePlay.gpLogServer(null, "OnBattleStoped: '{0}'", new object[] { ag.Name() });
                        TargetGroup = ag;
                        if (odenDebug) GamePlay.gpLogServer(null, "playerWasAirborne: bombersAtTarget={0} grp={1}", new object[] { bombersAtTarget, TargetGroup.Name() });
                        break;
                    };
            };
        };

        if (playerWasAirborne)
        {
            if (!bombersAtTarget)
            {   // someone tried to milk the mission :)
                if (targetAirport != null)
                    if (TargetGroup != null)
                        if (TargetGroup.hasBombs() || TargetGroup.hasBombsInInternalBays())
                            bombsOnTarget += rnd.Next(1, 5) * Math.Pow(targetAirport.CoverageR(), 2) / 100;

                if (convoyTarget != null)
                    if (TargetGroup != null)
                        if (TargetGroup.hasBombs() || TargetGroup.hasTorpedos() || TargetGroup.hasBombsInInternalBays())
                        {
                            if (odenDebug) GamePlay.gpLogServer(null, "playerWasAirborne: bombersAtTarget={0} convoyTarget={1}", new object[] { bombersAtTarget, convoyTarget.Name() });
                            Point3d pt3d = new Point3d(convoyTarget.Pos().x, convoyTarget.Pos().y, 0);
                            foreach (AiGroundGroup gg in ConvoyShips)
                                if (gg.Pos().distance(ref pt3d) < 16090)
                                {
                                    GndGrpDead.Add(gg.Name());
                                    misConvoyKills += 1;
                                    highOctaneDate = highOctaneDate.AddDays(1);
                                    bombsOnTarget = rnd.Next(1000, 3000);
                                };
                        };

                if (mainGSTarget != null)
                    if (TargetGroup != null)
                        if (TargetGroup.hasBombs() || TargetGroup.hasTorpedos() || TargetGroup.hasBombsInInternalBays())
                        {
                            if (odenDebug) GamePlay.gpLogServer(null, "playerWasAirborne: bombersAtTarget={0} mainGSTarget={1}", new object[] { bombersAtTarget, mainGSTarget.Name });
                            Point2d pt2d = new Point2d(mainGSTarget.pos.x, mainGSTarget.pos.y);
                            if (mainGSTarget.Title.Contains("EnglishRadar3"))
                            {
                                string areaName = ChainHomeRadarName(pt2d);
                                mainStoryList.Add("Luftwaffe destroyed " + areaName + " Chain Home Radar Station");
                                GndStatDead.Add(mainGSTarget.Name);
                            };
                            if (mainGSTarget.Title.Contains("PortCrane1_UK1"))
                            {
                                string areaName = PortDockName(pt2d);
                                mainStoryList.Add("Luftwaffe destroyed " + areaName + ", 100 octane fuel deliveries and factory output delayed for a day");
                                GndStatDead.Add(mainGSTarget.Name);
                            };
                            if (mainGSTarget.Title.Contains("Oxygen_Facility"))
                            {
                                mainStoryList.Add("Luftwaffe destroyed industry area in London, factory output delayed for a week");
                                GndStatDead.Add(mainGSTarget.Name);
                            };
                            mainGSTarget.Destroy();
                            bombsOnTarget = rnd.Next(1000, 3000);
                        };
                if (groundAssaultFrom.Length > 0)
                    MissionMarkers[idxTarget].army = ATK;
            };
            if (odenDebug)
                if (mainStoryList.Count > 0)
                    GamePlay.gpLogServer(null, "summary: bombsOnTarget={0}, {1}", new object[] { bombsOnTarget, mainStoryList[mainStoryList.Count - 1] });

            if (columnUnits > 0)
                if (columnUnits > destroyedUnits * 2)
                {
                    if (ATK == LW) mainStoryList.Add("Germany captured " + MissionMarkerName(MissionMarkers[idxTarget].key));
                    else mainStoryList.Add("Great Britain captured " + MissionMarkerName(MissionMarkers[idxTarget].key));
                    MissionMarkers[idxTarget].army = ATK;
                    if (idxTarget == idxRochester)
                    {
                        MissionMarkers[idxRamsgate].army = ATK;
                        MissionMarkers[idxDover].army = ATK;
                        MissionMarkers[idxDungeness].army = ATK;
                        MissionMarkers[idxCanterbury].army = ATK;
                    };
                };

            addDays = rnd.Next(1, 3);
            gameDateTime = gameDateTime.AddDays(addDays);
            if (highOctaneDate < gameDateTime) highOctaneDate = gameDateTime;
            UpdateInitFile();


            if (GameOver > 0)
            {
                /*
                if (File.Exists(CampInitFile))
                    File.Delete(CampInitFile);
                GameOverBriefing();
                */
                UpdateBriefing();
                UpdateMissionFile();
            }
            else
            {
                UpdateBriefing();
                UpdateMissionFile();
            };
            
        };

        base.OnBattleStoped();
        //foreach (GPUserLabel ul in UserLabels) GamePlay.gpDeleteUserLabel(ul); // if some are still around
        //UserLabels.Clear();
        //GamePlay.gpLogServer(null, "OnBattleStoped: '{0}'", new object[] { "Oden Dynamic Campaign" });
    }

    public override void OnMissionLoaded(int missionNumber)
    {
        base.OnMissionLoaded(missionNumber);

        CAPStaffelIndex = 0;
        CAPFlightIndex = 0;
        BMBStaffelIndex = 0;
        BMBFlightIndex = 0;
        BMBFlightAlt = 0;
        BMBFlightSpeed = 0;
        BMBFlightAB = null;
        activatedAirports.Clear();
    }

    public override void OnTickGame()
    {
        if (Time.tickCounter() % 150 == 0) // every 5s c:a
        {
            /*
            if (odenDebug)
                if (EscortGroup != null)
                    GamePlay.gpLogServer(null, "EscortGroup: {0}", new object[] { EscortGroup.Name() });
            */
            if (gameWon == 0)
                if (columnUnits > 0)
                    if (columnUnits < destroyedUnits * 2)
                    {
                        gameWon = ATK == RAF ? LW : RAF;
                        sayMissionOver(gameWon);
                        GamePlay.gpLogServer(null, "UpdateRTB from {0}", new object[] { "OnTickGame columnUnits < destroyedUnits * 2" });
                        UpdateRTB(bombersAtTarget);
                        TargetGroup = null;
                    };
        };

        if (Time.tickCounter() % 3600 == 3500) // every 2min c:a
        {
            if (groundAssaultFrom.Length > 0 && EscortGroup == null && GamePlay.gpPlayer() != null)
            {
                AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
                if (pac != null)
                    sayGroundTarget(pac, pac.AirGroup(), target2d);
                foreach (AiAirGroup ag in GamePlay.gpAirGroups(ATK == LW ? RAF : LW))
                    UpdateGroundTarget(ag);
                if (EscortGroup == null)
                    if (GroundTarget != null)
                        if (GroundTarget.GetItems().Length > 0)
                        {
                            ISectionFile f = GamePlay.gpCreateSectionFile();
                            CreateBMBMission(f, target2d, 3, ATK == LW ? RAF : LW, GroundTarget.GetItems()[0], null, false);
                            GamePlay.gpPostMissionLoad(f);
                            NextMissionNum++;
                        };
            };
        };

        if (Time.tickCounter() % 3600 == 1500) // every 2min c:a
        {
            DateTime currDT = DateTime.Now;


            foreach (AiAirGroup ag in GamePlay.gpAirGroups(LW == ATK ? RAF : LW))
                if (ag.isAircraftType(AircraftType.Fighter)
                    || ag.isAircraftType(AircraftType.BNZFighter)
                    || ag.isAircraftType(AircraftType.HeavyFighter)
                    || ag.isAircraftType(AircraftType.TNBFighter))
                    if (TargetGroup != null) InterceptWP((AiGroup)ag);
                    else UpdateWP((AiGroup)ag);


            if (EscortGroup != null)
                foreach (AiAirGroup ag in GamePlay.gpAirGroups(EscortGroup.getArmy()))
                    if (ag.isAircraftType(AircraftType.Fighter)
                        || ag.isAircraftType(AircraftType.BNZFighter)
                        || ag.isAircraftType(AircraftType.HeavyFighter)
                        || ag.isAircraftType(AircraftType.TNBFighter))
                        if (!ag.hasBombs() && !ag.hasTorpedos() && !ag.hasBombsInInternalBays())
                            GiveNewWP(ag, HalfwayToTarget2d(ag)); // only if at last WP

            if (playerIsBomber && GamePlay.gpPlayer() != null)
            {
                AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
                AiAirGroup pag = pac.AirGroup();
                if (pag.GetWay().Length == 2 && pag.GetCurrentWayPoint() == 1)
                {   // only default wp's
                    if (convoyTarget != null) CreatePlayerBombMission(convoyTarget.Pos(), convoyTarget);
                    else CreatePlayerBombMission(target3d, null);
                };
                sayBombTarget(pac, pag, target2d);
            };

            if (EscortGroup != null && timeoutRadio < currDT)
                EscortUpdateMsg();

            if (TargetGroup != null && timeoutRadio < currDT)
            {   // ground control Fighter Command
                GndCtrlUpdateMsg();
            }
            else
            {
                if (TargetGroup == null)
                {
                    if (gameIsOn)
                    {
                        if (gameWon == 0)
                            foreach (AiAirGroup ag in GamePlay.gpAirGroups(ATK))
                                if (ag.hasBombs() || ag.hasTorpedos() || ag.hasBombsInInternalBays())
                                {
                                    if (ag.GetItems()[0] != null)
                                    {
                                        if (TargetGroup == null)
                                            if (ChainHomeRadarDetected((AiAircraft)ag.GetItems()[0]))
                                                TargetGroup = ag;
                                            else
                                            {   // Observer Corps
                                                if (GamePlay.gpFrontArmy(ag.Pos().x, ag.Pos().y) == RAF)
                                                    if (detectedByObsCorps) TargetGroup = ag;
                                                    else detectedByObsCorps = true;
                                            };
                                        if (EscortGroup == null)
                                            EscortGroup = ag;
                                    };
                                    if (TargetGroup != null)
                                        break;
                                };
                        if (TargetGroup != null)
                            timeoutRadio = DateTime.Now; // interrupt any chattering
                    };
                };

                if (EscortGroup == null)
                {
                    if (gameIsOn)
                    {
                        if (gameWon == 0)
                            foreach (AiAirGroup ag in GamePlay.gpAirGroups(ATK == LW ? RAF : LW)) // counter bombers
                                if (ag.hasBombs() || ag.hasTorpedos() || ag.hasBombsInInternalBays())
                                    if (ag.GetItems()[0] != null)
                                        EscortGroup = ag;
                        if (EscortGroup != null)
                            timeoutRadio = DateTime.Now; // interrupt any chattering
                    };
                };
            };
        };

        if (Time.tickCounter() % 64 == 0)
        {
            List<GroundColumn> gcNew = new List<GroundColumn>();
            foreach (GroundColumn gc in GroundColumns)
                if (gc != null)
                    if (gc.group != null)
                        switch (gc.pathParams.State)
                        {
                            case RecalcPathState.SUCCESS:
                                gc.group.SetWay(gc.pathParams.Path);
                                if (odenDebug) GamePlay.gpLogServer(null, "RecalcPathState {0}", new object[] { gc.name });
                                break;
                            case RecalcPathState.WAIT:
                                gcNew.Add(gc);
                                break;
                            case RecalcPathState.FAILED:
                                if (odenDebug) GamePlay.gpLogServer(null, "RecalcPathState Failed {0}", new object[] { gc.name });
                                foreach (int i in GamePlay.gpArmies())
                                    if (GamePlay.gpGroundGroups(i) != null)
                                        foreach (AiGroundGroup gg in GamePlay.gpGroundGroups(i))
                                            if (gg.Name().Contains(gc.name))
                                            {
                                                if (odenDebug) GamePlay.gpLogServer(null, "RecalcPathState Failed {0}", new object[] { gg.Name() });
                                                foreach (AiActor a in gg.GetItems())
                                                    if (a is AiGroundActor)
                                                        (a as AiGroundActor).Destroy();
                                            };
                                break;
                        };

            GroundColumns.Clear();
            foreach (GroundColumn gc in gcNew)
                GroundColumns.Add(gc);
        };
    }

    public override void OnTrigger(int missionNumber, string shortName, bool active)
    {
        base.OnTrigger(missionNumber, shortName, active);

        if (odenDebug) GamePlay.gpLogServer(null, "Trigger for {0} ({1})", new object[] { shortName, active.ToString() });

        if (active)
        {
            //AiTrigger trigger = GamePlay.gpGetTrigger(shortName);
            Point2d pt2d;
            switch (shortName)
            {
                case "DeSpawnLW":
                    pt2d = new Point2d(360000, 270000);
                    List<AiActor> actorsLW = FindAircrafts(LW);
                    foreach (AiActor a in actorsLW)
                    {
                        Point2d ac2d = new Point2d(a.Pos().x, a.Pos().y);
                        if (odenDebug) GamePlay.gpLogServer(null, "DeSpawnLW for {0} ({1})", new object[] { a.Name(), ac2d.distance(ref pt2d) });
                        if (a is AiAircraft)
                            if (ac2d.distance(ref pt2d) < 1609)
                                (a as AiAircraft).Destroy();
                    };
                    break;
                case "DeSpawnRAF":
                    pt2d = new Point2d(180000, 310000);
                    List<AiActor> actorsRAF = FindAircrafts(RAF);
                    foreach (AiActor a in actorsRAF)
                    {
                        Point2d ac2d = new Point2d(a.Pos().x, a.Pos().y);
                        if (a is AiAircraft)
                            if (ac2d.distance(ref pt2d) < 1609)
                                (a as AiAircraft).Destroy();
                    };
                    break;
                case fmDungeness:
                case fmDover:
                case fmRamsgate:
                case fmCanterbury:
                case fmRochester:
                case fmLondon:
                    foreach (MissionMarker mm in MissionMarkers)
                        if (mm.key == shortName)
                            mm.army = ATK;
                    gameWon = ATK;
                    sayMissionOver(gameWon);
                    GamePlay.gpLogServer(null, "UpdateRTB from {0}", new object[] { "OnTrigger fmLondon" });
                    UpdateRTB(bombersAtTarget);
                    TargetGroup = null;
                    break;
            };
        }
        else
        {   // triggers going false for logic
            if (shortName.Equals("bombersAtTarget"))
                bombersAtTarget = true;
        };
    }

    public override void OnPlaceEnter(Player player, AiActor actor, int placeIndex)
    {
        base.OnPlaceEnter(player, actor, placeIndex);

        if (player.Place() is AiAircraft)
            if ((player.Place() as AiAircraft).IsAirborne())
                playerWasAirborne = true;
        if (odenDebug) GamePlay.gpLogServer(null, "OnPlaceEnter: {0} actor: {1} {2}", new object[] { player, actor.Name(), playerWasAirborne });

        PlayersData.Add(new PlayerData { player = player, homeplate = FindAirport(player.Army(), player.Place().Pos(), 0) });
        SetOrderMissionMenuOden(player);

        if (player.Army() == LW)
            Timeout(8, () =>
            {
                GamePlay.gpHUDLogCenter(new Player[] { player }, targetName);
            });
    }

    private void GroundControlCheckIn(AiAircraft ac)
    {
        FighterCommand = ac;
        Timeout(60.0, () =>
        {
            Player localPlayer = GamePlay.gpPlayer();
            AiAircraft localAircraft = (AiAircraft)localPlayer.Place();
            AiAirGroup localGroup = localAircraft.AirGroup();
            ac.SayToGroup(localGroup, "Ground_control_here_Stand_by");
        });
    }

    public override void OnActorCreated(int missionNumber, string shortName, AiActor actor)
    {
        base.OnActorCreated(missionNumber, shortName, actor);

        if (actor != null)
        {
            if (actor is AiAircraft)
            {
                if (shortName.Contains("gb01.") && missionNumber == 1)
                {
                    if (GamePlay.gpPlayer().Army() == RAF) GroundControlCheckIn(actor as AiAircraft);
                };
                if (shortName.Contains("g06.") && missionNumber == 1)
                {
                    if (GamePlay.gpPlayer().Army() == LW) GroundControlCheckIn(actor as AiAircraft);
                };
            };
            if (actor is AiAirGroup)
            {
                //if(odenDebug) GamePlay.gpLogServer(null, "OnActorCreated AiAirGroup: {0} actor: {1}", new object[] { shortName, actor.Name() });
                AiAirGroup airGroup = (AiAirGroup)actor;
                AirGroupNames.Add(shortName);
                AirGroupIDs.Add(airGroup.ID());
                if (airGroup.Army() != ATK) AiAirGroupsStdby.Add(airGroup);
            };
            if (actor is AiGroundGroup)
            {
                foreach (GroundColumn gc in GroundColumns)
                {
                    if (actor.Name().Contains(gc.name))
                    {
                        gc.group = (AiGroundGroup)actor;
                        break;
                    };
                    if (actor.Name().Contains(gc.name))
                    {
                        gc.group = (AiGroundGroup)actor;
                        break;
                    };
                };
            };
        };
    }

    public override void OnActorDead(int missionNumber, string shortName, AiActor actor, List<DamagerScore> initiatorList)
    {
        base.OnActorDead(missionNumber, shortName, actor, initiatorList);

        if (actor == null)
            return;
        if (Time.tickCounter() < 900) // spawn error within first 30 secs (30 ticks a second)
            return;
        /*
        if (odenDebug)
            if (actor.Group() != null)
                GamePlay.gpLogServer(null, "OnActorDead shortName: {0} actor: {1} hit by:{2}", new object[] { shortName, actor.Name(), initiatorList.Count });
        */
        /*
        DamagerScore ds = initiatorList[initiatorList.Count -1];
        if (ds == null)
            return;
        if (ds.initiator == null)
            return;
        if (ds.initiator.Actor == null)
            return;
        if (ds.initiator.Actor.Army() == actor.Army())
            return;
        if(odenDebug)
            if (actor.Group() != null)
                GamePlay.gpLogServer(null, "OnActorDead shortName: {0} by: {1}", new object[] { shortName, ds.initiator.Actor.Name() });
        */

        if (actor is AiGroundActor)
        {
            GndGrpDead.Add(actor.Name());
            AiGroundActor gndActor = (AiGroundActor)actor;
            if (gndActor.Type().ToString().Equals("ShipTransport"))
            {
                misConvoyKills += 1;
                highOctaneDate = highOctaneDate.AddDays(1);
            }
            else
            {
                if (actor.Army() == ATK) destroyedUnits += 1;
            };

        };
        if (actor is AiAircraft)
        {
            AiAircraft ac = (AiAircraft)actor;
            AiAirGroup ag = ac.AirGroup();
            //if (odenDebug) GamePlay.gpLogServer(null, "AiAircraft: '{0}' '{1}' '{2}'", new object[] { ac.InternalTypeName(), ac.Regiment().name(), ag.Name() });
            if (ag.Army() == ATK)
                bombersAtTarget = true; // might be downed before reaching area, this is another good-to-go signal for OnBattleStoped()

            AircraftUpdateDead(ac);

            if (GamePlay.gpPlayer() == null)
                return;
            if (GamePlay.gpPlayer().Place() == null)
                return;
            if ((AiGroup)ag == GamePlay.gpPlayer().Place().Group())
            {
                AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
                if (pac != null)
                {
                    AiActor leader = pac.AirGroup().GetItems()[0];
                    int entry = rnd.Next(0, 17);
                    if (actor == leader)
                        entry = 18 + rnd.Next(0, 2);

                    switch (entry)
                    {
                        case 0: pac.SayToGroup(pac.AirGroup(), "Watch_out1"); break;
                        case 1: pac.SayToGroup(pac.AirGroup(), "Watch_out2"); break;
                        case 2: pac.SayToGroup(pac.AirGroup(), "Watch_out3"); break;
                        case 3: pac.SayToGroup(pac.AirGroup(), "Watch_out4"); break;
                        case 4: pac.SayToGroup(pac.AirGroup(), "We_are_under_attack_Repeat_we_are_under_attack_We_have_lost_aircraft"); requestAssistance(0, false); break;
                        case 5: pac.SayToGroup(pac.AirGroup(), "We_are_under_attack_requesting_immediate_assistance"); requestAssistance(0, false); break;
                        case 6: pac.SayToGroup(pac.AirGroup(), "We_are_under_attack_we_need_assistance_over"); requestAssistance(0, false); break;
                        case 7: pac.SayToGroup(pac.AirGroup(), "You_Come_on_men_we_can_do_better_than_that"); break;
                        case 8: pac.SayToGroup(pac.AirGroup(), "Hell"); break;
                        case 9: pac.SayToGroup(pac.AirGroup(), "Hell1"); break;
                        case 10: pac.SayToGroup(pac.AirGroup(), "Hell2"); break;
                        case 11: pac.SayToGroup(pac.AirGroup(), "Hell3"); break;
                        case 12: pac.SayToGroup(pac.AirGroup(), "Hell4"); break;
                        case 13: pac.SayToGroup(pac.AirGroup(), "Damn"); break;
                        case 14: pac.SayToGroup(pac.AirGroup(), "Damn1"); break;
                        case 15: pac.SayToGroup(pac.AirGroup(), "Bail_out_for_Gods_sakes"); break;
                        case 16: pac.SayToGroup(pac.AirGroup(), "Bail_out_Bail_out"); break;
                        case 17: pac.SayToGroup(pac.AirGroup(), "Bail_Out"); break;

                        case 18: pac.SayToGroup(pac.AirGroup(), "Blast_it_Is_that_the_squadron_leader_going_down"); break;
                        case 19: pac.SayToGroup(pac.AirGroup(), "Bloody_hell_we_ve_lost_the_squadron_leader"); break;
                        case 20: pac.SayToGroup(pac.AirGroup(), "The_squadron_leader_s_done_for_What_are_we_going_to_do"); break;
                    };
                };
            };
        };
    }

    public override void OnStationaryKilled(int missionNumber, GroundStationary gs, AiDamageInitiator initiator, int eventArgInt)
    {
        base.OnStationaryKilled(missionNumber, gs, initiator, eventArgInt);

        //if (odenDebug) GamePlay.gpLogServer(null, "OnStationaryKilled: {0} {1} {2} {3} {4}", new object[] { gs.Category, gs.country, gs.Name, gs.Title, gs.Type.ToString() });
        GndStatDead.Add(gs.Name);
    }

    public override void OnBombExplosion(string title, double mass, Point3d pos3d, AiDamageInitiator initiator, int eventArgInt)
    {
        base.OnBombExplosion(title, mass, pos3d, initiator, eventArgInt);

        //if (GamePlay.gpLandType(pos3d.x, pos3d.y).ToString().Equals("WATER")) return;
        //GamePlay.gpHUDLogCenter("Boom said " + title + "   =" + mass.ToString() + "kg, by:" + initiator.Actor.Army().ToString());
        if (targetAirport != null)
            if (targetAirport.Pos().distance(ref pos3d) < targetAirport.CoverageR())
                bombsOnTarget += mass;
        if (mainGSTarget != null)
            if (mainGSTarget.pos.distance(ref pos3d) < 1000) // 1km target radius
                bombsOnTarget += mass;
    }
    /*
    public override void OnBuildingKilled(string title, maddox.GP.Point3d pos, maddox.game.world.AiDamageInitiator initiator, int eventArgInt)
    {
        base.OnBuildingKilled(title, pos, initiator, eventArgInt);
        if (odenDebug) GamePlay.gpLogServer(null, "OnBuildingKilled: '{0}' '{1}' '{2}' '{3}'", new object[] { title, pos, initiator, eventArgInt });
    }
    */
    public override void OnAircraftTookOff(int missionNumber, string shortName, AiAircraft aircraft)
    {
        if (aircraft.Player(0) != null)
        {
            playerWasAirborne = true;
        }
        else
        {
            AiAirGroup ag = aircraft.AirGroup();
            AiActor a = ag.GetItems()[0];
            if (a == (aircraft as AiActor) && a.Army() != ATK)
                if (target2d.x > 0 && TargetGroup == null)
                    Timeout(15, () => { GiveNewWP(ag, HalfwayToTarget2d(ag)); });
            if (a == (aircraft as AiActor) && a.Army() == ATK)
                if (target2d.x > 0 && EscortGroup != null)
                    Timeout(15, () => { GiveNewWP(ag, HalfwayToTarget2d(ag)); });
        };
    }

    public override void OnAircraftLanded(int missionNumber, string shortName, AiAircraft aircraft)
    {
        base.OnAircraftLanded(missionNumber, shortName, aircraft);

        if (aircraft == null)
            return;
        AiAirGroup ag = aircraft.AirGroup();

        Point2d lnd2d = new Point2d(aircraft.Pos().x, aircraft.Pos().y);
        AiAirport ap = FindAirport(ag.Army(), lnd2d, 0);
        if (ap != null)
        {
            Point3d pt3d = aircraft.Pos();
            if (ap.Pos().distance(ref pt3d) < ap.CoverageR())
            {
                if (ag.Army() == GamePlay.gpFrontArmy(lnd2d.x, lnd2d.y))
                {
                    if (aircraft.Player(0) != null)
                        if (Satellites.Contains(ap.Name()))
                            playerLandedAtAirport = ap;
                }
                else AircraftUpdateDead(aircraft);
            }
            else AircraftUpdateDead(aircraft);
        }
        else AircraftUpdateDead(aircraft);

        Timeout(150, () => { if (aircraft != null) aircraft.Destroy(); });
    }

    public virtual void OnAircraftCrashLanded(int missionNumber, string shortName, AiAircraft aircraft)
    {
        AircraftUpdateDead(aircraft);
    }

    private void StartMission()
    {
        timeoutRadio = DateTime.Now;
        timeoutRadio.AddSeconds(120);
        if (odenDebug) GamePlay.gpLogServer(null, "StartMission: {0}", new object[] { "" });


        AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
        if (pac == null)
        {
            if (File.Exists(CampInitFile))
                File.Delete(CampInitFile);
            CheckFiles();
            return; // let user reset all setup
        };


        string strSay = "You_re_cleared_for_take_off";
        switch (rnd.Next(1, 5))
        {
            case 1: strSay = "Takeoff";
                break;
            case 2: strSay = "Wingmen_make_sure_you_stay_with_your_leaders";
                break;
            case 3: strSay = "Stay_sharp_gentlemen_And_make_sure_to_stay_in_formation";
                if (GamePlay.gpPlayer().Army() == LW) strSay = "Use_your_rudder_to_counter_the_yaw";
                break;
            case 4: strSay = "Prepare_For_Takeoff";
                break;
        };

        AiAirGroup pag = pac.AirGroup();
        AiActor leader = pag.GetItems()[0];
        AiAircraft lac = (AiAircraft)leader;
        Timeout(rnd.Next(4, 12), () => { lac.SayToGroup(pag, strSay); });
        Point3d pt3d = pac.Pos();
        oldWP = new AiWayPoint(ref pt3d, 0);
        //if (odenDebug) GamePlay.gpLogServer(null, "TypedName: '{0}'", new object[] { pac.TypedName() }); == 'KL o D'

        Point2d pt2d = new Point2d(pt3d.x, pt3d.y);
        foreach (MissionAirbase mab in MissionAirbases)
            if (pag.Name().Contains(mab.sqn))
                homeBase = mab;

        if (homeBase == null)
        {
            GamePlay.gpHUDLogCenter("Mission Error: " + pag.Name() + " not found at any base, check DynMis40e.ini");
            return;
        };
        homePlate = homeBase.ap;
        homePlate3d = homePlate.Pos();
        activatedGroups.Add(homeBase.sqn);
        if (homeBase.country == "de") missionCaption = "Luftwaffe " + homeBase.name + " (" + vName + ")";

        // update target list
        foreach (GroundStationary gs in GamePlay.gpGroundStationarys())
            if (gs.Name != "NONAME")
            {
                if (GndStatDead.Contains(gs.Name))
                {
                    Point2d gs2d = new Point2d(gs.pos.x, gs.pos.y);
                    gs.Destroy();
                }
                else
                {
                    if (gs.Title.Contains("EnglishRadar3"))
                        ChainHomeRadars.Add(gs);
                    if (gs.Title.Contains("PortCrane1_UK1"))
                        PortCranes.Add(gs);
                    if (gs.Title.Contains("Oxygen_Facility"))
                        Industries.Add(gs);
                };
            };

        //foreach (int i in GamePlay.gpArmies())
        //if (GamePlay.gpGroundGroups(i) != null)
        foreach (AiGroundGroup gg in GamePlay.gpGroundGroups(RAF))
        {
            //if (odenDebug) GamePlay.gpLogServer(null, "gg.Name(): '{0}'", new object[] { gg.Name() });
            if (GndGrpDead.Contains(gg.Name()))
                foreach (AiActor a in gg.GetItems()) (a as AiGroundActor).Destroy();
            else
                if ((gg.GetItems()[0] as AiCart).InternalTypeName().Contains("Tanker_Medium1"))
                    ConvoyShips.Add(gg);
        };
        if (odenDebug) GamePlay.gpLogServer(null, "Targets: '{0}' '{1}' '{2}' '{3}'", new object[] { ConvoyShips.Count, ChainHomeRadars.Count, PortCranes.Count, Industries.Count });
        /*
        if(odenDebug)
            foreach (AiAirport ap in GamePlay.gpAirports())
                GamePlay.gpLogServer(null, "{0} is {1} x {2}", new object[] { ap.Name(), ap.Pos().x, ap.Pos().y });
        */

        // select mission
        GameOver = BattleOfBritainVictoryCheck();
        //if(odenDebug) GameOver = LW;
        switch (GameOver)
        {
            case 0:
                if (gameDateTime > dtFighterBomber)
                {   // Fighter Bomber Attacks along coastal assets (Sea Lion cancelled) during Oct
                    if (ConvoyShips.Count > 0 && highOctaneDate < gameDateTime.AddDays(rnd.Next(0, 2)) && rnd.Next(0, 9) > 5 && !gameIsOn)
                        CreateConvoyMission();
                    if (PortCranes.Count > 0 && rnd.Next(0, 9) > 6 && !gameIsOn)
                        CreatePortMission();
                    if (ChainHomeRadars.Count > 0 && rnd.Next(0, 9) > 5 && !gameIsOn)
                        CreateCHRadarMission();
                    if (!gameIsOn)
                        CreateAirbaseMission(LW, true);
                };
                if (gameDateTime > dtBlitz)
                {   // Daylight Attack on London, 7 - 30 Sept
                    if (Industries.Count > 0 && ReinforcementDate < gameDateTime.AddDays(rnd.Next(0, 5)) && !gameIsOn)
                        CreateIndustryMission();
                    if (ConvoyShips.Count > 0 && highOctaneDate < gameDateTime.AddDays(rnd.Next(0, 2)) && rnd.Next(0, 9) > 5 && !gameIsOn)
                        CreateConvoyMission();
                    if (PortCranes.Count > 0 && rnd.Next(0, 9) > 6 && !gameIsOn)
                        CreatePortMission();
                };
                if (gameDateTime > dtAdlerTag)
                {   // Assault on Fighter Command (AdlerTag), 24 Aug - 6 Sept
                    if (rnd.Next(0, 9) > 4 && !gameIsOn)
                        CreateAirbaseMission(LW, false);
                    if (ChainHomeRadars.Count > 0 && rnd.Next(0, 9) > 5 && !gameIsOn)
                        CreateCHRadarMission();
                    if (ConvoyShips.Count > 0 && highOctaneDate < gameDateTime.AddDays(rnd.Next(0, 2)) && rnd.Next(0, 9) > 5 && !gameIsOn)
                        CreateConvoyMission();
                    if (PortCranes.Count > 0 && rnd.Next(0, 9) > 6 && !gameIsOn)
                        CreatePortMission();
                };
                if (gameDateTime > dtCoastalBattles)
                {   // Coastal Battles (Kanalkampf), 8 - 23 August
                    if (PortCranes.Count > 0 && rnd.Next(0, 9) > 5 && !gameIsOn)
                        CreatePortMission();
                    if (ChainHomeRadars.Count > 0 && rnd.Next(0, 9) > 5 && !gameIsOn)
                        CreateCHRadarMission();
                    if (ConvoyShips.Count > 0 && highOctaneDate < gameDateTime.AddDays(rnd.Next(0, 2)) && rnd.Next(0, 9) > 5 && !gameIsOn)
                        CreateConvoyMission();
                    if (!gameIsOn)
                        CreateAirbaseMission(LW, true);
                };
                // Convoy Battles (Kanalkampf), 10 July - 7 August
                if (ConvoyShips.Count > 0 && highOctaneDate < gameDateTime.AddDays(rnd.Next(0, 2)) && rnd.Next(0, 9) > 5 && !gameIsOn)
                    CreateConvoyMission();
                if (PortCranes.Count > 0 && rnd.Next(0, 9) > 6 && !gameIsOn)
                    CreatePortMission();
                if (!gameIsOn)// only LW for now. ToDo: Blenheim dayraids
                    CreateAirbaseMission(LW, true);
                break;
            case RAF:
                /* 1941 style
    A. "Sweep" - search and destroy mission, strictly for fighters and operationally called "Rodeo".
    B. "Circus" - escort to bombers, which usually were small in numbers with very strong fighters presence and directed in the regions most saturated with enemy's fighters to goad them into fighting. Waves of fighters acted according to their position in regard to the bombers they escorted. They were divided into 'close cover', 'escort cover', 'high cover' and 'top cover', one flying slightly behind and above the other. Often, numerous squadrons flew so-called 'forward or rear support', which operated freely but still being part of the operation. The 308 flew 49 such sorties in 1941.
    C. "Ramrod" - escorting bombers raiding enemy's vital targets. Fighters' role was firstly to fight off enemy's attacks.
    (These three operations were often mounted all at the same time in order to engage as many enemy fighters as possible having maximum tactical advantage. Germans tried their own tactics, often successfully, causing confusion among RAF forces and their higher losses. Many Ramrods ended up as Circus and vice versa.)
    D. "Roadsted" - sorties directed against enemy's shipping and coastal traffic. Fighters provided escort and if possible, attacked ships and flak positions.
    E. "Rhubarb" - sweeping, 'on the deck' action against ground targets, mostly aerodromes and radio or radar stations.
                */
                // WIP
                CreateAirbaseMission(RAF, true);
                //CreateRodeoMission(RAF, true);
                //CreateCircusMission(RAF, true);
                //CreateRamrodMission(RAF, true);
                //CreateRoadstedMission(RAF, true);
                //CreateRhubarbMission(RAF, true);
                break;
            case LW: // Sea Lion style
                // WIP
                // CreateLandingFolkstone();
                // CreateLandingDover();
                // CreateLandingDeal();
                // CreateLandingRamsgate();
                // Ground units in Kent moving towards London
                // Ground Battle for London
                List<int> idxLandingAreas = new List<int>();
                int idxSeaLion = 0;
                if (MissionMarkers[idxDungeness].army == RAF) idxLandingAreas.Add(idxDungeness);
                else idxSeaLion = idxDungeness;
                if (MissionMarkers[idxDover].army == RAF) idxLandingAreas.Add(idxDover);
                else idxSeaLion = idxDover;
                if (MissionMarkers[idxRamsgate].army == RAF) idxLandingAreas.Add(idxRamsgate);
                else idxSeaLion = idxRamsgate;
                if (idxLandingAreas.Count == 3) CreateLanding(idxLandingAreas[rnd.Next(0, 2)]);
                else
                {   // one landing has been successful
                    if (MissionMarkers[idxCanterbury].army == RAF) CreateColumns(idxSeaLion, idxCanterbury);
                    else
                    {
                        if (MissionMarkers[idxRochester].army == RAF) CreateColumns(idxCanterbury, idxRochester);
                        else
                        {
                            if (MissionMarkers[idxLondon].army == RAF) CreateColumns(idxRochester, idxLondon);
                            else
                            {
                                if (File.Exists(CampInitFile))
                                    File.Delete(CampInitFile);
                                CheckFiles();
                                return; // let user restart mission from main menu
                            };
                        };
                    };
                };

                /*
                    if (ConvoyShips.Count > 0 && highOctaneDate < gameDateTime.AddDays(rnd.Next(0, 2)) && rnd.Next(0, 9) > 5 && !gameIsOn)
                        CreateConvoyMission();
                    if (Industries.Count > 0 && ReinforcementDate < gameDateTime.AddDays(rnd.Next(0, 5)) && !gameIsOn)
                        CreateIndustryMission();
                    if (PortCranes.Count > 0 && rnd.Next(0, 9) > 6 && !gameIsOn)
                        CreatePortMission();
                    if (ChainHomeRadars.Count > 0 && rnd.Next(0, 9) > 5 && !gameIsOn)
                        CreateCHRadarMission();
                    if (!gameIsOn)
                        CreateAirbaseMission(LW, true);
                */
                break;
        };

        ISectionFile fcap = GamePlay.gpCreateSectionFile();
        bool capexists = false;
        int cap = rnd.Next(1, 4);
        List<MissionAirbase> CAPbases = new List<MissionAirbase>();
        CAPbases = SortedAirbases(ATK == LW ? RAF : LW, target2d, false, true);
        foreach (MissionAirbase ab in CAPbases)
        {
            if (cap < 1) break;
            CreateCAPMission(fcap, new Point2d(ab.x, ab.y), 6, ab.army, false);
            capexists = true;
            cap--;
        };
        if (capexists)
        {
            GamePlay.gpPostMissionLoad(fcap);
            NextMissionNum++;
        };

    }

    private void AircraftUpdateDead(AiAircraft ac)
    {
        if (ac == null)
            return;

        AiAirGroup ag = ac.AirGroup();
        foreach (MissionAirbase ab in MissionAirbases)
            if (ab.sqn == ac.Regiment().name())
            {
                Point3d pt3d = ac.Pos();
                // ignore AI lawn darting for fun
                if (ab.Pos().distance(ref pt3d) > 2000)
                {
                    ab.maxPlanes -= 1;
                    //if (odenDebug) GamePlay.gpLogServer(null, "MissionAirbase: '{0}' '{1}' '{2}'", new object[] { ab.name, ab.sqn, ab.maxPlanes });
                    if (ab.type.Contains("Fighter"))
                        if (ac.Army() == RAF) misRAFDeadF += 1;
                        else misLWDeadF += 1;
                    if (ab.type.Contains("Bomber"))
                        if (ac.Army() == RAF) misRAFDeadB += 1;
                        else misLWDeadB += 1;
                    if (ab.type.Contains("Zerstorer")) misLWDeadF += 1;
                    if (ab.type.Contains("Stuka")) misLWDeadS += 1;
                };
                break;
            };
    }

    private void ArmyHUDLogCenter(int army, string msg)
    {
        Player localPlayer = GamePlay.gpPlayer();
        List<Player> players = new List<Player>();
        if (GamePlay.gpRemotePlayers() != null)
            foreach (Player p in GamePlay.gpRemotePlayers())
            {
                if (p.Army() == army) players.Add(p);
            };

        if (localPlayer != null)
            if (localPlayer.Army() == army) players.Add(localPlayer);
        GamePlay.gpHUDLogCenter(players.ToArray(), msg);
        //if(odenDebug) GamePlay.gpLogServer(null, "SendMessage: {0} {1}", new object[] { players.ToArray(), msg });
    }
    private List<Player> AllPlayers(int army)
    {
        List<Player> players = new List<Player>();
        Player lp = GamePlay.gpPlayer();
        if (lp != null)
            if (lp.Army() == army || army == 0)
                players.Add(lp);
        if (GamePlay.gpRemotePlayers() != null)
            foreach (Player p in GamePlay.gpRemotePlayers())
                if (p.Army() == army || army == 0)
                    players.Add(p);
        return players;
    }
    private bool PlayerIsBomber(Player player)
    {
        AiActor actor = player.Place();
        if (actor == null)
            return false;
        AiAircraft pac = (actor as AiAircraft);
        if (pac == null)
            return false;
        AiAirGroup pag = pac.AirGroup();
        return (pag.hasBombs() || pag.hasTorpedos() || pag.hasBombsInInternalBays());
    }
    internal AiAirport FindAirport(int army, Point2d pt2d, int runwaylength)
    {   // requires fronline
        AiAirport result = null;
        Point3d pt3d = new Point3d(pt2d.x, pt2d.y, 0);
        string sectorName;

        foreach (AiAirport ap in GamePlay.gpAirports())
        {
            sectorName = ""; // GamePlay.gpSectorName(airport.Pos().x, airport.Pos().y);

            if (GamePlay.gpFrontArmy(ap.Pos().x, ap.Pos().y) == army && ap.FieldR() > runwaylength && (sectorName.IndexOf(">") == -1 && sectorName.IndexOf("<") == -1))
            {
                if (result != null)
                {
                    if (result.Pos().distance(ref pt3d) > ap.Pos().distance(ref pt3d))
                        //if (1000 < ap.Pos().distance(ref pt3d)) // not same airport
                        result = ap;
                }
                else
                    result = ap;
            };
        };
        return result;
    }
    internal AiAirport FindAirport(int army, Point3d pt3d, int runwaylength)
    {   // requires fronline
        AiAirport result = null;
        string sectorName;

        foreach (AiAirport ap in GamePlay.gpAirports())
        {
            sectorName = ""; // GamePlay.gpSectorName(airport.Pos().x, airport.Pos().y);

            if (GamePlay.gpFrontArmy(ap.Pos().x, ap.Pos().y) == army && ap.FieldR() > runwaylength && (sectorName.IndexOf(">") == -1 && sectorName.IndexOf("<") == -1))
            {
                if (result != null)
                {
                    if (result.Pos().distance(ref pt3d) > ap.Pos().distance(ref pt3d))
                        //if (1000 < ap.Pos().distance(ref pt3d)) // not same airport
                        result = ap;
                }
                else
                    result = ap;
            };
        };
        return result;
    }

    internal List<AiAirport> SortedAirports(int army, Point2d pt2d, bool onlyBattleArea)
    {   // requires fronline
        List<AiAirport> SortedAirports = new List<AiAirport>();
        Point3d pt3d = new Point3d(pt2d.x, pt2d.y, 0);

        foreach (AiAirport ap in GamePlay.gpAirports())
        {
            string sectorName = "";
            if (onlyBattleArea)
                sectorName = GamePlay.gpSectorName(ap.Pos().x, ap.Pos().y);
            foreach (string apdmg in AirportsDamaged)
                if (apdmg.Contains(ap.Name()))
                    sectorName = "<>";
            //GamePlay.gpLogServer(null, "SortedAirports: {0} {1} ?= {2} at {3}", new object[] { ap.Name(), GamePlay.gpFrontArmy(ap.Pos().x, ap.Pos().y), army, sectorName });
            if (GamePlay.gpFrontArmy(ap.Pos().x, ap.Pos().y) == army && (sectorName.IndexOf(">") == -1 && sectorName.IndexOf("<") == -1))
            {
                int forEachIndex = 0;
                foreach (AiAirport sap in SortedAirports)
                {
                    if (sap.Pos().distance(ref pt3d) > ap.Pos().distance(ref pt3d))
                    {
                        SortedAirports.Insert(forEachIndex, ap);
                        break;
                    };
                    forEachIndex++;
                };
                if (!SortedAirports.Contains(ap)) SortedAirports.Add(ap); // add last
            };
        };
        return SortedAirports;
    }

    internal MissionAirbase FindAirbase(int army, Point2d pt2d, bool bomber, bool onlyBattleArea)
    {
        MissionAirbase result = null;
        Point3d pt3d = new Point3d(pt2d.x, pt2d.y, 0);
        string sectorName = "";
        double runwaylength = 90;
        if (bomber)
            runwaylength = 600;

        foreach (MissionAirbase ab in MissionAirbases)
        {
            if (onlyBattleArea)
                sectorName = GamePlay.gpSectorName(ab.x, ab.y);
            double fieldR = 0;
            if (ab.ap == null) fieldR = 9999; // airstart
            else fieldR = ab.ap.FieldR();
            bool fighterBase = false;
            if (ab.type.Contains("Fighter"))
                fighterBase = true;
            if (ab.type.Contains("Zerstorer"))
                fighterBase = !bomber;
            if (ab.type.Contains("Stab"))
                fighterBase = ab.sqn.Contains("JG"); // Stab./JG is only "Luftwaffe_Stabstaffel_1940", no type indicator

            if (bomber != fighterBase && ab.maxPlanes > 2 && ab.army == army && fieldR > runwaylength && (sectorName.IndexOf(">") == -1 && sectorName.IndexOf("<") == -1))
            {
                if (result != null)
                {
                    if (result.Pos().distance(ref pt3d) > ab.Pos().distance(ref pt3d))
                        //if (1000 < ab.Pos().distance(ref pt3d)) // not same airport
                        result = ab;
                }
                else
                    result = ab;
            };
        };
        return result;
    }

    internal List<MissionAirbase> SortedAirbases(int army, Point2d pt2d, bool bomber, bool onlyBattleArea)
    {
        List<MissionAirbase> SortedBases = new List<MissionAirbase>();
        Point3d pt3d = new Point3d(pt2d.x, pt2d.y, 0);
        string sectorName = "";
        double runwaylength = 90;
        if (bomber)
            runwaylength = 900;
        bool preferFB = false;
        if (gameDateTime > dtFighterBomber)
            preferFB = true;

        foreach (MissionAirbase ab in MissionAirbases)
        {
            if (onlyBattleArea)
                sectorName = GamePlay.gpSectorName(ab.x, ab.y);
            double fieldR = 0;
            if (ab.ap == null) fieldR = 9999; // airstart
            else fieldR = ab.ap.FieldR();
            bool fighterBase = false;
            if (ab.type.Contains("Fighter"))
                fighterBase = true;
            if (ab.type.Contains("Zerstorer"))
                if (bomber && ab.aircraftType == "Aircraft.Bf-110C-7") fighterBase = false;
                else fighterBase = true;
            if (ab.type.Contains("Stab"))
                fighterBase = ab.sqn.Contains("JG"); // Stab./JG is only "Luftwaffe_Stabstaffel_1940", no type indicator
            if (preferFB)
            {
                fighterBase = bomber; // ignore medium bombers
                if (bomber && ab.aircraftType == "Aircraft.Bf-110C-7") fighterBase = false;
                if (bomber && ab.aircraftType == "Aircraft.Bf-109E-1B") fighterBase = false;
                if (bomber && ab.aircraftType == "Aircraft.Bf-109E-3B") fighterBase = false;
                if (bomber && ab.aircraftType == "Aircraft.Bf-109E-4B") fighterBase = false;
            };
            foreach (string apdmg in AirportsDamaged)
                if (apdmg.Contains(ab.airportName))
                    fieldR = 0;

            if (bomber != fighterBase && ab.maxPlanes > 2 && !activatedGroups.Contains(ab.sqn) && ab.army == army && !ab.type.Contains("Recon") && fieldR > runwaylength && (sectorName.IndexOf(">") == -1 && sectorName.IndexOf("<") == -1))
            {
                int forEachIndex = 0;
                foreach (MissionAirbase sab in SortedBases)
                {
                    if (sab.Pos().distance(ref pt3d) > ab.Pos().distance(ref pt3d))
                    {
                        SortedBases.Insert(forEachIndex, ab);
                        break;
                    };
                    forEachIndex++;
                };
                if (!SortedBases.Contains(ab)) SortedBases.Add(ab); // add last
            };
        };
        return SortedBases;
    }

    internal GroundStationary FindChainHomeRadar(Point2d pt2d)
    {
        GroundStationary result = null;
        Point3d pt3d = new Point3d(pt2d.x, pt2d.y, 0);
        foreach (GroundStationary gs in ChainHomeRadars)
        {
            if (result != null)
            {
                if (result.pos.distance(ref pt3d) > gs.pos.distance(ref pt3d))
                    result = gs;
            }
            else
                result = gs;
        };
        return result;
    }

    internal List<AiAircraft> SortedAircrafts(int army, Point3d pt3d)
    {
        List<AiAircraft> sorted = new List<AiAircraft>();
        AiAirGroup[] airGroups = GamePlay.gpAirGroups(army);

        if (airGroups != null)
            foreach (AiAirGroup airGroup in airGroups)
                if (airGroup != null)
                    if (airGroup.GetItems().Length > 0)
                        foreach (AiActor actor in airGroup.GetItems())
                            if (actor != null)
                                if (actor is AiAircraft)
                                {
                                    AiAircraft ac = (AiAircraft)actor;
                                    int forEachIndex = 0;
                                    foreach (AiAircraft sac in sorted)
                                    {
                                        if (sac.Pos().distance(ref pt3d) > ac.Pos().distance(ref pt3d))
                                        {
                                            sorted.Insert(forEachIndex, ac);
                                            break;
                                        };
                                        forEachIndex++;
                                    };
                                    if (!sorted.Contains(ac)) sorted.Add(ac); // add last
                                };

        return sorted;
    }

    internal List<AiActor> FindAircrafts(int army)
    {
        List<AiActor> AllActors = new List<AiActor>();
        foreach (AiAirGroup airGroup in GamePlay.gpAirGroups(army))
        {
            foreach (AiActor actor in ((airGroup as AiGroup).GetItems()))
                AllActors.Add(actor);
        };
        return AllActors;
    }

    internal List<Point2d> SortByDistance(List<Point2d> ListPt2d, Point2d RefPt2d)
    {
        List<Point2d> SortedPt2d = new List<Point2d>();

        foreach (Point2d pt2d in ListPt2d)
        {
            if (SortedPt2d.Count == 0) SortedPt2d.Add(pt2d);
            else
            {
                int forEachIndex = 0;
                foreach (Point2d spt2d in SortedPt2d)
                {
                    if (spt2d.distance(ref RefPt2d) > pt2d.distance(ref RefPt2d))
                    {
                        SortedPt2d.Insert(forEachIndex, pt2d);
                        break;
                    };
                    forEachIndex++;
                };
            };
            if (!SortedPt2d.Contains(pt2d)) SortedPt2d.Add(pt2d);
        };

        return SortedPt2d;
    }

    private int BattleOfBritainVictoryCheck()
    {
        if (GameOver > 0) return GameOver;

        int gbTotal = 0, deTotal = 0;
        foreach (MissionAirbase mab in MissionAirbases)
            if (mab.army == RAF) gbTotal += mab.maxPlanes;
            else deTotal += mab.maxPlanes;
        if (gbTotal > deTotal)
            return RAF;
        if (gameDateTime.Month > 8 && deTotal > gbTotal * 3)
            return LW;
        if (gameDateTime.Month > 10)
            return RAF;
        return 0;
    }

    private void AddToFile(ISectionFile f, string sect, string key, string value)
    {
        if (odenDebug) GamePlay.gpLogServer(null, "AddToFile {0} {1} {2}", new object[] { sect, key, value });
        f.add(sect, key, value);
    }

    private string CreateCAPMission(ISectionFile f, Point2d CAP2d, int size, int army, bool isEscort)
    {
        MissionAirbase abCAP = null;
        List<MissionAirbase> MAList = SortedAirbases(army, CAP2d, false, false);
        if (odenDebug) GamePlay.gpLogServer(null, "CreateCAPMission #{0} x{1} army:{2} ({3} sq avail)", new object[] { CAPFlightIndex, size, army, MAList.Count });
        if (MAList == null)
            return "";
        if (MAList.Count == 0)
            return "";
        if (MAList.Count > 12)
            abCAP = MAList[rnd.Next(0, 11)];
        else
            if (MAList.Count > 3) // random 1 of 4 first/closest
                abCAP = MAList[rnd.Next(0, 3)];
            else abCAP = MAList[0];
        if (abCAP == null)
            return "";

        AiAirport ap = abCAP.ap;
        activatedGroups.Add(abCAP.sqn);

        if (size > abCAP.maxPlanes) size = abCAP.maxPlanes;
        if (odenDebug) GamePlay.gpLogServer(null, "CreateCAPMission {0} {1} max:{2} of {3}", new object[] { abCAP.name, abCAP.sqn, abCAP.maxPlanes, abCAP.aircraftType });

        AiActor[] actorsQueue = null;
        string apName = "AIR";
        if (ap != null)
        {
            actorsQueue = ap.QueueTakeoff();
            apName = ap.Name();
        };
        bool setOnPark = false;
        if (actorsQueue != null || CAPFlightIndex > 0 || activatedAirports.Contains(apName))
            setOnPark = true;
        if (ap != null)
            activatedAirports.Add(apName);

        string value;
        Point2d pt2d = new Point2d(abCAP.x, abCAP.y);

        // each MAB should represent a RAF Sqn (12) or LW Gruppe (36)
        // RAF Airfields usually house 2-4 squadrons while a LW airfield usually house a Gruppe's 2-3 Staffeln
        // "II/JG52" 2 276985 209109 0 36 1 1 de"Luftwaffe_Fighter_1940 BoB_LW_JG52_II
        // "I/LG2" 2 291127 216041 0 36 1 1 de"Luftwaffe_Fighter_1940 BoB_LW_LG2_I
        // "66 Sqn" 1 180549 264239 0 12 1 1 gb RAF_Fighter_1940 BoB_RAF_F_66Sqn_Late

        string sqn = abCAP.sqn; // bp.Regiment().name();
        //string[] PlaneSet = abCAP.aircraftTypes; // bp.GetAircraftTypes();
        string planeType = abCAP.aircraftType; // PlaneSet[rnd.Next(0, PlaneSet.Length)];

        string form = "FINGERFOUR";
        if (sqn.Contains("Early")) form = "VIC3";
        if (sqn.Contains("RA_")) form = "VIC3";

        /*
        value = "1";
        switch (form)
        {
            case "FINGERFOUR":
                if (size == 2) value = "1 2"; // 2-ship flight
                if (size == 3) value = "1 2 3"; // 3-ship flight (single flight)
                if (size == 4) value = "1 2 3 4"; // 4-ship flight (schwarm x3 of staffel)
                if (size == 5) value = "1 2 3 4 5"; // 5-ship flight
                if (size == 6) value = "1 2 3 4 5 6"; // 6-ship flight (flight x2 of squadron)
                if (size > 6) value = "1 2 3 4"; // 3x4-ship staffel (or RAF 1941 Sqn)
                AddToFile(f, sqn, "Flight0", value);
                if (size == 7) AddToFile(f, sqn, "Flight1", "5 6 7");
                if (size > 7) AddToFile(f, sqn, "Flight1", "5 6 7 8"); // 2x4-ship staffel
                if (size == 9) AddToFile(f, sqn, "Flight2", "9");
                if (size == 10) AddToFile(f, sqn, "Flight2", "9 10");
                if (size == 11) AddToFile(f, sqn, "Flight2", "9 10 11");
                if (size > 11) AddToFile(f, sqn, "Flight2", "9 10 11 12"); // 3x4-ship staffel
                break;
            default:
                if (size < 10)
                {
                    if (size == 2) value = "1 2"; // 2-ship flight
                    if (size > 2) value = "1 2 3"; // 3x3-ship
                    AddToFile(f, sqn, "Flight0", value);
                    if (size == 4) AddToFile(f, sqn, "Flight1", "4");
                    if (size == 5) AddToFile(f, sqn, "Flight1", "4 5");
                    if (size > 5) AddToFile(f, sqn, "Flight1", "4 5 6"); // 2x3-ship
                    if (size == 7) AddToFile(f, sqn, "Flight2", "7");
                    if (size == 8) AddToFile(f, sqn, "Flight2", "7 8");
                    if (size > 9) AddToFile(f, sqn, "Flight2", "7 8 9"); // 3x3-ship
                }
                else
                {
                    AddToFile(f, sqn, "Flight0", "1 2 3 4 5 6");
                    if (size == 10) value = "7 8 9 10";
                    if (size == 11) value = "7 8 9 10 11";
                    if (size > 11) value = "7 8 9 10 11 12";
                    AddToFile(f, sqn, "Flight1", value);
                };
                break;
        };
        */
        string form2 = form;
        if (army == RAF && form == "VIC3" && size > 6) form2 = "VIC6"; // fake to put 12 in 2x6
        string[] FlightSection = FlightSections(army, size, form2, CAPStaffelIndex);
        sqn += "." + FlightSection[4];
        AddToFile(f, "AirGroups", sqn, "");
        int sectionQty = (FlightSection[0].Length + 1) / 2;
        if (FlightSection[0].Length > 0) AddToFile(f, sqn, "Flight0", FlightSection[0]);
        if (FlightSection[1].Length > 0) AddToFile(f, sqn, "Flight1", FlightSection[1]);
        if (FlightSection[2].Length > 0) AddToFile(f, sqn, "Flight2", FlightSection[2]);
        if (FlightSection[3].Length > 0) AddToFile(f, sqn, "Flight3", FlightSection[3]);

        AddToFile(f, sqn, "Class", planeType);
        AddToFile(f, sqn, "Formation", form);
        // Stuka max 8 +VIC
        // Bomber max 3 +VIC3
        // Zerstörer max 4 +FF
        // Fighter max 4 +FF
        // Recon max 1 +LINEABREAST
        // Transp. max 3 +VIC
        // UK Bomber max 10 +VIC
        // UK Fighter max 6 +VIC3
        // UK Fighter max 4 +FF

        AddToFile(f, sqn, "CallSign", ((int)(rnd.Next(0, 35))).ToString());
        AddToFile(f, sqn, "Fuel", "84");
        AddToFile(f, sqn, "Weapons", "1");
        if (setOnPark)
            AddToFile(f, sqn, "SetOnPark", "1");
        //AddToFile(f, sqn, "SpawnFromScript", "1");
        AddToFile(f, sqn, "Skill", "0.84 0.19 0.53 0.74 0.74 0.89 0.53 0.74");
        int aging = (gameDateTime - BoBDateTime).Days;
        if (aging > 99) aging = 99;
        AddToFile(f, sqn, "Aging", aging.ToString());

        if (!isEscort)
        {
            double heading = calcDir(pt2d, CAP2d);

            int alt = rnd.Next(1200, 3500);
            int speed = rnd.Next(300, 400);
            if (ap == null)
            {   // airstart if map entry point
                pt2d = calcPos(pt2d, rnd.Next(2000, 3000), heading);
                AddToFile(f, sqn + "_Way", "NORMFLY", pt2d.x.ToString() + " " + pt2d.y.ToString() + " " + alt.ToString() + " " + speed.ToString());
            }
            else
            {
                AddToFile(f, sqn + "_Way", "TAKEOFF", abCAP.x.ToString("F2") + " " + abCAP.y.ToString("F2") + " 0 0");
                double dir = calcDir(pt2d, CAP2d);
                Point2d wp2d = calcPos(pt2d, 16090 / 2, dir);
                AddToFile(f, sqn + "_Way", "HUNTING", wp2d.x.ToString("F2") + " " + wp2d.y.ToString("F2") + " 1680.00" + " 280.00");
            };
            if (abCAP.ap == null)
                if (army == RAF) CAP2d = calcPos(CAP2d, 1609 * rnd.Next(8, 20), 120 + rnd.Next(0, 120));
                else CAP2d = calcPos(CAP2d, 1609 * rnd.Next(8, 20), 210 + rnd.Next(0, 120));

            AddToFile(f, sqn + "_Way", "HUNTING", CAP2d.x.ToString("F2") + " " + CAP2d.y.ToString("F2") + " " + alt.ToString() + ".00 " + speed.ToString() + ".00");
        };

        //CAPFlightIndex++;
        //CAPStaffelIndex++;

        return sqn;
    }

    private void CreateBMBMission(ISectionFile f, Point2d BMB2d, int size, int army, AiActor aTarget, GroundStationary gs, bool escortReq)
    {
        //if (odenDebug) GamePlay.gpLogServer(null, "CreateBMBMission {0} {1} {2} {3}", new object[] { BMB2d.x, size, army, escortReq });
        MissionAirbase abBMB = null;

        if (BMBFlightAB == null)
        {
            List<MissionAirbase> MAList = SortedAirbases(army, BMB2d, true, false);
            if (MAList == null)
                return;
            if (MAList.Count == 0)
                return;
            if (MAList.Count > 12)
                abBMB = MAList[rnd.Next(0, 12)];
            else
                if (MAList.Count > 3) // random 1 of 4 first/closest
                    abBMB = MAList[rnd.Next(0, 3)];
                else abBMB = MAList[0];
        }
        else abBMB = BMBFlightAB;

        if (abBMB == null)
            return;
        activatedGroups.Add(abBMB.sqn);
        if (size > abBMB.maxPlanes) size = abBMB.maxPlanes;
        if (!abBMB.aircraftType.Contains("Ju-87"))
            gs = null; // just area bomb

        BMBFlightAB = abBMB;
        if (odenDebug) GamePlay.gpLogServer(null, "CreateBMBMission {0} from {1} in {2}", new object[] { abBMB.name, abBMB.airportName, abBMB.aircraftType });

        AiAirport ap = abBMB.ap;
        bool antiShip = false;
        if (aTarget != null)
        {
            if (aTarget is AiGroundActor)
            {
                //AiGroundActor gndActor = (AiGroundActor)aTarget;
                //if (gndActor.Type().ToString().Equals("ShipTransport"))
                // always go low on fround units
                antiShip = true;
            };
        };
        //if(odenDebug) GamePlay.gpLogServer(null, "antiShip {0} {1} {2} {3}", new object[] { BMB2d.x, size, antiShip, abBMB.name });

        AiActor[] actorsQueue = null;
        string apName = "AIR";
        if (ap != null)
        {
            actorsQueue = ap.QueueTakeoff();
            apName = ap.Name();
        };
        bool setOnPark = false;
        if (actorsQueue != null || BMBFlightIndex > 0 || activatedAirports.Contains(apName))
            setOnPark = true;
        activatedAirports.Add(apName);

        Point2d pt2d = new Point2d(abBMB.x, abBMB.y);

        //if(odenDebug) GamePlay.gpLogServer(null, "pt2d {0} {1} {2} {3}", new object[] { pt2d.x, pt2d.y, antiShip, abBMB.name });
        string sqn = abBMB.sqn; // bp.Regiment().name();
        //string[] PlaneSet = abBMB.aircraftTypes; // bp.GetAircraftTypes();
        string planeType = abBMB.aircraftType; // PlaneSet[rnd.Next(0, PlaneSet.Length - 1)];
        if (odenDebug) GamePlay.gpLogServer(null, "CreateBMBMission {0} {1} {2} {3}", new object[] { abBMB.name, abBMB.airportName, planeType, antiShip });

        // 01 = Stab./1plane
        // 11 = 1./1plane
        // 21 = 2./1plane
        // 31 = 3./1plane
        // 17 = 1./3x1plane
        // 27 = 1./3x2plane
        // 37 = 1./3x3plane
        // 11 = 1./1x3plane#1
        // 12 = 1./1x3plane#2
        // 14 = 1./1x3plane#3
        // 01 = 1./1x1plane
        // 02 = 2./1x1plane
        // 04 = 3./1x1plane
        // 08 = 4./1x1plane
        // == 27 is 2nd flight planes in section 3 (+04), 2 (+02) and 1 (+01)
        /*
        string[] section = { "1", "2", "4" };
        if (size < 6)
            sqn = sqn + "." + BMBFlightIndex.ToString() + section[BMBSectionIndex]; // only single flight, max 3x4 planes (2x3 planes early style)
        else
            sqn = sqn + "." + BMBFlightIndex.ToString() + "3"; // only single flight, max 3x4 planes (2x3 planes early style)
        // big sets fly bad anyway, use max 2 flights of 3 or 4 planes
        */
        Point2d offset2d = new Point2d(0, 0);
        double heading = calcDir(pt2d, BMB2d);
        int mult = 0;
        foreach (string str in activatedAirports)
            if (str.Equals(apName))
                mult++;
        if (ap == null) offset2d = calcPos(offset2d, (1609 * 3) - (mult * 700), heading); // map entry point trigger
        else offset2d = calcPos(offset2d, mult * 700, -heading);
        /*
        switch (BMBSectionIndex)
        {
            case 1: { offset2d = calcPos(offset2d, 300, heading + 120); break; };
            case 2: { offset2d = calcPos(offset2d, 300, heading - 120); break; };
        };
        */
        string form = "VIC3";
        if (abBMB.type.Contains("Stuka")) form = "VIC";
        if (abBMB.type.Contains("Zerstorer")) form = "FINGERFOUR";
        if (abBMB.type.Contains("Fighter")) form = "FINGERFOUR";
        if (abBMB.type.Contains("Recon")) form = "LINEABREAST";
        if (abBMB.type.Contains("Bomber") && abBMB.sqn.Contains("RAF_")) form = "VIC";
        if (abBMB.type.Contains("Fighter") && abBMB.sqn.Contains("Early")) form = "VIC3";

        if (abBMB.type.Contains("Recon") && size > 4) size = 4;

        string value = "1"; // LINEABREAST Recon flights
        string[] FlightSection = FlightSections(army, size, form, BMBStaffelIndex);
        sqn += "." + FlightSection[4];
        AddToFile(f, "AirGroups", sqn, "");
        if (FlightSection[0].Length > 0) AddToFile(f, sqn, "Flight0", FlightSection[0]);
        if (FlightSection[1].Length > 0) AddToFile(f, sqn, "Flight1", FlightSection[1]);
        if (FlightSection[2].Length > 0) AddToFile(f, sqn, "Flight2", FlightSection[2]);
        if (FlightSection[3].Length > 0) AddToFile(f, sqn, "Flight3", FlightSection[3]);

        AddToFile(f, sqn, "Class", planeType);

        //if (size > 3) AddToFile(f, sqn, "Formation", "LINEABREAST");
        //else AddToFile(f, sqn, "Formation", "VIC3");
        if (aTarget != null)
            if (aTarget is AiGroundActor)
                if (!(aTarget as AiGroundActor).Type().ToString().Equals("ShipTransport"))
                    form = "LINEASTERN";
        AddToFile(f, sqn, "Formation", form);

        AddToFile(f, sqn, "CallSign", ((int)(rnd.Next(0, 35))).ToString());
        AddToFile(f, sqn, "Fuel", "84");

        switch (planeType)
        {
            case "Aircraft.BlenheimMkI": value = "1 1 2 1 1"; break;
            case "Aircraft.BlenheimMkIV": value = "1 1 3 1 1"; break;
            case "Aircraft.WellingtonMkIc": value = "1 1 3"; break;
            case "Aircraft.Bf-110C-7": value = "1 1 1 2"; break;
            case "Aircraft.Bf-109E-1B": value = "1 1 2"; break;
            case "Aircraft.Bf-109E-3B": value = "1 1 2"; break;
            case "Aircraft.Bf-109E-4B": value = "1 1 2"; break;
            case "Aircraft.Ju-87B-2": value = "1 1 2 1"; break;
            case "Aircraft.Do-17Z-1": value = "1 1 1 1 4 2"; break;
            case "Aircraft.Do-17Z-2": value = "1 1 1 1 1 1 4 2"; break;
            case "Aircraft.Do-215B-1": value = "1 1 1 2"; break;
            case "Aircraft.Ju-88A-1": value = "1 1 1 2 2 4"; break;
            case "Aircraft.He-111P-2": value = "1 1 1 1 1 2"; break;
            case "Aircraft.He-111H-2": value = "1 1 1 1 1 1 2"; break;
            case "Aircraft.FW-200C-1": value = "1 1 1 1 2 2 2"; break;
            case "Aircraft.BR-20M": value = "1 1 1 3"; break;
        };
        if (antiShip)
            switch (planeType)
            {
                case "Aircraft.Bf-110C-7": value = "1 1 1 5"; break;
                case "Aircraft.Ju-87B-2": value = "1 1 4 1"; break;
                case "Aircraft.Do-17Z-1": value = "1 1 1 1 5 3"; break;
                case "Aircraft.Do-17Z-2": value = "1 1 1 1 1 1 5 3"; break;
                case "Aircraft.Do-215B-1": value = "1 1 1 3"; break;
                case "Aircraft.Ju-88A-1": value = "1 1 1 3 2 7"; break;
                case "Aircraft.He-111P-2": value = "1 1 1 1 1 3"; break;
                case "Aircraft.He-111H-2": value = "1 1 1 1 1 1 3"; break;
                case "Aircraft.BR-20M": value = "1 1 1 6"; break;
                case "Aircraft.He-115B-2": value = "1 1 2"; break;
            };
        AddToFile(f, sqn, "Weapons", value);
        if (setOnPark)
            AddToFile(f, sqn, "SetOnPark", "1");
        //AddToFile(f, sqn, "SpawnFromScript", "1");
        AddToFile(f, sqn, "Skill", "1.0 0.2 0.7 0.2 0.7 0.8 0.5 0.8");
        int aging = (gameDateTime - BoBDateTime).Days;
        if (aging > 99) aging = 99;
        AddToFile(f, sqn, "Aging", aging.ToString());

        int alt = rnd.Next(2500, 4500);
        if (planeType.Contains("Do-17Z")) alt = rnd.Next(100, 800); // schnellbomber best utilized at low alt (engines)
        if (planeType.Contains("Bf-110")) alt = rnd.Next(100, 800);
        if (antiShip && !planeType.Contains("Ju-87"))
            alt = rnd.Next(600, 1200);
        if (BMBFlightAlt == 0) BMBFlightAlt = alt;
        else alt = BMBFlightAlt;
        int speed = rnd.Next(300, 350);
        if (planeType.Contains("Bf-109")) speed = rnd.Next(350, 400);
        if (planeType.Contains("Bf-110")) speed = rnd.Next(320, 380);
        if (planeType.Contains("Blenheim")) speed = 320;
        if (planeType.Contains("Wellington")) speed = 290;
        if (planeType.Contains("Beaufighter")) speed = rnd.Next(350, 400);
        if (planeType.Contains("Ju-88")) speed = rnd.Next(320, 380);
        if (planeType.Contains("He-111P")) speed = rnd.Next(310, 370);
        if (planeType.Contains("He-111H")) speed = rnd.Next(310, 370);
        if (BMBFlightSpeed == 0) BMBFlightSpeed = speed;
        else speed = BMBFlightSpeed;

        double rwlen = 0;
        if (ap != null)
            rwlen = ap.FieldR();
        if (army == LW)
        {   // fast forward to CH Radar range (85 miles at 15kft)
            // 25miles at 1000ft, 35miles at 2000ft, 50miles at 5000ft and 83miles at 13000ft +
            // 207ft /mile = 63m /mile
            int rng = 25 + (int)(alt / 63); // miles
            if (rng > 85)
                rng = 85;
            //Chain Home Low 500ft 35 miles
            if (alt < 200)
                rng = 35;

            GroundStationary chRadar = FindChainHomeRadar(pt2d);
            if (chRadar != null)
            {
                Point2d radar2d = new Point2d(chRadar.pos.x, chRadar.pos.y);
                double dir = calcDir(radar2d, pt2d);
                Point2d rdp2d = calcPos(radar2d, rng * 1609, dir);
                //if(odenDebug) GamePlay.gpLogServer(null, "CH Radar: target to RDP {0}, target to spawn {1}, RDP to spawn {2} at {3} deg", new object[] { BMB2d.distance(ref rdp2d).ToString("F2"), BMB2d.distance(ref pt2d).ToString("F2"), rdp2d.distance(ref pt2d).ToString("F2"), dir.ToString("F2") });
                if (BMB2d.distance(ref rdp2d) < BMB2d.distance(ref pt2d))
                {
                    //pt2d = rdp2d; // radar detection point (rdp2d) is closer to target (BMB2d) than base (pt2d) is
                    pt2d = calcPos(radar2d, rng * 1609, dir);
                    rwlen = 0; // airstart
                    //if(odenDebug) GamePlay.gpLogServer(null, "CH Radar: new spawn {0} {1} at {2} deg from {3} {4}", new object[] { pt2d.x.ToString("F2"), pt2d.y.ToString("F2"), dir.ToString("F2"), radar2d.x.ToString("F2"), radar2d.y.ToString("F2") });
                };
            }
            else
            {   // all radars destroyed
                rwlen = 0;
                Point2d spawn2d = calcPos(BMB2d, 15 * 1609, -heading);
                if (BMB2d.distance(ref spawn2d) < BMB2d.distance(ref pt2d))
                    pt2d = spawn2d; // detection point 15 miles out - check land/observer corps?
            };
        };

        pt2d.x = pt2d.x + offset2d.x;
        pt2d.y = pt2d.y + offset2d.y;
        string strWP = "";
        List<string> listWP = new List<string>();
        Point2d esc2d;
        int escAlt = 100 + rnd.Next(0, 200);

        if (1000 > rwlen)
        {   // airstart if short runway or map entry point
            //pt2d = calcPos(pt2d, rnd.Next(1000, 3000), rnd.Next(0, 359));
            AddToFile(f, sqn + "_Way", "NORMFLY", pt2d.x.ToString("F2") + " " + pt2d.y.ToString("F2") + " " + alt.ToString("F2") + " " + speed.ToString("F2"));
            esc2d = calcPos(pt2d, 700, heading + 90);
            strWP = esc2d.x.ToString("F2") + " " + esc2d.y.ToString("F2") + " " + (alt + escAlt).ToString("F2") + " " + speed.ToString("F2");
            listWP.Add(strWP);
        }
        else
        {
            AddToFile(f, sqn + "_Way", "TAKEOFF", abBMB.x.ToString("F2") + " " + abBMB.y.ToString("F2") + " 0 0");
            double dir = calcDir(pt2d, BMB2d);
            Point2d wp2d = calcPos(pt2d, 16090 / 2, dir);
            AddToFile(f, sqn + "_Way", "NORMFLY", wp2d.x.ToString("F2") + " " + wp2d.y.ToString("F2") + " 1680.00" + " 280.00");
        };

        Point2d mid2d = calcPos(BMB2d, 16090 / 2, -heading);
        AddToFile(f, sqn + "_Way", "NORMFLY", mid2d.x.ToString("F2") + " " + mid2d.y.ToString("F2") + " " + alt.ToString() + ".00 " + speed.ToString() + ".00");

        esc2d = calcPos(BMB2d, 700, heading + 90);
        strWP = esc2d.x.ToString("F2") + " " + esc2d.y.ToString("F2") + " " + (alt + escAlt).ToString("F2") + " " + speed.ToString("F2");
        listWP.Add(strWP);
        if (aTarget == null && gs == null)
        {
            AddToFile(f, sqn + "_Way", "GATTACK_POINT", BMB2d.x.ToString("F2") + " " + BMB2d.y.ToString("F2") + " " + alt.ToString() + ".00 " + speed.ToString() + ".00");
        }
        else
        {
            string profile = "SHALLOW_DIVE";
            if (alt < 500) profile = "TOP_MAST";
            if (gs != null) profile = "LEVEL";
            if (planeType.Contains("Ju-87")) profile = "DIVE";
            BMB2d = calcPos(BMB2d, 16100 / 4, -heading);
            //GATTACK_TARG 110495.05 146698.26 3500.00 290.00 0_Chief 0 /GAttackType DIVE
            strWP = BMB2d.x.ToString("F2") + " " + BMB2d.y.ToString("F2") + " " + alt.ToString() + ".00 " + speed.ToString() + ".00";
            if (aTarget != null)
                AddToFile(f, sqn + "_Way", "GATTACK_TARG", strWP + " " + aTarget.Name() + " 0 /GAttackType " + profile);
            if (gs != null)
                AddToFile(f, sqn + "_Way", "GATTACK_TARG", strWP + " " + gs.Name + " 0 /GAttackType " + profile);
        };

        // RTB
        AddToFile(f, sqn + "_Way", "NORMFLY", pt2d.x.ToString("F2") + " " + pt2d.y.ToString("F2") + " " + alt.ToString() + ".00 " + speed.ToString() + ".00");
        //esc2d = calcPos(pt2d, 700, heading + 90);
        //strWP = esc2d.x.ToString("F2") + " " + esc2d.y.ToString("F2") + " " + (alt + 100).ToString("F2") + " " + speed.ToString("F2");        
        //listWP.Add(strWP);

        alt = (int)(alt / 2);
        strWP = abBMB.x.ToString("F2") + " " + abBMB.y.ToString("F2") + " " + alt.ToString() + ".00 " + speed.ToString() + ".00";
        if (rwlen == 0)
            AddToFile(f, sqn + "_Way", "NORMFLY", strWP);
        else
            AddToFile(f, sqn + "_Way", "LANDING", strWP);
        listWP.Add(strWP);
        /*
        BMBSectionIndex++;
        if (BMBSectionIndex > 2)
        {
            BMBSectionIndex = 0;
            BMBFlightIndex++;
        };
        */
        switch (form)
        {
            // Bomber max 3 +VIC3
            // Zerstörer max 4 +FF
            // Fighter max 4 +FF
            // Recon max 1 +LINEABREAST
            // Transp. max 3 +VIC
            // UK Bomber max 10 +VIC
            // UK Fighter max 6 +VIC3
            // UK Fighter max 4 +FF
            // LINEASTERN
            case "VIC":
                if (size > 8) BMBFlightIndex = 9;
                break;
            case "VIC3":
                if (army == RAF && size > 11) BMBFlightIndex = 3;
                if (army != RAF && size > 8) BMBFlightIndex = 3;
                break;
            case "FINGERFOUR":
                if (size > 11) BMBFlightIndex = 3;
                break;
            default:
                BMBFlightIndex = 3;
                break;
        };

        BMBFlightIndex++;
        BMBStaffelIndex++;
        if (BMBFlightIndex > 2 || BMBStaffelIndex > 4)
        {
            BMBFlightAB = null;
            BMBFlightIndex = 0;
            BMBStaffelIndex = 0;
        };

        if (escortReq)
        {
            MissionAirbase home = null;
            speed += 50;
            string sqnF;
            if (army == LW)
            {
                sqnF = CreateCAPMission(f, BMB2d, 12, LW, true);
                if (odenDebug) GamePlay.gpLogServer(null, "Escort {0}", new object[] { sqnF });
                if (sqnF.Length > 0)
                {
                    foreach (MissionAirbase mab in MissionAirbases)
                        if (sqnF.Contains(mab.sqn))
                            home = mab;
                    if (rwlen > 0)
                        AddToFile(f, sqnF + "_Way", "TAKEOFF", home.x.ToString("F2") + " " + home.y.ToString("F2") + " 0 0");
                    foreach (string str in listWP)
                        AddToFile(f, sqnF + "_Way", "ESCORT", str + " " + sqn + " 1");
                    AddToFile(f, sqnF + "_Way", "LANDING", home.x.ToString("F2") + " " + home.y.ToString("F2") + " " + alt.ToString() + ".00 " + speed.ToString() + ".00");
                };
            }
            else
            {
                for (int cnt = 1; cnt < 2; cnt++)
                {
                    sqnF = CreateCAPMission(f, BMB2d, 6, RAF, true);
                    if (sqnF.Length > 0)
                    {
                        foreach (MissionAirbase mab in MissionAirbases)
                            if (sqnF.Contains(mab.sqn))
                                home = mab;
                        if (rwlen > 0)
                            AddToFile(f, sqnF + "_Way", "TAKEOFF", home.x.ToString("F2") + " " + home.y.ToString("F2") + " 0 0");
                        foreach (string str in listWP)
                            AddToFile(f, sqnF + "_Way", "ESCORT", str + " " + sqn + " 1");
                        AddToFile(f, sqnF + "_Way", "LANDING", home.x.ToString("F2") + " " + home.y.ToString("F2") + " " + alt.ToString() + ".00 " + speed.ToString() + ".00");
                    };
                };
            };
            if (rnd.Next(0, 9) > 6)
            {   // fighter sweep
                sqnF = CreateCAPMission(f, BMB2d, 12, army, true);
                if (odenDebug) GamePlay.gpLogServer(null, "Sweep {0}", new object[] { sqnF });
                sweepGroups.Add(sqnF);
                foreach (MissionAirbase mab in MissionAirbases)
                    if (sqnF.Contains(mab.sqn))
                        home = mab;
                alt = rnd.Next(2500, 4500);
                if (rnd.Next(0, 99) < 10) alt = rnd.Next(700, 1500); // lure defence down low
                esc2d = calcPos(pt2d, 700, heading);

                if (rwlen > 0)
                    AddToFile(f, sqnF + "_Way", "TAKEOFF", home.x.ToString("F2") + " " + home.y.ToString("F2") + " 0 0");
                else
                    AddToFile(f, sqnF + "_Way", "AATTACK_FIGHTERS", esc2d.x.ToString("F2") + " " + esc2d.y.ToString("F2") + " " + alt.ToString("F2") + " " + speed.ToString("F2"));
                AddToFile(f, sqnF + "_Way", "AATTACK_FIGHTERS", BMB2d.x.ToString("F2") + " " + BMB2d.y.ToString("F2") + " " + alt.ToString() + ".00 " + speed.ToString() + ".00");
                AddToFile(f, sqnF + "_Way", "LANDING", home.x.ToString("F2") + " " + home.y.ToString("F2") + " " + alt.ToString() + ".00 " + speed.ToString() + ".00");
            };
        };
        //AddToFile(f,"Trigger","BombGround","TPassThrough 1 " + sqn + " " + BMB2d.x.ToString() + " " + BMB2d.y.ToString() + " 2000");
        //trigger TPassThrough 5 2 249488 256453 1750: 5 = ArmyAir, 2 = LW
        //when trigger goes inactive - check damage
    }

    private Point2d calcPos(Point2d pt2d, double dist, double dir)
    {
        dir = dir * Math.PI / 180;
        return new Point2d(pt2d.x + (dist * Math.Sin(dir)), pt2d.y + (dist * Math.Cos(dir)));
    }

    private double calcDir(Point2d from2d, Point2d to2d)
    {
        double rad = Math.Atan2(to2d.x - from2d.x, to2d.y - from2d.y);
        double deg = rad * 180 / Math.PI;
        if (rad < 0)
            return 360 + deg;
        return deg;
        //double dir = ((double)(-57.3f * Math.Atan2(from2d.y - to2d.y, from2d.x - to2d.x)) - 90f);
        //if (dir < 0f) dir += 360f;
        //if (dir > 360f) dir -= 360f;
        //return dir;
    }

    private bool ChainHomeRadarDetected(AiAircraft ac)
    {
        Point2d pt2d = new Point2d(ac.Pos().x, ac.Pos().y);
        GroundStationary chRadar = FindChainHomeRadar(pt2d);
        if (chRadar == null)
            return false;
        // 25miles at 1000ft, 35miles at 2000ft, 50miles at 5000ft and 83miles at 13000ft +
        // 207ft /mile = 63m /mile
        double alt = ac.Pos().z;
        int rng = 25 + (int)(alt / 63); // miles
        if (rng > 85)
            rng = 85;
        //Chain Home Low 500ft 35 miles
        if (alt < 200)
            rng = 35;

        Point2d radar2d = new Point2d(chRadar.pos.x, chRadar.pos.y);
        if (radar2d.distance(ref pt2d) < rng * 1609)
            return true;
        return false;
    }

    private void UpdateWP(AiGroup ag)
    {
        if (ag == null)
            return;
        if (ag.GetItems().Length < 1)
            return;
        if (TargetGroup != null)
            return;
        if (target2d.x == 0)
            return;

        AiActor agLeader = ag.GetItems()[0];
        Point3d refPos3d = agLeader.Pos();

        AiWayPoint[] arrWP = ag.GetWay();
        int currWPidx = ag.GetCurrentWayPoint();
        AiWayPoint currWP = arrWP[currWPidx];
        AiAirWayPoint currAWP = (AiAirWayPoint)arrWP[currWPidx];
        AiAirWayPoint nextAWP = (AiAirWayPoint)arrWP[currWPidx];
        AiAirWayPoint dmmyAWP = (AiAirWayPoint)arrWP[currWPidx];
        // 180 mph = 290 kmh
        // 200 mph = 321 kmh
        // 220 mph = 354 kmh
        // 240 mph = 386 kmh
        // 260 mph = 418 kmh

        if (currWPidx == 1 && currWP.P.distance(ref refPos3d) < (2 * 16090))
        {   // last wp
            //if (odenDebug) GamePlay.gpLogServer(null, "wp {0} for {1} do {2} {3} at {4} m away", new object[] { currWPidx, ag.Name(), currAWP.Action, currAWP.Target, dist });
            currWP.Speed = 290; // 180 mph
            // seems not if(ag.Army() == RAF ) currWP.Speed = 180; // test: is this why they fly so darn fast?
            currAWP.Action = AiAirWayPointType.NORMFLY;

            currAWP.P = new Point3d(refPos3d.x, refPos3d.y, refPos3d.z);
            nextAWP.Speed = currAWP.Speed;
            dmmyAWP.Speed = currAWP.Speed;
            nextAWP.Action = currAWP.Action;
            dmmyAWP.Action = currAWP.Action;
            nextAWP.P = new Point3d(target2d.x, target2d.y, rnd.Next(800, 3200));
            dmmyAWP.P = new Point3d(0, 0, 1000);

            Point2d pt2d = new Point2d(agLeader.Pos().x, agLeader.Pos().y);
            arrWP = new AiWayPoint[] { currAWP, nextAWP, dmmyAWP };
            ag.SetWay(arrWP);
            LeaderUpdateMsg();
        };
    }

    private void InterceptWP(AiGroup ag)
    {
        if (ag == null)
            return;
        if (ag.GetItems().Length < 1)
            return;
        AiActor agLeader = ag.GetItems()[0];
        if (TargetGroup.GetItems().Length < 1)
            return;
        AiActor TargetLeader = TargetGroup.GetItems()[0];
        if (TargetLeader == null)
            for (int i = 1; i < TargetGroup.GetItems().Length; i++)
            {
                TargetLeader = TargetGroup.GetItems()[i];
                if (TargetLeader != null)
                    break;
            };
        if (TargetLeader == null)
        {
            TargetGroup = null;
            return;
        };
        Point3d refPos3d = TargetLeader.Pos();

        AiWayPoint[] arrWP = ag.GetWay();
        int currWPidx = ag.GetCurrentWayPoint();
        AiWayPoint currWP = arrWP[currWPidx];
        AiAirWayPoint currAWP = (AiAirWayPoint)arrWP[currWPidx];
        AiAirWayPoint nextAWP = (AiAirWayPoint)arrWP[currWPidx];
        AiAirWayPoint dmmyAWP = (AiAirWayPoint)arrWP[currWPidx];
        // 180 mph = 290 kmh
        // 200 mph = 321 kmh
        // 220 mph = 354 kmh
        // 240 mph = 386 kmh
        // 260 mph = 418 kmh

        if (currWP.P.distance(ref refPos3d) > (2 * 16090)) // target outside RAF grid
        {   // last wp
            //if (odenDebug) GamePlay.gpLogServer(null, "wp {0} for {1} do {2} {3} at {4} m away", new object[] { currWPidx, ag.Name(), currAWP.Action, currAWP.Target, dist });
            currAWP.Action = AiAirWayPointType.HUNTING;
            currAWP.Speed = 354; // 220 mph
            // seems not if (ag.Army() == RAF) currAWP.Speed = 220; // test: is this why they fly so darn fast?

            nextAWP.Speed = currAWP.Speed;
            dmmyAWP.Speed = currAWP.Speed;
            nextAWP.Action = currAWP.Action;
            dmmyAWP.Action = currAWP.Action;

            Point2d pt2d = new Point2d(agLeader.Pos().x, agLeader.Pos().y);
            Point2d tg2d = new Point2d(TargetLeader.Pos().x, TargetLeader.Pos().y);
            //double dir = calcDir(pt2d, tg2d);
            double dir = calcDir(target2d, tg2d);
            int dist = (int)(TargetLeader.Pos().distance(ref target3d)); // bombers to target
            Point2d next2d = calcPos(target2d, dist * 0.5, dir); // intercept point
            // use target2d Point2d extr2d = calcPos(target2d, dist * 0.67, dir); // never to reach wp as new will be given after next2d
            string tgtSector = GamePlay.gpSectorName(next2d.x, next2d.y);

            nextAWP.P = new Point3d(next2d.x, next2d.y, TargetLeader.Pos().z + 300);
            currAWP.Target = TargetLeader;
            nextAWP.Target = TargetLeader;
            dmmyAWP.Target = TargetLeader;
            arrWP = new AiWayPoint[] { currAWP, nextAWP, dmmyAWP };
            ag.SetWay(arrWP);
            LeaderUpdateMsg();
        };
    }

    private void UpdateGroundTarget(AiGroup ag)
    {
        if (ag == null)
            return;
        if (ag.Name() == "NONAME")
            return;
        //if (odenDebug) GamePlay.gpLogServer(null, "UpdateGroundTarget for {0}", new object[] { ag.Name() });
        if (ag.GetItems().Length < 1)
            return;

        AiActor agLeader = ag.GetItems()[0];
        Point3d refPos3d = agLeader.Pos();
        double minDist = 999999f;
        GroundTarget = null;
        int enemy = ag.Army() == RAF ? LW : RAF;
        foreach (AiGroundGroup gg in GamePlay.gpGroundGroups(enemy))
            if (gg.GroupType() == AiGroundGroupType.Vehicle)
            {
                Point3d pt3d;
                gg.GetPos(out pt3d);
                double ggDist = ag.Pos().distance(ref pt3d);
                if (ggDist < minDist)
                {
                    GroundTarget = gg;
                    minDist = ggDist;
                };
            };
        if (GroundTarget == null)
            return;
        if (GroundTarget.GetItems().Length < 1)
            return;

        AiActor aTarget = GroundTarget.GetItems()[0];
        AiWayPoint[] arrWP = ag.GetWay();
        int currWPidx = ag.GetCurrentWayPoint();
        AiAirWayPoint currAWP = (AiAirWayPoint)arrWP[currWPidx];
        AiAirWayPoint nextAWP = (AiAirWayPoint)arrWP[currWPidx];
        AiAirWayPoint extrAWP = (AiAirWayPoint)arrWP[currWPidx];

        //if (odenDebug) GamePlay.gpLogServer(null, "UpdateGroundTarget check {0} {1} {2}", new object[] { ag.Name(), currWPidx, arrWP.Length });
        if (currWPidx == arrWP.Length - 1 && aTarget != currAWP.Target && currAWP.P.distance(ref refPos3d) < 16090)
        {   // last wp            
            Point2d pt2d = new Point2d(agLeader.Pos().x, agLeader.Pos().y);
            Point2d tg2d = new Point2d(aTarget.Pos().x, aTarget.Pos().y);
            double dir = calcDir(tg2d, pt2d);
            Point2d inb2d = calcPos(tg2d, 5 * 1609, dir);
            nextAWP.Speed = 290; // 180 mph
            // nope if (ag.Army() == RAF) nextAWP.Speed = 180; // test: is this why they fly so darn fast?
            nextAWP.P = new Point3d(inb2d.x, inb2d.y, rnd.Next(800, 1200));
            nextAWP.Action = AiAirWayPointType.GATTACK_TARG;
            nextAWP.Target = aTarget;
            Point2d out2d = calcPos(tg2d, 5 * 1609, dir + 90 - rnd.Next(0, 180));
            extrAWP.P = new Point3d(out2d.x, out2d.y, nextAWP.P.z);
            extrAWP.Action = AiAirWayPointType.NORMFLY;
            extrAWP.Target = null;
            arrWP = new AiWayPoint[] { currAWP, nextAWP, extrAWP };
            ag.SetWay(arrWP);
            //if (odenDebug) GamePlay.gpLogServer(null, "UpdateGroundTarget new {0}", new object[] { nextAWP.Target.Name() });
        };
    }

    private void RedirectCAP(int army, Point2d pt2d)
    {
        target2d = pt2d;
        AiAirGroup[] ags = GamePlay.gpAirGroups(army);
        if (ags != null)
            foreach (AiAirGroup ag in ags)
                if (ag.isAircraftType(AircraftType.Fighter)
                    || ag.isAircraftType(AircraftType.BNZFighter)
                    || ag.isAircraftType(AircraftType.HeavyFighter)
                    || ag.isAircraftType(AircraftType.TNBFighter))
                    if (!ag.hasBombs() && !ag.hasTorpedos() && !ag.hasBombsInInternalBays())
                        if (!AiAirGroupsStdby.Contains(ag))
                            GiveNewWP(ag, pt2d);

        if (GamePlay.gpPlayer() != null)
            if (rnd.Next(0, 9) > 7 || GamePlay.gpPlayer().Army() != army)
            {
                ISectionFile f = GamePlay.gpCreateSectionFile();
                CreateCAPMission(f, pt2d, 12, army, false);
                GamePlay.gpPostMissionLoad(f);
                NextMissionNum++;
            };
    }

    private void GiveNewWP(AiGroup grp, Point2d wp2d)
    {
        if (grp == null)
            return;
        if (grp.GetItems().Length < 1)
            return;
        AiActor grpLeader = grp.GetItems()[0];
        Point3d refPos3d = grpLeader.Pos();
        AiAircraft ac = (AiAircraft)grpLeader;
        AiAirGroup ag = ac.AirGroup();

        AiWayPoint[] arrWP = grp.GetWay();
        int currWPidx = grp.GetCurrentWayPoint();
        AiAirWayPoint currAWP = (AiAirWayPoint)arrWP[currWPidx];
        AiAirWayPoint nextAWP = (AiAirWayPoint)arrWP[currWPidx];
        AiAirWayPoint lastAWP = (AiAirWayPoint)arrWP[currWPidx]; // dummy

        if (currWPidx == 1 && currAWP.P.distance(ref refPos3d) < 16090)
        {
            //if (odenDebug) GamePlay.gpLogServer(null, "GiveNewWP {0} for {1} do {2} {3}", new object[] { currWPidx, ag.Name(), currAWP.Action, currAWP.Target });
            currAWP.Action = AiAirWayPointType.NORMFLY;
            if (EscortGroup != null)
                if (EscortGroup.Army() == grp.Army())
                    if (EscortGroup.GetItems().Length > 0)
                    {
                        //nope currAWP.Action = AiAirWayPointType.LANDING;
                        currAWP.Action = AiAirWayPointType.ESCORT;
                        currAWP.Target = EscortGroup.GetItems()[0];
                    };

            currAWP.Action = AiAirWayPointType.NORMFLY;
            if (EscortGroup != null)
                if (EscortGroup.Army() == grp.Army())
                    if (EscortGroup.GetItems().Length > 0)
                    {
                        currAWP.Action = AiAirWayPointType.ESCORT;
                        currAWP.Target = EscortGroup.GetItems()[0];
                        //currAWP.Action = AiAirWayPointType.LANDING;
                        // after setWay? ag.setTask(AiAirGroupTask.DEFENDING, EscortGroup);
                    };

            currAWP.Speed = 321; // 200 mph
            if (grp.Army() == RAF) currAWP.Speed = 200; // test for fast going buggers

            nextAWP = currAWP;
            nextAWP.P = new Point3d(wp2d.x, wp2d.y, rnd.Next(1000, 4000));
            if (TargetGroup != null)
                if (TargetGroup.Army() != grp.Army() && TargetGroup.GetItems().Length > 0)
                    nextAWP.P.z = TargetGroup.GetItems()[0].Pos().z + rnd.Next(100, 400);
            if (EscortGroup != null)
                if (EscortGroup.Army() == grp.Army() && EscortGroup.GetItems().Length > 0)
                    nextAWP.P.z = EscortGroup.GetItems()[0].Pos().z + rnd.Next(100, 400);

            if (nextAWP.P.z > refPos3d.z + 1000)
            {
                nextAWP.Speed = 290; // 180 mph
                if (grp.Army() == RAF) nextAWP.Speed = 180; // test for fast going buggers
            };
            lastAWP = nextAWP;
            lastAWP.P = new Point3d(target2d.x, target2d.y, nextAWP.P.z); // keep ai from calling out RTB?
            arrWP = new AiWayPoint[] { currAWP, nextAWP, lastAWP };
            grp.SetWay(arrWP);

            if (EscortGroup != null)
                if (EscortGroup.Army() == grp.Army())
                {
                    /*
                    [14:23:09]	Server: AiAirGroup: '2:BoB_LW_JG51_I.03/LANDING' attached.gps:'0'
                    [14:23:09]	Server: AiAirGroup: '2:BoB_LW_JG51_I.03/LANDING' client:'2:BoB_LW_KG4_I.01'
                    [14:23:09]	Server: AiAirGroup: '2:BoB_LW_JG51_I.03/LANDING' dau.gps:'0'
                    [14:23:09]	Server: AiAirGroup: '2:BoB_LW_JG51_I.03/LANDING' task:'DEFENDING'
                    */
                    //currWP = ag.GetWay()[ag.GetCurrentWayPoint()];
                    //currAWP = (AiAirWayPoint)currWP;                    
                    //ag.setTask(AiAirGroupTask.DO_NOTHING, null); // resets?
                    //ag.setTask(AiAirGroupTask.PURSUIT, EscortGroup);
                    ag.setTask(AiAirGroupTask.DEFENDING, EscortGroup); // bite me
                    //if (odenDebug) GamePlay.gpLogServer(null, "GiveNewTask {0} {1} {2}", new object[] { ag.getTask().ToString(), currAWP.Target.Name(), currAWP.Action.ToString() });
                    /*Timeout(5, () => 
                    {
                        //ag.setTask(AiAirGroupTask.DEFENDING, EscortGroup);
                        ag.setTask(AiAirGroupTask.LANDING, EscortGroup);
                        if (odenDebug) GamePlay.gpLogServer(null, "GiveNewTask 5 {0}", new object[] { ag.getTask().ToString() });
                    });
                    */
                };
            //if (odenDebug) GamePlay.gpLogServer(null, "GaveNewWP {0} for {1} do {2} {3} ({4})", new object[] { currWPidx, ag.Name(), currAWP.Action, currAWP.Target.Name(), ag.getTask().ToString() });
        };
    }

    private Point2d HalfwayToTarget2d(AiAirGroup ag)
    {
        AiActor a = ag.GetItems()[0];
        AiAircraft ac = (AiAircraft)a;
        Point2d halfw2d = new Point2d(ac.Pos().x, ac.Pos().y); // temp use
        Point3d refPos3d = new Point3d(target2d.x, target2d.y, ac.Pos().z);
        int dist = (int)(ac.Pos().distance(ref refPos3d));
        double dir = calcDir(halfw2d, target2d);
        halfw2d = calcPos(halfw2d, dist, dir);
        return halfw2d;
    }

    private void UpdateRTB(bool _bombersAtTarget)
    {
        if (GamePlay.gpPlayer() == null)
            return;
        AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
        if (pac == null || !_bombersAtTarget)
            return;
        AiAirGroup pag = pac.AirGroup();
        GamePlay.gpLogServer(null, "UpdateRTB for {0}", new object[] { pag.Name() });
        AiWayPoint[] arrWP = pag.GetWay();
        int currWPidx = pag.GetCurrentWayPoint();
        AiAirWayPoint currAWP = (AiAirWayPoint)arrWP[currWPidx];
        Point2d inb2d = new Point2d(homePlate3d.x, homePlate3d.y);
        inb2d = calcPos(inb2d, 1609 * 3, rnd.Next(0, 359));
        currAWP.P = new Point3d(inb2d.x, inb2d.y, currAWP.P.y);
        //AiAirWayPoint nextAWP = (AiAirWayPoint)arrWP[currWPidx]; // copy prev.
        //nextAWP.P = homePlate3d;
        AiAirWayPoint nextAWP = null;
        nextAWP = new AiAirWayPoint(ref homePlate3d, 120);
        nextAWP.Action = AiAirWayPointType.LANDING;
        nextAWP.Target = homePlate;

        if (currAWP.Action != AiAirWayPointType.TAKEOFF)
        {
            AiAirWayPoint homeAWP = null;
            homeAWP = new AiAirWayPoint(ref homePlate3d, 120);
            homeAWP.Action = AiAirWayPointType.TAKEOFF;
            homeAWP.Target = homePlate;
            arrWP = new AiWayPoint[] { homeAWP, currAWP, nextAWP };
        }
        else
            arrWP = new AiWayPoint[] { currAWP, nextAWP };
        pag.SetWay(arrWP);

        AiWayPoint wp = arrWP[arrWP.Length - 1];
        (wp as AiAirWayPoint).Action = AiAirWayPointType.LANDING;
        (wp as AiAirWayPoint).Target = homePlate;
        //pag.SetWay(new AiAirWayPoint[] { currAWP, nextAWP });
        pag.setTask(AiAirGroupTask.RETURN, null);
        //debugWaypointValues();
    }

    private void debugWaypointValues()
    {
        if (GamePlay.gpPlayer() == null)
            return;
        AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
        if (pac == null)
            return;
        AiAirGroup pag = pac.AirGroup();
        GamePlay.gpLogServer(null, "debugWaypointValues for {0}", new object[] { pag.Name() });
        AiWayPoint[] arrWP = pag.GetWay();
        foreach (AiWayPoint wp in arrWP)
        {
            AiAirWayPoint awp = (AiAirWayPoint)wp;
            GamePlay.gpLogServer(null, "WP {0} {1} {2}", new object[] { awp.Action, awp.Target, awp.GAttackType });
        };
        GamePlay.gpLogServer(null, "---", new object[] { });
    }

    private void LeaderUpdateMsg()
    {
        Player localPlayer = GamePlay.gpPlayer();
        if (localPlayer == null)
            return;
        AiAircraft pac = (AiAircraft)localPlayer.Place();
        if (pac == null)
            return; // ejected etc.
        AiAirGroup pag = pac.AirGroup();
        if (pag.GetItems().Length < 2)
            return; // only talk to myself
        AiAircraft lac = (AiAircraft)pag.GetItems()[0];
        if (lac == null)
            return; // no one to talk to

        AiWayPoint[] arrWP = pag.GetWay();
        int currWPidx = pag.GetCurrentWayPoint();
        if (timeoutRadio < DateTime.Now)
        {
            AiWayPoint currWP = arrWP[currWPidx];
            Point3d pt3d = currWP.P;
            if (pt3d.x != oldWP.P.x || pt3d.z != oldWP.P.z)
            {
                timeoutRadio = DateTime.Now;
                timeoutRadio.AddSeconds(12);
                if (odenDebug) GamePlay.gpLogServer(null, "LeaderUpdateMsg {0} '{1}m'", new object[] { GamePlay.gpSectorName(pt3d.x, pt3d.y), pt3d.z });
                //acLeader.SayToGroup(acLeader.AirGroup(), "This_is");
                lac.SayToGroup(pag, lac.CallSign());
                lac.SayToGroup(pag, "Next_check_point");
                sayGrid(lac, pag, GamePlay.gpSectorName(pt3d.x, pt3d.y));
                lac.SayToGroup(pag, "Changed_course");
                sayDirection(lac, pag, pt3d);
                sayAltitude(lac, pag, pt3d.z);
            };
            oldWP = currWP;
        };
    }

    private void GndCtrlUpdateMsg()
    {
        if (TargetGroup == null)
            return;

        if (GamePlay.gpPlayer() == null)
            return;
        AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
        if (pac == null)
            return;
        if (pac.Army() == ATK)
            return;

        if (TargetGroup.GetItems().Length > 1)
        {
            AiAirGroup pag = pac.AirGroup();
            AiActor TargetLeader = TargetGroup.GetItems()[0];

            string planeSector = GamePlay.gpSectorName(TargetLeader.Pos().x, TargetLeader.Pos().y);
            AiActor refActor = pag.GetItems()[0];
            Point3d refPos3d = refActor.Pos();
            int dist = (int)(TargetLeader.Pos().distance(ref refPos3d) / 1000);
            int miles = (int)(dist * .62f);
            if (miles > 5)
            {
                if ((planeSector.IndexOf(">") == -1 && planeSector.IndexOf("<") == -1))
                {
                    timeoutRadio = DateTime.Now;
                    timeoutRadio.AddSeconds(12);

                    AiAircraft fc = FighterCommand;
                    fc.SayToGroup(pag, "Enemy_group");
                    sayGrid(fc, pag, planeSector);
                    sayAltitude(fc, pag, TargetLeader.Pos().z);
                    gameIsOn = true;
                    if (odenDebug) GamePlay.gpLogServer(null, "GndCtrlUpdateMsg {0} {1}", new object[] { planeSector, TargetGroup.Name() });
                    if (firstMessage)
                    {
                        firstMessage = false;
                        AiAircraft lac = (AiAircraft)pag.GetItems()[0];
                        if (lac != null)
                        {
                            Timeout(12, () =>
                            {
                                timeoutRadio = DateTime.Now;
                                timeoutRadio.AddSeconds(12);
                                lac.SayToGroup(pag, lac.CallSign());
                                lac.SayToGroup(pag, "Let_s_go_get_them");
                                InterceptWP(pag);
                            });
                        };
                    }
                    else { InterceptWP(pag); };
                }
                else
                {
                    if (gameIsOn && gameWon > 0 && !TargetGroup.hasTorpedos() && !TargetGroup.hasBombs() && !TargetGroup.hasBombsInInternalBays())
                    {   // mission over
                        if (odenDebug) GamePlay.gpLogServer(null, "GndCtrlUpdateMsg {0} {1}", new object[] { "FC:", "!hasBombs" });
                        sayMissionOver(gameWon);
                        GamePlay.gpLogServer(null, "UpdateRTB from {0}", new object[] { "GndCtrlUpdateMsg mission over no TargetGroup wpn" });
                        UpdateRTB(bombersAtTarget);
                        TargetGroup = null;
                    };
                };
            };
        }
        else
        {   // look for new target?
            gameWon = ATK == RAF ? LW : RAF;
            sayMissionOver(gameWon);
            GamePlay.gpLogServer(null, "UpdateRTB from {0}", new object[] { "GndCtrlUpdateMsg mission over no TargetGroup" });
            UpdateRTB(bombersAtTarget);
            TargetGroup = null;
        };
    }

    private void EscortUpdateMsg()
    {
        //if (odenDebug) GamePlay.gpLogServer(null, "EscortUpdateMsg {0} {1}", new object[] { EscortGroup.Name(), ATK });
        if (EscortGroup == null)
            return;
        //if (odenDebug) GamePlay.gpLogServer(null, "EscortUpdateMsg {0} {1}", new object[] { GamePlay.gpPlayer().Name(), ATK });
        if (GamePlay.gpPlayer() == null)
            return;
        AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
        //if (odenDebug) GamePlay.gpLogServer(null, "EscortUpdateMsg {0} {1}", new object[] { pac.Army(), ATK });
        if (pac == null)
            return;
        if (odenDebug) GamePlay.gpLogServer(null, "EscortUpdateMsg {0} {1}", new object[] { pac.Army(), ATK });
        if (pac.Army() != EscortGroup.getArmy())
            return;

        if (EscortGroup.GetItems().Length > 1)
        {
            AiAirGroup pag = pac.AirGroup();
            AiActor EscortLeader = EscortGroup.GetItems()[0];

            string planeSector = GamePlay.gpSectorName(EscortLeader.Pos().x, EscortLeader.Pos().y);
            AiActor refActor = pag.GetItems()[0];
            Point3d refPos3d = refActor.Pos();
            int dist = (int)(EscortLeader.Pos().distance(ref refPos3d) / 1000);
            if (dist > 5)
            {
                if ((planeSector.IndexOf(">") == -1 && planeSector.IndexOf("<") == -1))
                {
                    timeoutRadio = DateTime.Now;
                    timeoutRadio.AddSeconds(12);

                    AiAircraft fc = (AiAircraft)EscortLeader;
                    fc.SayToGroup(pag, "Bomber_formation");
                    sayGrid(fc, pag, planeSector);
                    sayAltitude(fc, pag, EscortLeader.Pos().z);
                    gameIsOn = true;
                    //if (odenDebug) GamePlay.gpLogServer(null, "EscortUpdateMsg {0} {1}", new object[] { planeSector, dist });
                }
                else
                {
                    if (gameIsOn && gameWon > 0 && !EscortGroup.hasTorpedos() && !EscortGroup.hasBombs() && !EscortGroup.hasBombsInInternalBays())
                    {   // mission over
                        sayMissionOver(gameWon);
                        GamePlay.gpLogServer(null, "UpdateRTB from {0}", new object[] { "EscortUpdateMsg mission over no EscortGroup wpn" });
                        UpdateRTB(bombersAtTarget);
                        EscortGroup = null;
                    };
                };
            };
        }
        else
        {   // look for new target?
            gameWon = ATK == RAF ? LW : RAF;
            sayMissionOver(gameWon);
            GamePlay.gpLogServer(null, "UpdateRTB from {0}", new object[] { "EscortUpdateMsg mission over no EscortGroup" });
            UpdateRTB(bombersAtTarget);
            EscortGroup = null;
        };
    }

    private void sayDirection(AiAircraft ac, AiAirGroup grp, Point3d pt3d)
    {
        AiActor refActor = grp.GetItems()[0];
        int dir = (int)(calcDir(new Point2d(refActor.Pos().x, refActor.Pos().y), new Point2d(pt3d.x, pt3d.y)));
        string strDir = dir.ToString();
        for (int i = 0; i < strDir.Length; i++)
            ac.SayToGroup(grp, strDir[i].ToString());
    }

    private void sayDistance(AiAircraft ac, AiAirGroup grp, Point3d pt3d)
    {
        AiActor refActor = grp.GetItems()[0];
        Point3d refPos3d = refActor.Pos();
        int dist = (int)(pt3d.distance(ref refPos3d) / 1000);
        if (grp.Army() == RAF)
        {
            int miles = (int)(dist * .62f);
            if (miles > 5)
            {
                ac.SayToGroup(grp, "Distance"); // "Range"
                if (miles < 16) ac.SayToGroup(grp, miles.ToString() + "_miles");
                else
                {
                    int tens = 10 * (miles / 10); // and hundreds
                    sayNumber(ac, grp, tens.ToString());
                    sayNumber(ac, grp, (miles % 10).ToString() + "_miles");
                };
            };
        }
        else
        {
            if (dist > 34)
                return;
            int km = dist;
            if (km > 15) km = km - (km % 5);
            if (km > 2)
            {
                ac.SayToGroup(grp, "Distance"); // "Range"
                ac.SayToGroup(grp, km.ToString() + "_kilometers");
            };
        };
    }

    private void sayAltitude(AiAircraft ac, AiAirGroup grp, double m)
    {
        if (grp.Army() == RAF)
        {
            int angels = (int)((m + 164) / 328);
            if (angels > 0)
                ac.SayToGroup(grp, angels.ToString() + "0hi");
        }
        else
        {
            int km500 = (int)((m + 250) / 100);
            km500 = km500 - (km500 % 5);
            if (km500 > 0)
            {
                ac.SayToGroup(grp, "Height");
                ac.SayToGroup(grp, km500.ToString() + "h");
            }
            else
            {
                int m100 = (int)(m + 50 / 100);
                if (m100 > 0)
                {
                    ac.SayToGroup(grp, "Height");
                    ac.SayToGroup(grp, m100.ToString() + "h");
                }
            };
            //if (odenDebug) GamePlay.gpLogServer(null, "sayAltitude: {0} >> {1}", new object[] { m, km500.ToString() });
        };
    }

    private void sayGrid(AiAircraft ac, AiAirGroup grp, string targetSector)
    {
        ac.SayToGroup(grp, "In_square");
        string[] arrEWNS = { "vb_East_of", "vb_West_of" };
        for (int i = 0; i < targetSector.Length; i++)
        {
            switch (targetSector[i])
            {
                case ',':
                    arrEWNS[0] = "vb_North_of";
                    arrEWNS[1] = "vb_South_of";
                    break;
                case '>':
                    ac.SayToGroup(grp, arrEWNS[0]);
                    break;
                case '<':
                    ac.SayToGroup(grp, arrEWNS[1]);
                    break;
                default:
                    ac.SayToGroup(grp, targetSector[i].ToString());
                    break;
            };
        };
    }

    private void sayNumber(AiAircraft ac, AiAirGroup ag, string mixedString)
    {
        //char[] numerals = new char[]{'0','1','2','3','4','5','6','7','8','9'};
        List<string> nums = new List<string>();
        string num = "0123456789";

        //foreach (char ch in ac.Regiment().name()) //BoB_RAF_F_141Sqn_Early
        for (int i = 0; i < mixedString.Length; i++)
            if (num.Contains(mixedString[i].ToString()))
                nums.Add(mixedString[i].ToString());

        num = ""; // hundreds
        if (nums.Count == 3)
        {
            num = nums[0] + "h";
            nums.RemoveAt(0);
        };
        if (nums.Count == 2)
            switch (nums[0])
            {
                case "0":
                    nums.RemoveAt(0);
                    break;
                case "1":
                    nums[0] = nums[0] + nums[1];
                    nums.RemoveAt(1);
                    break;
                case "2":
                    if (nums[1].IndexOfAny(new char[] { '4', '5', '6', '7', '8' }) > -1)
                    {
                        nums[0] = nums[0] + nums[1];
                        nums.RemoveAt(1);
                    };
                    break;
                case "3":
                    nums[0] = nums[0] + nums[1];
                    nums.RemoveAt(1);
                    break;
                case "4": nums[0] = "n40"; break;
                case "5": nums[0] = "50s"; break;
                case "6": nums[0] = "60s"; break;
                case "7": nums[0] = "70s"; break;
                case "8": nums[0] = "80s"; break;
                case "9": nums[0] = "90s"; break;
            };

        if (num.Length > 0) ac.SayToGroup(ag, num); // hundreds
        foreach (string str in nums)
            ac.SayToGroup(ag, str);
    }

    private void sayGroundTarget(AiAircraft ac, AiAirGroup ag, Point2d pt2d)
    {
        ac.SayToGroup(ag, "Enemy_group");
        if (GamePlay.gpLandType(pt2d.x, pt2d.y) == LandTypes.WATER) ac.SayToGroup(ag, "Cargo_Ships");
        else ac.SayToGroup(ag, "Transport_column");
        ac.SayToGroup(ag, groundAssaultFrom);
        // doesnt work ac.SayToGroup(ag, "Friendlies");
        ac.SayToGroup(ag, "Next_check_point");
        sayGrid(ac, ag, GamePlay.gpSectorName(pt2d.x, pt2d.y));
    }

    private void sayBombTarget(AiAircraft ac, AiAirGroup ag, Point2d pt2d)
    {
        string targetType = "Enemy_group";
        if (convoyTarget != null) targetType = "Cargo_Ships";
        if (mainGSTarget != null) targetType = "Enemy_Position";
        if (targetAirport != null) targetType = "Enemy_Airfield";
        ac.SayToGroup(ag, targetType);
        sayGrid(ac, ag, GamePlay.gpSectorName(pt2d.x, pt2d.y));
    }

    private void sayMissionOver(int win)
    {
        timeoutRadio = DateTime.Now;
        timeoutRadio.AddSeconds(12);
        if (GamePlay.gpPlayer() == null)
            return;
        AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
        if (pac == null)
            return;
        AiAirGroup pag = pac.AirGroup();
        AiAircraft lac = (AiAircraft)pag.GetItems()[0]; // it's ok to talk to one self at times like this :)
        if (win == pag.getArmy())
        {
            switch (rnd.Next(1, 8))
            {
                case 1: lac.SayToGroup(pag, "All_right_gentlemen_I_think_that_was_quite_enough"); break;
                case 2: lac.SayToGroup(pag, "Everyone_let_s_go_home"); break;
                case 3: lac.SayToGroup(pag, "Good_show_everyone"); break;
                case 4: lac.SayToGroup(pag, "Heading_home"); break;
                case 5: lac.SayToGroup(pag, "Let_s_go_home"); break;
                case 6: lac.SayToGroup(pag, "Mission_accomplished_Excellent_Just_like_the_movies"); break;
                case 7: lac.SayToGroup(pag, "Okay_we_are_finished_here_Let_s_go_home"); break;
                case 8: lac.SayToGroup(pag, "That_was_easy_enough_Let_s_go_home"); break;
            };
        }
        else
        {
            switch (rnd.Next(1, 10))
            {
                case 1: lac.SayToGroup(pag, "Disengage_disengage_let_s_get_out_of_here"); break;
                case 2: lac.SayToGroup(pag, "Everyone_let_s_go_home"); break;
                case 3: lac.SayToGroup(pag, "Heading_home"); break;
                case 4: lac.SayToGroup(pag, "Let_s_go_home"); break;
                case 5: lac.SayToGroup(pag, "Okay_we_are_finished_here_Let_s_go_home"); break;
                case 6: lac.SayToGroup(pag, "Return_to_base_I_repeat_return_to_home_base"); break;
                case 7: lac.SayToGroup(pag, "Return_to_base_immediately"); break;
                case 8: lac.SayToGroup(pag, "Returning_to_base"); break;
                case 9: lac.SayToGroup(pag, "We_are_heading_home"); break;
                case 10: lac.SayToGroup(pag, "We_are_returning_to_base"); break;
            };
        };
    }

    private void SetOrderMissionMenuOden(Player player)
    {
        bool isSubMenu = false;
        int menuID = 0;

        PlayerData pd = PlayersData.Find(x => x.player == player);
        string[] OrderMissionMenuKeys = new string[]
	    {
            "Request Assistance"
		    ,"Leader, where are you?"
            ,"Closest Friendly Flight"
            ,"Closest Enemy Flight"
            ,"Request Vector to " +pd.homeplate.Name().Replace('_', ' ')
            ,"Select Airfield"
	    };

        bool[] OrderMissionMenuSubMenu = new bool[]
	    {
		    true
		    ,false
		    ,false
		    ,false
		    ,false
		    ,true
	    };
        if (player.Army() == LW) OrderMissionMenuKeys[5] = "Target Information";
        GamePlay.gpSetOrderMissionMenu(player, isSubMenu, menuID, OrderMissionMenuKeys, OrderMissionMenuSubMenu);
    }

    private void SetOrderMissionSubMenuRequest()
    {
        bool isSubMenu = true;
        int menuID = 1;
        string[] OrderMissionMenuKeys = SquadronsOnCAP();
        List<bool> menuActive = new List<bool>();
        foreach (string str in OrderMissionMenuKeys)
            menuActive.Add(false);
        bool[] OrderMissionMenuSubMenu = menuActive.ToArray();

        GamePlay.gpSetOrderMissionMenu(GamePlay.gpPlayer(), isSubMenu, menuID, OrderMissionMenuKeys, OrderMissionMenuSubMenu);
    }

    private string[] SquadronsOnCAP()
    {
        List<string> result = new List<string>();

        // CAP
        AiAirGroupsCAP.Clear();
        //foreach (AiAirGroup ag in GamePlay.gpAirGroups(GamePlay.gpPlayer().Army()))
        foreach (AiAirGroup ag in AiAirGroupsStdby)
            if (ag.Army() == GamePlay.gpPlayer().Army())
                if ((AiGroup)ag != GamePlay.gpPlayer().Place().Group())
                    foreach (MissionAirbase ab in MissionAirbases)
                        if (ag.Name().Contains(ab.sqn) && !ab.name.Contains(homeBase.name))
                        {
                            if (ab.army == RAF) result.Add("RAF " + ab.name + " " + ab.airportName.Replace('_', ' ') + ", " + Designation(ab.aircraftType));
                            else result.Add(ab.name + " " + ab.airportName.Replace('_', ' ') + ", " + Designation(ab.aircraftType));
                            AiAirGroupsCAP.Add(ag);
                            break;
                        };

        // sweep
        foreach (AiAirGroup ag in GamePlay.gpAirGroups(GamePlay.gpPlayer().Army()))
            if (sweepGroups.Contains(ag.Name()))
                foreach (MissionAirbase ab in MissionAirbases)
                    if (ag.Name().Contains(ab.sqn) && !ab.name.Contains(homeBase.name))
                    {
                        if (ab.army == RAF) result.Add("RAF " + ab.name + " " + ab.airportName.Replace('_', ' ') + ", " + Designation(ab.aircraftType));
                        else result.Add(ab.name + " " + ab.airportName.Replace('_', ' ') + ", " + Designation(ab.aircraftType));
                        AiAirGroupsCAP.Add(ag);
                        break;
                    };

        return result.ToArray();
    }

    private void SetOrderMissionSubMenuAirfields(Player player)
    {
        bool isSubMenu = true;
        int menuID = 2;
        string[] OrderMissionMenuKeys = PlayerPossibleAirports(player);
        List<bool> menuActive = new List<bool>();
        foreach (string str in OrderMissionMenuKeys)
            menuActive.Add(false);
        bool[] OrderMissionMenuSubMenu = menuActive.ToArray();

        GamePlay.gpSetOrderMissionMenu(player, isSubMenu, menuID, OrderMissionMenuKeys, OrderMissionMenuSubMenu);
    }

    private string[] PlayerPossibleAirports(Player player)
    {
        List<string> result = new List<string>();

        AiActor actor = player.Place();
        Point3d playerPos3d = actor.Pos();
        Point2d playerPos2d = new Point2d(playerPos3d.x, playerPos3d.y);
        // AiAirports
        PossibleAirports.Clear();
        List<AiAirport> saps = SortedAirports(player.Army(), playerPos2d, false);
        //GamePlay.gpLogServer(null, "PlayerPossibleAirports: {0} from {1}", new object[] { saps.Count, GamePlay.gpAirports().Length });
        foreach (AiAirport ap in saps)
        {
            result.Add(ap.Name().Replace('_', ' '));
            PossibleAirports.Add(ap);
            if (result.Count > 7)
                break;
        };
        //GamePlay.gpLogServer(null, "PlayerPossibleAirports: {0} to {1}", new object[] { saps.Count, PossibleAirports.Count });

        return result.ToArray();
    }

    public override void OnOrderMissionMenuSelected(Player player, int ID, int menuItemIndex)
    {
        base.OnOrderMissionMenuSelected(player, ID, menuItemIndex);

        AiAircraft fc = FighterCommand;

        switch (ID)
        {
            case 0:
                AiActor actor = player.Place();
                Point3d playerPos3d = actor.Pos();
                Point2d playerPos2d = new Point2d(playerPos3d.x, playerPos3d.y);
                double mag_compass = 0;
                AiAirGroup playerGroup = (actor as AiAircraft).AirGroup();
                string playerSector = GamePlay.gpSectorName(actor.Pos().x, actor.Pos().y);
                AiActor leader = playerGroup.GetItems()[0];
                AiAircraft ac;

                switch (menuItemIndex)
                {
                    case 0:
                        SetOrderMissionMenuOden(player);
                        break;
                    case 1: // "Request Fighter Support"
                        if (AiAirGroupsStdby.Count > 0) SetOrderMissionSubMenuRequest();
                        else SetOrderMissionMenuOden(player);
                        break;
                    case 2: //"Leader, where are you?"
                        timeoutRadio = DateTime.Now;
                        timeoutRadio.AddSeconds(12);
                        //(actor as AiAircraft).SayToGroup(playerGroup, (leader as AiAircraft).CallSign()); // Leader
                        //(actor as AiAircraft).SayToGroup(playerGroup, "n_");
                        (actor as AiAircraft).SayToGroup(playerGroup, "Where_are_you");
                        //(actor as AiAircraft).SayToGroup(playerGroup, "Over");
                        if (leader == null)
                            return;
                        ac = (AiAircraft)leader;
                        string leaderSector = GamePlay.gpSectorName(ac.Pos().x, ac.Pos().y);
                        Timeout(5, () =>
                        {
                            ac.SayToGroup(playerGroup, ac.CallSign());
                            ac.SayToGroup(playerGroup, "n1"); // Leader
                            sayGrid(ac, playerGroup, leaderSector);
                            sayAltitude(ac, playerGroup, leader.Pos().z);
                            if (TargetGroup == null && EscortGroup == null && gameIsOn)
                                sayMissionOver(gameWon);
                            int angels = (int)((actor.Pos().z + 164) / 328);
                            if (player.Army() == RAF) GamePlay.gpHUDLogCenter(new Player[] { player }, "Flight is in [" + leaderSector + "] at Angels " + angels.ToString());
                            else GamePlay.gpHUDLogCenter(new Player[] { player }, "Flight is in [" + leaderSector + "] at " + actor.Pos().z.ToString("F0") + "m");

                        });
                        break;
                    case 3: // "Closest Friendly Flight"
                        if (player.Army() == LW)
                            if (EscortGroup == null) return;
                            else
                                if (EscortGroup.GetItems().Length > 0) fc = (AiAircraft)EscortGroup.GetItems()[0];
                                else return;

                        timeoutRadio = DateTime.Now;
                        timeoutRadio.AddSeconds(12);
                        //(actor as AiAircraft).SayToGroup(playerGroup, "callsign1");
                        (actor as AiAircraft).SayToGroup(playerGroup, "Vector");
                        (actor as AiAircraft).SayToGroup(playerGroup, "Friendlies");
                        (actor as AiAircraft).SayToGroup(playerGroup, "Over");
                        Timeout(5, () =>
                        {
                            List<AiAircraft> sacs = SortedAircrafts((actor.Army() == RAF) ? LW : RAF, playerPos3d);
                            if (sacs.Count > 0)
                            {
                                ac = sacs[1];
                                if (ac.IsValid() && ac.IsAlive())
                                {
                                    //string planeType = bobDesignation((actorPlane as AiAircraft).InternalTypeName());
                                    fc.SayToGroup(playerGroup, "Ground_control_here_Stand_by");
                                    //if (actorPlane is AiAircraft) mag_compass = (actorPlane as AiAircraft).getParameter(part.ParameterTypes.I_MagneticCompass, 0); 
                                    fc.SayToGroup(playerGroup, "Friendlies");
                                    fc.SayToGroup(playerGroup, "Direction"); // "Bearing"
                                    sayDirection(fc, playerGroup, ac.Pos());
                                    sayDistance(fc, playerGroup, ac.Pos());
                                    /*angels = (int)((ac.Pos().z + 164) / 328);
                                    if (angels > 0)
                                    {
                                        //fc.SayToGroup(localGroup, "Height");
                                        fc.SayToGroup(playerGroup, angels.ToString() + "0hi");
                                    };
                                    */
                                    sayAltitude(fc, playerGroup, leader.Pos().z);

                                    string sn = GamePlay.gpSectorName(ac.Pos().x, ac.Pos().y);
                                    AiActor refActor = playerGroup.GetItems()[0];
                                    Point3d refPos3d = refActor.Pos();
                                    int dir = (int)(calcDir(new Point2d(refActor.Pos().x, refActor.Pos().y), new Point2d(ac.Pos().x, ac.Pos().y)));
                                    double dist = (int)(ac.Pos().distance(ref refPos3d) / 1000);
                                    int miles = (int)(dist * .62f);
                                    int angels = (int)((ac.Pos().z + 164) / 328);
                                    if (player.Army() == RAF) GamePlay.gpHUDLogCenter(new Player[] { player }, "Nearest Friendly Flight Bearing " + dir.ToString() + ", " + miles.ToString() + " miles at Angels " + angels.ToString() + " [" + sn + "]");
                                    else GamePlay.gpHUDLogCenter(new Player[] { player }, "Nearest Friendly Flight Bearing " + dir.ToString() + ", " + dist.ToString("F0") + "km at " + ac.Pos().z.ToString("F0") + "m" + " [" + sn + "]");

                                }
                                else { fc.SayToGroup(playerGroup, "Return_to_base_immediately"); };
                            }
                            else { fc.SayToGroup(playerGroup, "Return_to_base_immediately"); };
                        });
                        break;
                    case 4: // "Closest Enemy Flight"
                        if (player.Army() == LW)
                            if (EscortGroup == null) return;
                            else
                                if (EscortGroup.GetItems().Length > 0) fc = (AiAircraft)EscortGroup.GetItems()[0];
                                else return;

                        timeoutRadio = DateTime.Now;
                        timeoutRadio.AddSeconds(12);
                        //(actor as AiAircraft).SayToGroup(playerGroup, "callsign1");
                        (actor as AiAircraft).SayToGroup(playerGroup, "Vector");
                        (actor as AiAircraft).SayToGroup(playerGroup, "Enemy_planes");
                        (actor as AiAircraft).SayToGroup(playerGroup, "Over");
                        Timeout(10, () =>
                        {
                            List<AiAircraft> sacs = SortedAircrafts((actor.Army() == RAF) ? LW : RAF, playerPos3d);
                            if (sacs.Count > 0)
                            {
                                ac = sacs[0];
                                if (ac.IsValid() && ac.IsAlive())
                                {
                                    fc.SayToGroup(playerGroup, "Ground_control_here_Stand_by");
                                    fc.SayToGroup(playerGroup, "Enemy_group");
                                    //fc.SayToGroup(playerGroup, "Direction"); // "Bearing"
                                    sayDirection(fc, playerGroup, ac.Pos());
                                    sayDistance(fc, playerGroup, ac.Pos());
                                    sayAltitude(fc, playerGroup, leader.Pos().z);

                                    string sn = GamePlay.gpSectorName(ac.Pos().x, ac.Pos().y);
                                    AiActor refActor = playerGroup.GetItems()[0];
                                    Point3d refPos3d = refActor.Pos();
                                    int dir = (int)(calcDir(new Point2d(refActor.Pos().x, refActor.Pos().y), new Point2d(ac.Pos().x, ac.Pos().y)));
                                    double dist = (int)(ac.Pos().distance(ref refPos3d) / 1000);
                                    int miles = (int)(dist * .62f);
                                    int angels = (int)((ac.Pos().z + 164) / 328);
                                    if (player.Army() == RAF) GamePlay.gpHUDLogCenter(new Player[] { player }, "Nearest Enemy Flight Bearing " + dir.ToString() + ", " + miles.ToString() + " miles at Angels " + angels.ToString() + " [" + sn + "]");
                                    else GamePlay.gpHUDLogCenter(new Player[] { player }, "Nearest Enemy Flight Bearing " + dir.ToString() + ", " + dist.ToString("F0") + "km at " + ac.Pos().z.ToString("F0") + "m" + " [" + sn + "]");

                                    if (playerGroup.isAircraftType(AircraftType.Fighter)
                                        || playerGroup.isAircraftType(AircraftType.BNZFighter)
                                        || playerGroup.isAircraftType(AircraftType.HeavyFighter)
                                        || playerGroup.isAircraftType(AircraftType.TNBFighter))
                                    {   // assign as target if none or new is closer
                                        if (TargetGroup == null)
                                            TargetGroup = ac.AirGroup();
                                        else
                                        {
                                            AiAirGroup ag = ac.AirGroup();
                                            if (ag.hasTorpedos() || ag.hasBombs() || ag.hasBombsInInternalBays())
                                            {
                                                if (ag.Pos().distance(ref playerPos3d) < TargetGroup.Pos().distance(ref playerPos3d))
                                                    TargetGroup = ac.AirGroup();
                                            };
                                        };
                                    };
                                }
                                else { fc.SayToGroup(playerGroup, "Mission_accomplished"); };
                            }
                            else { fc.SayToGroup(playerGroup, "Mission_accomplished"); };
                        });
                        break;
                    case 5: // "Request Vector to Airfield"
                        (actor as AiAircraft).SayToGroup(playerGroup, (actor as AiAircraft).CallSign());
                        (actor as AiAircraft).SayToGroup(playerGroup, "Check");
                        (actor as AiAircraft).SayToGroup(playerGroup, "Vector");
                        (actor as AiAircraft).SayToGroup(playerGroup, "Course");
                        Timeout(4, () => { fc.SayToGroup(playerGroup, "Ground_control_here_Stand_by"); });
                        Timeout(8, () =>
                        {
                            fc.SayToGroup(playerGroup, "Vector_to_base_is");
                            sayDirection(fc, playerGroup, homePlate.Pos());
                            sayDistance(fc, playerGroup, homePlate.Pos());

                            AiActor refActor = playerGroup.GetItems()[0];
                            Point3d refPos3d = refActor.Pos();
                            int dir = (int)(calcDir(new Point2d(refActor.Pos().x, refActor.Pos().y), new Point2d(homePlate.Pos().x, homePlate.Pos().y)));
                            double dist = (int)(homePlate.Pos().distance(ref refPos3d) / 1000);
                            int miles = (int)(dist * .62f);
                            if (player.Army() == RAF) GamePlay.gpHUDLogCenter(new Player[] { player }, homePlate.Name().Replace('_', ' ') + " is Bearing " + dir.ToString() + ", " + miles.ToString() + " miles out");
                            else GamePlay.gpHUDLogCenter(new Player[] { player }, homePlate.Name().Replace('_', ' ') + " is Bearing " + dir.ToString() + ", " + dist.ToString("F0") + "km out");

                        });
                        break;
                    case 6: // "Select Airfield" for RAF, Target Info for LW
                        if (player.Army() == RAF)
                        {
                            SetOrderMissionSubMenuAirfields(player);
                        }
                        else
                        {
                            GamePlay.gpHUDLogCenter(new Player[] { player }, targetName);
                        };
                        break;
                };
                break;
            case 1: // "Request Fighter Support"
                switch (menuItemIndex)
                {
                    case 0:
                        SetOrderMissionMenuOden(player);
                        break;
                    default://
                        if (menuItemIndex < AiAirGroupsCAP.Count) requestAssistance(menuItemIndex, true);
                        SetOrderMissionMenuOden(player);
                        break;
                };
                break;
            case 2: // "Select Airfield"
                switch (menuItemIndex)
                {
                    case 0:
                        SetOrderMissionMenuOden(player);
                        break;
                    default:
                        if (menuItemIndex < PossibleAirports.Count) SetTempHomebase(player, menuItemIndex, true);
                        SetOrderMissionMenuOden(player);
                        break;
                };
                break;
            default:
                SetOrderMissionMenuOden(player);
                break;
        };
    }

    private void requestAssistance(int menuItemIndex, bool sayRequest)
    {
        if (AiAirGroupsCAP.Count < 1)
            return;
        AiAircraft fc = FighterCommand;
        if (fc == null)
            return;
        AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
        if (pac == null)
            return;
        AiAirGroup pag = pac.AirGroup();
        timeoutRadio = DateTime.Now;
        timeoutRadio.AddSeconds(30);

        if (sayRequest) pac.SayToGroup(pag, "We_are_under_attack_we_need_assistance_over");
        else SquadronsOnCAP();

        Timeout(4, () => { fc.SayToGroup(pag, "Ground_control_here_Stand_by"); });

        Point2d pt2d = new Point2d(pac.Pos().x, pac.Pos().y);
        if (menuItemIndex > AiAirGroupsCAP.Count)
            Timeout(8, () => { fc.SayToGroup(pag, "Negative_all_available_fighters_are_busy_at_this_time_Sorry"); });
        else
        {
            AiAirGroup agSup = AiAirGroupsCAP[menuItemIndex - 1];
            if (agSup == null)
                Timeout(8, () => { fc.SayToGroup(pag, "Negative_all_available_fighters_are_busy_at_this_time_Sorry"); });
            else
            {
                GiveNewWP(agSup, pt2d);
                AiAirGroupsStdby.Remove(agSup);
                Timeout(8, () => { fc.SayToGroup(pag, "We_are_sending_fighters_to_your_location_Hang_on"); });
                Timeout(12, () =>
                {
                    AiAircraft ac = (AiAircraft)agSup.GetItems()[0];
                    ac.SayToGroup(pag, "This_Is_Commander");
                    sayNumber(ac, pag, ac.Regiment().name());
                    //ac.SayToGroup(pag, ac.Regiment().speech()); //"Squadron");
                    //ac.SayToGroup(pag, "Sqaudron");
                    //ac.SayToGroup(pag, "parts\\bob\\speech\\gb\\Actor1\\Squadron.ogg");
                    ac.SayToGroup(pag, "1_Flight");
                    ac.SayToGroup(pag, "We_re_coming_in_to_assist");
                });
            };
        };
    }

    private void SetTempHomebase(Player player, int menuItemIndex, bool sayRequest)
    {
        //GamePlay.gpLogServer(null, "SetTempHomebase: {0} at {1}", new object[] { PossibleAirports.Count, menuItemIndex });
        PlayerData pd = PlayersData.Find(x => player == x.player);
        pd.homeplate = PossibleAirports[menuItemIndex - 1];
        homePlate = pd.homeplate;
        List<Player> players = AllPlayers(player.Army());
        foreach (Player p in players)
            SetOrderMissionMenuOden(p);
        ArmyHUDLogCenter(player.Army(), "New homebase: " + homePlate.Name().Replace('_', ' '));
    }

    private void ReadMissionFile(ISectionFile missFile)
    {
        if (odenDebug) GamePlay.gpLogServer(null, "ReadMissionFile:  {0} ({1})", new object[] { MissionMarkers.Count, "Start" });
        int n = missFile.lines("FrontMarker");
        for (int i = 0; i < n; i++)
        {
            string key = "FrontMarker" + i.ToString();
            if (missFile.exist("FrontMarker", key))
            {	//FrontMarker0 287950.90 85215.27 2
                string value = missFile.get("FrontMarker", key);
                string[] strs = value.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                if (strs.Length == 3)
                {	//double.TryParse(s, System.Globalization.NumberStyles.Any, rts.RTSConf.enCultureInfo, out res)
                    MissionMarker mm = new MissionMarker(0, 0, 0, key);
                    double.TryParse(strs[0], System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture.NumberFormat, out mm.x);
                    double.TryParse(strs[1], System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture.NumberFormat, out mm.y);
                    int.TryParse(strs[2], out mm.army);
                    MissionMarkers.Add(mm);
                    //if (odenDebug) GamePlay.gpLogServer(null, "ReadMissionFile:  {0} ({1})", new object[] { MissionMarkers.Count, "" });
                };
            };
        };

        if (!missFile.exist("DYNMIS", "DynMisPlayer"))
        {
            if (missFile.exist("MAIN", "player"))
            {
                defPlayer = missFile.get("MAIN", "player");
                defSquadron = defPlayer.Substring(0, defPlayer.IndexOf("."));
                string sect = defPlayer.Substring(defPlayer.IndexOf(".") + 1, 1); // identifies AirGroup
                for (int a = 0; a < 10; a++)
                    if (missFile.exist("AirGroups", defSquadron + "." + sect + a.ToString()))
                    {
                        sect = defSquadron + "." + sect + a.ToString();
                        break;
                    };

                defClass = missFile.get(sect, "Class");
                defCallSign = missFile.get(sect, "CallSign");
                string slot = defPlayer.Substring(defPlayer.IndexOf(".") + 2); // 001 for 2nd slot first section, 010 for first slot 2nd section, 101 2nd slot second section, 000 first-first
                // slot is now 2 chars, 00, 01, 02 from first section or 10 11 12 from 2nd section (talking 3-ship sections)
                if (missFile.exist(sect, "Skin" + slot))
                    defSkin = missFile.get(sect, "Skin" + slot);
                if (missFile.exist(sect, "Serial" + slot))
                    defSerial = missFile.get(sect, "Serial" + slot);
                defTail = missFile.get(sect, "Flight" + slot[0].ToString());
                string[] strs = defTail.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                defTail = strs[int.Parse(slot[1].ToString())]; // Flight1 4 5 6 > defTail is now "4" denoting 'D' marker on plane
                if (defSquadron.Contains("_Early"))
                    defSquadron = defSquadron.Remove(defSquadron.IndexOf("_Early"));
            }
            else GamePlay.gpHUDLogCenter("This mission has no player slot");
        }
        else
        {
            defPlayer = missFile.get("DYNMIS", "DynMisPlayer");
            defSquadron = missFile.get("DYNMIS", "DynMisSquadron");
            defClass = missFile.get("DYNMIS", "DynMisClass");
            defCallSign = missFile.get("DYNMIS", "DynMisCallSign");
            defSkin = missFile.get("DYNMIS", "DynMisSkin");
            defSerial = missFile.get("DYNMIS", "DynMisSerial");
            defTail = missFile.get("DYNMIS", "DynMisTail");
            if (odenDebug) GamePlay.gpLogServer(null, "DYNMIS: {0} {1} {2} {3} {4}", new object[] { defPlayer, defSquadron, defClass, defCallSign, defSerial });
        };
        string abc = " ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        int pos = int.Parse(defTail);
        //tailNoSerialNo = "(verify tailnumber 'D' and serialno. 'R6708' in <Player Plane>)";        
        if (defClass.Contains("109") || defPlayer.Contains("RA_")) tailNoSerialNo = "Verify tailnumber '" + defTail + "', serialno. '" + defSerial + "' and";
        else tailNoSerialNo = "Verify tailnumber '" + abc[pos].ToString() + "', serialno. '" + defSerial + "' and";

        string subType = "";
        if (defSquadron.Contains("RAF_")) subType = defSquadron.Substring(10); // BoB_RAF_F_264Sqn >> 264Sqn same goes for BoB_RAF_B_123Sqn
        if (defSquadron.Contains("LW_")) subType = defSquadron.Substring(7); // BoB_LW_KG2_I >> KG2_I
        if (defSquadron.Contains("RA_")) subType = defSquadron.Substring(7); // BoB_RA_56St_20Gruppo_353Sq >> 56St_20Gruppo_353Sq
        subType = subType.Replace("_", " ");
        if (defSquadron.Contains("RAF_"))
            missionCaption = "RAF Fighter Command " + subType + " (" + vName + ")";
        if (defSquadron.Contains("LW_"))
            missionCaption = "Luftwaffe " + subType + " (" + vName + ")"; // will add gruppe/JG later
        if (defSquadron.Contains("RA_"))
            missionCaption = "Corpo Aereo Italiano " + subType + " (" + vName + ")";
    }

    private void SetMissionMarkerArmy(string markerName, int army)
    {
        foreach (MissionMarker mm in MissionMarkers)
            if (mm.key == markerName)
            {
                mm.army = army;
                break;
            };
    }

    private void CheckFiles()
    {
        string myDocPath = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
        DoverFolder = myDocPath + @"\1C SoftClub\il-2 sturmovik cliffs of dover";
        if (!Directory.Exists(DoverFolder))
        {
            GamePlay.gpHUDLogCenter("Cannot locate '1C SoftClub' folder");
            return;
        };
        if (odenDebug) GamePlay.gpHUDLogCenter("'1C SoftClub' folder is there");
        //string[] filenames = Directory.GetFiles(DoverFolder, vFile +".*", SearchOption.AllDirectories);
        string subType = "";
        if (defSquadron.Contains("RAF_")) subType = defSquadron.Substring(10); // BoB_RAF_F_264Sqn >> 264Sqn same goes for BoB_RAF_B_123Sqn
        if (defSquadron.Contains("LW_")) subType = defSquadron.Substring(7); // BoB_LW_KG2_I >> KG2_I
        if (defSquadron.Contains("RA_")) subType = defSquadron.Substring(7); // BoB_RA_56St_20Gruppo_353Sq >> 56St_20Gruppo_353Sq
        string[] filenames = Directory.GetFiles(DoverFolder, vFile + subType + ".*", SearchOption.AllDirectories);
        foreach (string str in filenames)
        {
            //if (odenDebug) GamePlay.gpLogServer(null, "Found: {0}", new object[] { str });
            //if (str.Contains(vFile +".mis")) CampMisFile = str;
            //if (str.Contains(vFile +".bri")) CampBriFile = str;
            if (str.Contains(vFile + subType + ".mis"))
            {
                CampMisFile = str;
                CampBriFile = CampMisFile.Replace(".mis", ".briefing");
            };
        };
        string MisFolder = CampMisFile.Substring(0, CampMisFile.LastIndexOf('\\'));
        if (odenDebug) GamePlay.gpLogServer(null, "MisFolder: {0}", new object[] { MisFolder });
        CampInitFile = MisFolder + "\\" + vFile + subType + ".ini";
        /*
        if (odenDebug)
            if (File.Exists(CampInitFile))
                File.Delete(CampInitFile);
        */
        if (!File.Exists(CampInitFile))
            CreateInitFiles();
    }

    private void CreateInitFiles()
    {
        string[] strDynCamp = { "//Battle Of Britain - Default setting July 10th 1940"
                            , "//Dynamic Mission by theOden"
                            , "//This file keeps track of units etc. and may carefully be edited manually"
                            , "//Entries can be in any order"
                            , "//AirGroup,Sector,Squadron/Gruppe,Quantity,Airportname,AircraftType - all CliffsOfDover data"
                            , "//AirportDamage,Airportname,Damage - all CliffsOfDover data (GamePlay.gpAirports())"
                            , "//GndStatDead,StaticName - all CliffsOfDover data as per mis-file [Stationary]"
                            , "//GndGrpDead,GroundGroupName - all CliffsOfDover data as per mis-file [Chiefs]"
                            , "//Event,add/remove/move/newplanetype,date as '16 August 1940',AirGroup,{newAirport/newPlaneType}"
                            , "//Player Squadron must be edited in mission editor"
                            , "//-"
                            , "CampaignDate,10 July 1940"
                            , "ReinforcementDate,13 July 1940"
                            , "highOctaneDate,10 July 1940"
                            };
        foreach (string str in strDynCamp)
            strDynCampList.Add(str);

        // RAF: July10 -----------------------------------
        // 12 Group
        // RAF Duxford -  airstart north entry point
        strDynCampList.Add("AirGroup,RAF Duxford,BoB_RAF_F_264Sqn,12,Duxford,Aircraft.DefiantMkI"); // S Cambridge
        strDynCampList.Add("AirGroup,RAF Duxford,BoB_RAF_F_19Sqn,12,Fowlmere,Aircraft.SpitfireMkIa_100oct"); // S Cambridge
        // RAF Coltishall -  airstart north entry point
        strDynCampList.Add("AirGroup,RAF Coltishall,BoB_RAF_F_66Sqn,12,Coltishall,Aircraft.SpitfireMkIa_100oct"); // N Norwich
        strDynCampList.Add("AirGroup,RAF Coltishall,BoB_RAF_F_242Sqn,12,Coltishall,Aircraft.HurricaneMkI_100oct");
        // skipping RAF Kirton-in-Lindsay, RAF Digby
        // RAF Wittering, NW Cambridge, E Norwich -  airstart north entry point
        strDynCampList.Add("AirGroup,RAF Wittering,BoB_RAF_F_229Sqn,12,Wittering,Aircraft.HurricaneMkI_100oct");
        strDynCampList.Add("AirGroup,RAF Wittering,BoB_RAF_F_266Sqn,12,Wittering,Aircraft.SpitfireMkIa_100oct");
        strDynCampList.Add("AirGroup,RAF Wittering,BoB_RAF_F_23Sqn,12,Collyweston,Aircraft.BlenheimMkIVF");
        // skipping RAF Church Fenton (NE Leeds)
        // skipping 13 Group

        // 11 Group
        // RAF Biggin Hill
        strDynCampList.Add("AirGroup,RAF Biggin Hill,BoB_RAF_F_32Sqn,12,Biggin_Hill,Aircraft.HurricaneMkI_100oct");
        strDynCampList.Add("AirGroup,RAF Biggin Hill,BoB_RAF_F_141Sqn,12,Biggin_Hill,Aircraft.DefiantMkI");
        strDynCampList.Add("AirGroup,RAF Biggin Hill,BoB_RAF_F_610Sqn,12,Gravesend,Aircraft.SpitfireMkIa_100oct");
        strDynCampList.Add("AirGroup,RAF Biggin Hill,BoB_RAF_F_600Sqn,12,Manston,Aircraft.BlenheimMkIVF");
        strDynCampList.Add("AirGroup,RAF Biggin Hill,BoB_RAF_F_1Sqn,12,Hawkinge,Aircraft.HurricaneMkI_100oct"); // gameplay (RAF Northolt)
        // RAF North Weald
        strDynCampList.Add("AirGroup,RAF North Weald,BoB_RAF_F_56Sqn,12,North_weald,Aircraft.HurricaneMkI_100oct");
        strDynCampList.Add("AirGroup,RAF North Weald,BoB_RAF_F_151Sqn,12,North_weald,Aircraft.HurricaneMkI_100oct");
        strDynCampList.Add("AirGroup,RAF North Weald,BoB_RAF_F_85Sqn,12,Martlesham,Aircraft.HurricaneMkI_100oct"); // airstart north entry point
        strDynCampList.Add("AirGroup,RAF North Weald,BoB_RAF_F_25Sqn,12,Martlesham,Aircraft.BlenheimMkIVF");
        // RAF Kenley
        strDynCampList.Add("AirGroup,RAF Kenley,BoB_RAF_F_64Sqn,12,Kenley,Aircraft.SpitfireMkIa_100oct");
        strDynCampList.Add("AirGroup,RAF Kenley,BoB_RAF_F_615Sqn,12,Kenley,Aircraft.HurricaneMkI_100oct");
        strDynCampList.Add("AirGroup,RAF Kenley,BoB_RAF_F_111Sqn,12,Croydon,Aircraft.HurricaneMkI_100oct");
        strDynCampList.Add("AirGroup,RAF Kenley,BoB_RAF_F_501Sqn,12,Croydon,Aircraft.HurricaneMkI_100oct");
        // RAF Hornchurch
        strDynCampList.Add("AirGroup,RAF Hornchurch,BoB_RAF_F_65Sqn,12,Hornchurch,Aircraft.SpitfireMkIa_100oct");
        strDynCampList.Add("AirGroup,RAF Hornchurch,BoB_RAF_F_74Sqn,12,Hornchurch,Aircraft.SpitfireMkIa_100oct");
        strDynCampList.Add("AirGroup,RAF Hornchurch,BoB_RAF_F_54Sqn,12,Rochford,Aircraft.SpitfireMkIa_100oct"); // Rochford
        // RAF Tangmere
        strDynCampList.Add("AirGroup,RAF Tangmere,BoB_RAF_F_43Sqn,12,Tangmere,Aircraft.HurricaneMkI_100oct");
        strDynCampList.Add("AirGroup,RAF Tangmere,BoB_RAF_F_145Sqn,12,Tangmere,Aircraft.HurricaneMkI_100oct");
        strDynCampList.Add("AirGroup,RAF Tangmere,BoB_RAF_F_601Sqn,12,Tangmere,Aircraft.HurricaneMkI_100oct");
        strDynCampList.Add("AirGroup,RAF Tangmere,BoB_RAF_F_219Sqn,12,Lympne,Aircraft.BeaufighterMkIF"); // gameplay
        // RAF Debden
        strDynCampList.Add("AirGroup,RAF Debden,BoB_RAF_F_17Sqn,12,North_weald,Aircraft.HurricaneMkI_100oct"); // Debden
        // RAF Northolt
        strDynCampList.Add("AirGroup,RAF Northolt,BoB_RAF_F_604Sqn,12,Northolt,Aircraft.BeaufighterMkIF");
        strDynCampList.Add("AirGroup,RAF Northolt,BoB_RAF_F_257Sqn,12,Northolt,Aircraft.HurricaneMkI_100oct");

        // 10 Group
        // skipping RAF Filton
        // RAF Middle Wallop
        strDynCampList.Add("AirGroup,RAF Middle Wallop,BoB_RAF_F_609Sqn,12,Andover,Aircraft.SpitfireMkIa_100oct"); // Middle Wallop not present
        strDynCampList.Add("AirGroup,RAF Middle Wallop,BoB_RAF_F_238Sqn,12,Andover,Aircraft.HurricaneMkI_100oct");

        // Coastal Command - Thorney_Island
        strDynCampList.Add("AirGroup,RAF Thorney_Island,BoB_RAF_F_58Sqn,12,Thorney_Island,Aircraft.BlenheimMkIV"); // use 58 as 59Sqn
        strDynCampList.Add("AirGroup,RAF Thorney_Island,BoB_RAF_F_236Sqn,12,Thorney_Island,Aircraft.BlenheimMkIVF");

        // Bomber Command
        // No. 2 Group RAF Wyton (NE London), Blenheim MkI and MkIV
        //strDynCampList.Add("AirGroup,RAF Wyton,BoB_RAF_B_18Sqn,12,Upper_Heyford,Aircraft.BlenheimMkI");
        //strDynCampList.Add("AirGroup,RAF Wyton,BoB_RAF_B_57Sqn,12,Upper_Heyford,Aircraft.BlenheimMkI");
        strDynCampList.Add("AirGroup,RAF Wyton,BoB_RAF_B_21Sqn,12,Watton,Aircraft.BlenheimMkI");
        strDynCampList.Add("AirGroup,RAF Wyton,BoB_RAF_B_82Sqn,12,Watton,Aircraft.BlenheimMkI");
        strDynCampList.Add("AirGroup,RAF Wyton,BoB_RAF_B_90Sqn,12,West_Raynham,Aircraft.BlenheimMkIV");
        strDynCampList.Add("AirGroup,RAF Wyton,BoB_RAF_B_101Sqn,12,West_Raynham,Aircraft.BlenheimMkIV");
        strDynCampList.Add("AirGroup,RAF Wyton,BoB_RAF_B_114Sqn,12,Wyton,Aircraft.BlenheimMkIV");
        strDynCampList.Add("AirGroup,RAF Wyton,BoB_RAF_B_139Sqn,12,Wyton,Aircraft.BlenheimMkIV");
        strDynCampList.Add("AirGroup,RAF Wyton,BoB_RAF_B_107Sqn,12,Wattisham,Aircraft.BlenheimMkIV");
        strDynCampList.Add("AirGroup,RAF Wyton,BoB_RAF_B_110Sqn,12,Wattisham,Aircraft.BlenheimMkIV");
        //No. 3 Group RAF Mildenhall (NNE London), Wellington
        strDynCampList.Add("AirGroup,RAF Mildenhall,BoB_RAF_B_10Sqn,12,Dishforth,Aircraft.WellingtonMkIc");
        strDynCampList.Add("AirGroup,RAF Mildenhall,BoB_RAF_B_78Sqn,12,Dishforth,Aircraft.WellingtonMkIc");
        strDynCampList.Add("AirGroup,RAF Mildenhall,BoB_RAF_B_77Sqn,12,Driffield,Aircraft.WellingtonMkIc");
        strDynCampList.Add("AirGroup,RAF Mildenhall,BoB_RAF_B_102Sqn,12,Driffield,Aircraft.WellingtonMkIc");
        strDynCampList.Add("AirGroup,RAF Mildenhall,BoB_RAF_B_51Sqn,12,Linton-on-Ouse,Aircraft.WellingtonMkIc");
        strDynCampList.Add("AirGroup,RAF Mildenhall,BoB_RAF_B_58Sqn,12,Linton-on-Ouse,Aircraft.WellingtonMkIc");
        // skipping No. 5 Group Lincolnshire (offmap NE coast), Hampden (later Lancaster)

        // LuftWaffe: Aug 13 -----------------------------------
        //I. Fliegerkorps, Beauvais, LF2
        //strDynCampList.Add("AirGroup,BoB_LW_KG1_Stab,4,Amiens_Glisy,Aircraft.He-111H-2"); // Jumo211
        strDynCampList.Add("AirGroup,I. Fliegerkorps,BoB_LW_KG1_I,23,Montdidier,Aircraft.He-111P-2"); //DB601A
        strDynCampList.Add("AirGroup,I. Fliegerkorps,BoB_LW_KG1_II,29,Amiens_Glisy,Aircraft.He-111H-2"); // Jumo211
        strDynCampList.Add("AirGroup,I. Fliegerkorps,BoB_LW_KG1_III,15,Rosieres_en_Santerre,Aircraft.He-111H-2");

        //strDynCampList.Add("AirGroup,BoB_LW_KG76_Stab,5,Pontoise/Cormeilles,Aircraft.Do-17Z-2"); //1000hp BMW Bramo 323P Fafnir, 1000kg
        strDynCampList.Add("AirGroup,I. Fliegerkorps,BoB_LW_KG76_I,29,Beauvais_Tille,Aircraft.Do-17Z-1"); //900hp BMW Bramo 323A-1 Fafnir, 500kg
        //strDynCampList.Add("AirGroup,I. Fliegerkorps,BoB_LW_KG76_II,28,Creil,Aircraft.Ju-88A-1");
        strDynCampList.Add("AirGroup,I. Fliegerkorps,BoB_LW_KG76_II,28,Merville_Calonne,Aircraft.Ju-88A-1"); // for playability - Creil is rather far away
        strDynCampList.Add("AirGroup,I. Fliegerkorps,BoB_LW_KG76_III,19,Pontoise/Cormeilles,Aircraft.Do-17Z-1");

        strDynCampList.Add("AirGroup,I. Fliegerkorps,BoB_LW_AufklGr_122,4,Haute-Fontaine,Aircraft.Do-17Z-1"); //BoB_LW_AufklGr_122.01, 4 single-flights

        //II. Fliegerkorps, Ghent, LF2
        //strDynCampList.Add("AirGroup,BoB_LW_KG2_Stab,5,Arras_Saint_Liger,Aircraft.Do-17Z-2"); //Epinoy (SE Arras)
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_KG2_I,27,Arras_Saint_Liger,Aircraft.Do-17Z-1"); //Epinoy (SE Arras)
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_KG2_II,27,Arras,Aircraft.Do-17Z-2");
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_KG2_III,18,Arras_Saint_Liger,Aircraft.Do-17Z-1"); //Cambrai (SE Arras)

        //strDynCampList.Add("AirGroup,BoB_LW_KG3_Stab,5,Rosieres_en_Santerre,Aircraft.Do-17Z-2"); //Le Culot (Belgium)
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_KG3_I,27,Rosieres_en_Santerre,Aircraft.Do-17Z-1"); //Le Culot (Belgium)
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_KG3_II,27,Rosieres_en_Santerre,Aircraft.Do-17Z-2"); //Antwerp/Deurne (Belgium)
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_KG3_III,18,Rosieres_en_Santerre,Aircraft.Do-17Z-1"); //Sint-Truiden (Belgium)

        //strDynCampList.Add("AirGroup,BoB_LW_KG53_Stab,5,Rosieres_en_Santerre,Aircraft.He-111H-2"); //Lille-Nord (Belgium)
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_KG53_I,27,Rosieres_en_Santerre,Aircraft.He-111H-2"); //Lille-Nord (Belgium)
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_KG53_II,27,Rosieres_en_Santerre,Aircraft.He-111H-2"); //Lille-Nord (Belgium)
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_KG53_III,18,Rosieres_en_Santerre,Aircraft.He-111H-2"); //Lille-Nord (Belgium)

        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_StG1_II,27,Rosieres_en_Santerre,Aircraft.Ju-87B-2"); //Lille-Nord (Belgium)
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_LG1_IV,27,Rosieres_en_Santerre,Aircraft.Ju-87B-2"); //Lille-Nord (Belgium)

        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_ErprGr210,24,Calais_Marck,Aircraft.Bf-110C-7"); //from 1./ZG1 with Bf 110C and 3./St.G.77 with Bf 110D
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_ErprGr210F,12,Calais_Marck,Aircraft.Bf-109E-4B"); //4./TrGr.186 with Bf 109E-4B

        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_LG2_I,36,Calais_Marck,Aircraft.Bf-109E-4");
        strDynCampList.Add("AirGroup,II. Fliegerkorps,BoB_LW_LG2_II,27,Calais_Marck,Aircraft.Bf-109E-4B");

        //IX. Fliegerkorps, Soesterberg (Utrecht NL)
        strDynCampList.Add("AirGroup,IX. Fliegerkorps,BoB_LW_KG4_I,27,Soesterberg,Aircraft.He-111H-2");
        strDynCampList.Add("AirGroup,IX. Fliegerkorps,BoB_LW_KG4_II,27,Eindhoven,Aircraft.He-111H-2"); //Eindhoven
        strDynCampList.Add("AirGroup,IX. Fliegerkorps,BoB_LW_KG4_III,27,Schiphol,Aircraft.Ju-88A-1"); //Amsterdam/Schiphol

        strDynCampList.Add("AirGroup,IX. Fliegerkorps,BoB_LW_AufklGr_122,4,Eindhoven,Aircraft.Ju-88A-1"); // 3.Staffel ?  //BoB_LW_AufklGr_122.03, 4 single-flights

        //Jagdfliegerfuhrer 2, Wissant
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG3_I,36,Colembert,Aircraft.Bf-109E-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG3_II,36,Samer,Aircraft.Bf-109E-4B");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG3_III,36,Desvres,Aircraft.Bf-109E-3");

        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG26_I,36,Audembert,Aircraft.Bf-109E-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG26_II,36,Marquise_West,Aircraft.Bf-109E-4B");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG26_III,36,Caffiers,Aircraft.Bf-109E-3");

        //strDynCampList.Add("AirGroup,BoB_LW_JG51_Stab,36,Saint_Inglevert,Aircraft.Bf-109E-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG51_I,36,Wissant,Aircraft.Bf-109E-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG51_II,36,Desvres,Aircraft.Bf-109E-3B");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG51_III,36,St-Omer-Clairmarais,Aircraft.Bf-109E-3");

        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG52_I,36,Coquelles,Aircraft.Bf-109E-3");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG52_II,36,Peuplingues,Aircraft.Bf-109E-3B");

        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG54_I,36,Guines,Aircraft.Bf-109E-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG54_II,36,Hermelinghen,Aircraft.Bf-109E-4B");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_JG54_III,36,Campagne_Les_Guines,Aircraft.Bf-109E-3");

        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_ZG26_I,36,Yvrench,Aircraft.Bf-110C-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_ZG26_II,36,Crecy,Aircraft.Bf-110C-7");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_ZG26_III,36,Barly,Aircraft.Bf-110C-4");

        // Scandinavia strDynCampList.Add("AirGroup,BoB_LW_ZG76_I,36,Yvrench,Aircraft.Bf-110C-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_ZG76_II,36,Bernay_Saint_Martin,Aircraft.Bf-110C-4"); // Laval
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 2,BoB_LW_ZG76_III,36,Bernay_Saint_Martin,Aircraft.Bf-110C-4"); // Laval

        //IV. Fliegerkorps, Dinard, LF3
        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_LG1_I,23,Creil,Aircraft.Ju-88A-1"); // Orléans/Bricy south Paris
        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_LG1_II,24,Creil,Aircraft.Ju-88A-1"); // Orléans/Bricy south Paris
        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_LG1_III,23,Creil,Aircraft.Ju-88A-1"); // Châteaudun

        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG27_I,22,Caen_carpiquet,Aircraft.He-111P-2"); // Tours (C France offmap)
        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG27_II,21,Caen_carpiquet,Aircraft.He-111H-2"); // Dinard (just west offmap)
        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG27_III,23,Caen_carpiquet,Aircraft.He-111H-2"); // Rennes (just s.west offmap)

        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_AufklGr_31,4,Caen_carpiquet,Aircraft.Ju-88A-1"); // Rennes

        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG51_I,21,Caen_carpiquet,Aircraft.Ju-88A-1"); // Melun (S Paris)
        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG51_II,24,Caen_carpiquet,Aircraft.Ju-88A-1"); // Étampes (S Paris)
        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG51_III,25,Caen_carpiquet,Aircraft.Ju-88A-1"); // Étampes (S Paris)

        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG54_I,29,Bernay_Saint_Martin,Aircraft.Ju-88A-1"); // Évreux (Normandy, SSE of Rouen)
        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG54_II,23,Bernay_Saint_Martin,Aircraft.Ju-88A-1"); // St. Andre-de-L’Eure

        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG55_I,35,Bernay_Saint_Martin,Aircraft.He-111P-2"); // Dreux (W Paris)
        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG55_II,28,Bernay_Saint_Martin,Aircraft.He-111H-2"); // Chartres (SW Paris)
        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_KG55_III,33,Bernay_Saint_Martin,Aircraft.He-111H-2"); // Villacoublay (Paris)

        strDynCampList.Add("AirGroup,IV. Fliegerkorps,BoB_LW_AufklGr_14,9,Theville,Aircraft.Bf-110C-4"); // Cherbourg

        // VIII. Fliegerkorps
        strDynCampList.Add("AirGroup,VIII. Fliegerkorps,BoB_LW_StG1_I,27,Carquebut,Aircraft.Ju-87B-2"); // Angers (S Caen/SW Paris)
        strDynCampList.Add("AirGroup,VIII. Fliegerkorps,BoB_LW_StG1_III,26,Carquebut,Aircraft.Ju-87B-2"); // Angers (S Caen/SW Paris)

        strDynCampList.Add("AirGroup,VIII. Fliegerkorps,BoB_LW_StG2_I,29,Carquebut,Aircraft.Ju-87B-2"); // St Malo (N.Coast S Jersey)
        strDynCampList.Add("AirGroup,VIII. Fliegerkorps,BoB_LW_StG2_II,33,Carquebut,Aircraft.Ju-87B-2"); // Lannion (N.Coast SW Jersey)

        strDynCampList.Add("AirGroup,VIII. Fliegerkorps,BoB_LW_StG3_I,24,Caen_carpiquet,Aircraft.Ju-87B-2");

        strDynCampList.Add("AirGroup,VIII. Fliegerkorps,BoB_LW_StG77_I,33,Caen_carpiquet,Aircraft.Ju-87B-2");
        strDynCampList.Add("AirGroup,VIII. Fliegerkorps,BoB_LW_StG77_II,25,Caen_carpiquet,Aircraft.Ju-87B-2");
        strDynCampList.Add("AirGroup,VIII. Fliegerkorps,BoB_LW_StG77_III,37,Caen_carpiquet,Aircraft.Ju-87B-2");

        strDynCampList.Add("AirGroup,VIII. Fliegerkorps,BoB_LW_LG1_V,29,Caen_carpiquet,Aircraft.Bf-110C-7");

        strDynCampList.Add("AirGroup,VIII. Fliegerkorps,BoB_LW_AufklGr_11,6,Bernay_Saint_Martin,Aircraft.Bf-110C-4");

        // Jagdfliegerfuhrer 3
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_JG2_I,32,Beaumont_Le_Roger,Aircraft.Bf-109E-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_JG2_II,28,Beaumont_Le_Roger,Aircraft.Bf-109E-4B");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_JG2_III,28,Le_Havre_Octeville,Aircraft.Bf-109E-3");

        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_JG27_I,32,Plumetot,Aircraft.Bf-109E-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_JG27_II,32,Crepon,Aircraft.Bf-109E-4B");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_JG27_III,32,Le_Havre_Octeville,Aircraft.Bf-109E-3");
        /* too far off west
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_JG53_I,37,Rennes,Aircraft.Bf-109E-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_JG53_II,34,Dinan,Aircraft.Bf-109E-4");
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_JG53_III,35,Brest,Aircraft.Bf-109E-3");
        */
        strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_ZG2_I,35,Caen_carpiquet,Aircraft.Bf-110C-4");
        //strDynCampList.Add("AirGroup,Jagdfliegerfuhrer 3,BoB_LW_ZG2_II,34,Crepon,Aircraft.Bf-110C-4"); // Guyancourt (Paris)
        if (defPlayer.Contains("RA_")) // CAI version get G.50 sqns from 10 July as "what if scenario"
        {
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_13St_11Gruppo_1Sq,9,Merville_Calonne,Aircraft.BR-20M");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_13St_11Gruppo_4Sq,9,Merville_Calonne,Aircraft.BR-20M");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_13St_43Gruppo_3Sq,9,Merville_Calonne,Aircraft.BR-20M");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_13St_43Gruppo_5Sq,9,Merville_Calonne,Aircraft.BR-20M");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_43St_98Gruppo_240Sq,9,Merville_Calonne,Aircraft.BR-20M");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_43St_98Gruppo_241Sq,9,Merville_Calonne,Aircraft.BR-20M");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_43St_98Gruppo_242Sq,9,Merville_Calonne,Aircraft.BR-20M");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_43St_98Gruppo_243Sq,9,Merville_Calonne,Aircraft.BR-20M");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_56St_18Gruppo_83Sq,12,Oye-Plage,Aircraft.CR42"); // editor only allows 3x3
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_56St_18Gruppo_85Sq,12,Oye-Plage,Aircraft.CR42");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_56St_18Gruppo_95Sq,12,Oye-Plage,Aircraft.CR42");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_56St_20Gruppo_351Sq,12,Oye-Plage,Aircraft.G50");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_56St_20Gruppo_352Sq,12,Oye-Plage,Aircraft.G50");
            strDynCampList.Add("AirGroup,Corpo Aereo Italiano,BoB_RA_56St_20Gruppo_353Sq,12,Oye-Plage,Aircraft.G50");
        };

        // Events
        strDynCampList.Add("Event,12 July 1940,remove,AirGroup,BoB_RAF_F_79Sqn,Turnhouse");
        strDynCampList.Add("Event,20 July 1940,remove,AirGroup,BoB_RAF_F_245Sqn,Aldergrove");
        strDynCampList.Add("Event,24 July 1940,move,AirGroup,BoB_RAF_F_1Sqn,Northolt");
        strDynCampList.Add("Event,13 August 1940,move,AirGroup,BoB_RAF_F_266Sqn,Eastchurch");
        strDynCampList.Add("Event,14 August 1940,move,AirGroup,BoB_RAF_F_74Sqn,Wittering");
        strDynCampList.Add("Event,14 August 1940,remove,AirGroup,BoB_RAF_F_145Sqn,Dyce");
        strDynCampList.Add("Event,14 August 1940,add,AirGroup,BoB_RAF_F_249Sqn,Boscombe_Down,12,Aircraft.HurricaneMkI_100oct,RAF Middle Wallop");
        strDynCampList.Add("Event,19 August 1940,move,AirGroup,BoB_RAF_F_17Sqn,Tangmere");
        strDynCampList.Add("Event,19 August 1940,remove,AirGroup,BoB_RAF_F_64Sqn,Leconfield");
        strDynCampList.Add("Event,19 August 1940,remove,AirGroup,BoB_RAF_F_601Sqn,Debden");
        strDynCampList.Add("Event,20 August 1940,add,AirGroup,BoB_RAF_F_616Sqn,Kenley,12,Aircraft.SpitfireMkIa_100oct,RAF Kenley");
        strDynCampList.Add("Event,21 August 1940,move,AirGroup,BoB_RAF_F_266Sqn,Manston");
        strDynCampList.Add("Event,22 August 1940,move,AirGroup,BoB_RAF_F_264Sqn,Hornchurch");
        strDynCampList.Add("Event,24 August 1940,move,AirGroup,BoB_RAF_F_600Sqn,Hornchurch");
        strDynCampList.Add("Event,27 August 1940,remove,AirGroup,BoB_RAF_F_65Sqn,Turnhouse");
        strDynCampList.Add("Event,27 August 1940,add,AirGroup,BoB_RAF_F_603Sqn,Hornchurch,12,Aircraft.SpitfireMkIa_100oct,RAF Hornchurch");
        strDynCampList.Add("Event,27 August 1940,add,AirGroup,BoB_RAF_F_79Sqn,Hawkinge,12,Aircraft.HurricaneMkI_100oct,RAF Biggin Hill");
        strDynCampList.Add("Event,28 August 1940,move,AirGroup,BoB_RAF_F_32Sqn,Acklington");
        strDynCampList.Add("Event,29 August 1940,remove,AirGroup,BoB_RAF_F_151Sqn,Stapleford");
        strDynCampList.Add("Event,29 August 1940,remove,AirGroup,BoB_RAF_F_264Sqn,Kirton");
        strDynCampList.Add("Event,29 August 1940,remove,AirGroup,BoB_RAF_F_615Sqn,Prestwick");
        strDynCampList.Add("Event,30 August 1940,add,AirGroup,BoB_RAF_F_253Sqn,Kenley,12,Aircraft.HurricaneMkI_100oct,RAF Kenley");
        strDynCampList.Add("Event,30 August 1940,add,AirGroup,BoB_RAF_F_222Sqn,Hornchurch,12,Aircraft.SpitfireMkIa_100oct,RAF Hornchurch");
        strDynCampList.Add("Event,31 August 1940,move,AirGroup,BoB_RAF_F_610Sqn,Acklington");
        strDynCampList.Add("Event,31 August 1940,add,AirGroup,BoB_RAF_F_303Sqn,Croydon,12,Aircraft.HurricaneMkI_100oct,RAF Kenley");
        strDynCampList.Add("Event,31 August 1940,remove,AirGroup,BoB_RAF_F_257Sqn,Martlesham");
        strDynCampList.Add("Event,31 August 1940,add,AirGroup,BoB_RAF_F_72Sqn,Biggin_Hill,12,Aircraft.SpitfireMkIa_100oct,RAF Biggin Hill");
        strDynCampList.Add("Event,1 September 1940,move,AirGroup,BoB_RAF_F_249Sqn,North_weald");
        strDynCampList.Add("Event,2 September 1940,remove,AirGroup,BoB_RAF_F_17Sqn,Debden");
        strDynCampList.Add("Event,2 September 1940,move,AirGroup,BoB_RAF_F_601Sqn,Tangmere");
        strDynCampList.Add("Event,1 September 1940,add,AirGroup,BoB_RAF_F_46Sqn,North_weald,12,Aircraft.HurricaneMkI_100oct,RAF North Weald");
        strDynCampList.Add("Event,1 September 1940,move,AirGroup,BoB_RAF_F_72Sqn,Croydon");
        strDynCampList.Add("Event,2 September 1940,remove,AirGroup,BoB_RAF_F_242Sqn,Duxford"); // already offmap unit..
        strDynCampList.Add("Event,3 September 1940,move,AirGroup,BoB_RAF_F_66Sqn,Hornchurch");
        //strDynCampList.Add("Event,3 September 1940,move,AirGroup,BoB_RAF_F_54Sqn,Catterick");
        strDynCampList.Add("Event,3 September 1940,remove,AirGroup,BoB_RAF_F_111Sqn,Debden");
        strDynCampList.Add("Event,3 September 1940,remove,AirGroup,BoB_RAF_F_616Sqn,Kirton");
        strDynCampList.Add("Event,5 September 1940,type,AirGroup,BoB_RAF_F_266Sqn,Manston,0,Aircraft.SpitfireMkIIa");
        strDynCampList.Add("Event,6 September 1940,add,AirGroup,BoB_RAF_F_504Sqn,Hendon,12,Aircraft.HurricaneMkI_100oct,RAF Northolt");
        strDynCampList.Add("Event,7 September 1940,remove,AirGroup,BoB_RAF_F_601Sqn,Exeter");
        strDynCampList.Add("Event,7 September 1940,move,AirGroup,BoB_RAF_F_111Sqn,Kenley");
        strDynCampList.Add("Event,7 September 1940,add,AirGroup,BoB_RAF_F_213Sqn,Tangmere,12,Aircraft.HurricaneMkI_100oct,RAF Tangmere");
        strDynCampList.Add("Event,7 September 1940,add,AirGroup,BoB_RAF_F_234Sqn,Andover,12,Aircraft.SpitfireMkIa_100oct,RAF Middle Wallop"); // middle wallop
        strDynCampList.Add("Event,7 September 1940,add,AirGroup,BoB_RAF_F_605Sqn,Croydon,12,Aircraft.HurricaneMkI_100oct,RAF Kenley");
        strDynCampList.Add("Event,8 September 1940,remove,AirGroup,BoB_RAF_F_43Sqn,Usworth");
        strDynCampList.Add("Event,8 September 1940,add,AirGroup,BoB_RAF_F_92Sqn,Biggin_Hill,12,Aircraft.SpitfireMkIa_100oct,RAF Biggin Hill");
        strDynCampList.Add("Event,8 September 1940,remove,AirGroup,BoB_RAF_F_79Sqn,Pembrey");
        strDynCampList.Add("Event,9 September 1940,remove,AirGroup,BoB_RAF_F_1Sqn,Wittering");
        strDynCampList.Add("Event,9 September 1940,move,AirGroup,BoB_RAF_F_229Sqn,Northolt");
        strDynCampList.Add("Event,9 September 1940,add,AirGroup,BoB_RAF_F_607Sqn,Tangmere,12,Aircraft.HurricaneMkI_100oct,RAF Tangmere");
        strDynCampList.Add("Event,10 September 1940,type,AirGroup,BoB_RAF_F_74Sqn,Wittering,0,Aircraft.SpitfireMkIIa");
        strDynCampList.Add("Event,10 September 1940,move,AirGroup,BoB_RAF_F_501Sqn,Kenley");
        strDynCampList.Add("Event,11 September 1940,remove,AirGroup,BoB_RAF_F_234Sqn,StEval");
        strDynCampList.Add("Event,11 September 1940,move,AirGroup,BoB_RAF_F_66Sqn,Gravesend");
        strDynCampList.Add("Event,13 September 1940,move,AirGroup,BoB_RAF_F_23Sqn,Ford"); // Fleet Air Arm Station
        strDynCampList.Add("Event,14 September 1940,move,AirGroup,BoB_RAF_F_604Sqn,Andover"); // middle wallop
        strDynCampList.Add("Event,15 September 1940,move,AirGroup,BoB_RAF_F_600Sqn,Kenley");
        strDynCampList.Add("Event,20 September 1940,type,AirGroup,BoB_RAF_F_19Sqn,Fowlmere,0,Aircraft.SpitfireMkIIa");
        strDynCampList.Add("Event,25 September 1940,type,AirGroup,BoB_RAF_F_66Sqn,Gravesend,0,Aircraft.SpitfireMkIIa");
        //strDynCampList.Add("Event,10 October 1940,type,AirGroup,BoB_RAF_F_41Sqn,Catterick,0,Aircraft.SpitfireMkIIa");
        strDynCampList.Add("Event,15 October 1940,type,AirGroup,BoB_RAF_F_603Sqn,Hornchurch,0,Aircraft.SpitfireMkIIa");

        if (!defPlayer.Contains("RA_")) // CAI version get G.50 sqns from 10 July as "what if scenario"
        {
            strDynCampList.Add("Event,28 September 1940,add,AirGroup,BoB_RA_13St_11Gruppo_1Sq,Melsbroech,9,Aircraft.BR-20M,Corpo Aereo Italiano");
            strDynCampList.Add("Event,28 September 1940,add,AirGroup,BoB_RA_13St_11Gruppo_4Sq,Melsbroech,9,Aircraft.BR-20M,Corpo Aereo Italiano");
            strDynCampList.Add("Event,28 September 1940,add,AirGroup,BoB_RA_13St_43Gruppo_3Sq,Melsbroech,9,Aircraft.BR-20M,Corpo Aereo Italiano");
            strDynCampList.Add("Event,28 September 1940,add,AirGroup,BoB_RA_13St_43Gruppo_5Sq,Melsbroech,9,Aircraft.BR-20M,Corpo Aereo Italiano");
            strDynCampList.Add("Event,28 September 1940,add,AirGroup,BoB_RA_43St_98Gruppo_240Sq,Chievres,9,Aircraft.BR-20M,Corpo Aereo Italiano");
            strDynCampList.Add("Event,28 September 1940,add,AirGroup,BoB_RA_43St_98Gruppo_241Sq,Chievres,9,Aircraft.BR-20M,Corpo Aereo Italiano");
            strDynCampList.Add("Event,28 September 1940,add,AirGroup,BoB_RA_43St_98Gruppo_242Sq,Chievres,9,Aircraft.BR-20M,Corpo Aereo Italiano");
            strDynCampList.Add("Event,28 September 1940,add,AirGroup,BoB_RA_43St_98Gruppo_243Sq,Chievres,9,Aircraft.BR-20M,Corpo Aereo Italiano");
            strDynCampList.Add("Event,22 October 1940,add,AirGroup,BoB_RA_56St_18Gruppo_83Sq,Ursel,12,Aircraft.CR42,Corpo Aereo Italiano"); // editor only allows 3x3
            strDynCampList.Add("Event,22 October 1940,add,AirGroup,BoB_RA_56St_18Gruppo_85Sq,Ursel,12,Aircraft.CR42,Corpo Aereo Italiano");
            strDynCampList.Add("Event,22 October 1940,add,AirGroup,BoB_RA_56St_18Gruppo_95Sq,Ursel,12,Aircraft.CR42,Corpo Aereo Italiano");
            strDynCampList.Add("Event,22 October 1940,add,AirGroup,BoB_RA_56St_20Gruppo_351Sq,Ursel,12,Aircraft.G50,Corpo Aereo Italiano");
            strDynCampList.Add("Event,22 October 1940,add,AirGroup,BoB_RA_56St_20Gruppo_352Sq,Ursel,12,Aircraft.G50,Corpo Aereo Italiano");
            strDynCampList.Add("Event,22 October 1940,add,AirGroup,BoB_RA_56St_20Gruppo_353Sq,Ursel,12,Aircraft.G50,Corpo Aereo Italiano");
        };
        /*
        if (odenDebug)
        {
            strDynCampList.Add("AirportDamage,Abbeville,75");
            strDynCampList.Add("GndStatDead,0:Static1357");
            strDynCampList.Add("GndGrpDead,0:0_Chief");
            strDynCampList.Add("GndGrpDead,0:1_Chief");
        };
        */
        // 360000x:270000y is Belgium/Germany Map Entry Point - use when Base name not found (KG4 Schiphol etc)
        // 180000x:310000y is Essex/Suffolk Map Entry Point (RAF Bomber Squadrons)
        if (odenDebug) GamePlay.gpLogServer(null, "CreateIniFiles: {0}", new object[] { defSquadron });

        foreach (string str in strDynCampList)
            if (str.Contains("AirGroup"))
                if (str.Contains(defSquadron))
                {   // "AirGroup,RAF Hornchurch,BoB_RAF_F_54Sqn,12,Sounthend,Aircraft.SpitfireMkIa_100oct"
                    string[] strs = str.Split(new char[] { ',' }, StringSplitOptions.None);
                    homeBase = ParseAirGroup(strs);
                    break;
                };

        if (odenDebug) GamePlay.gpLogServer(null, "CreateIniFiles: {0}", new object[] { homeBase.airportName });

        // refresh briefing
        string strBase = homeBase.airportName.Replace('_', ' ');
        if (strBase == "Sounthend")
            strBase = "Rochford";
        int len = 6;
        if (homeBase.aircraftType.Length < 9 + 6)
            len = homeBase.aircraftType.Length - 9;

        defSatellites = SatelliteStations(homeBase.sector);
        strDynCampList.Add("Satellites," + defSatellites);
        if (odenDebug) GamePlay.gpLogServer(null, "CreateIniFiles: {0}", new object[] { defSatellites });
        if (odenDebug) GamePlay.gpLogServer(null, "CreateIniFiles: {0}", new object[] { CampInitFile });

        using (StreamWriter sw = File.CreateText(CampInitFile))
        {
            foreach (string str in strDynCampList)
                sw.WriteLine(str);
        };

        string type = "DynMis40" + homeBase.aircraftType.Substring(9, len).ToLower();
        string slide = type + ".jpg";
        string thin = type + "_thin.jpg";
        AiAircraft ac = (AiAircraft)GamePlay.gpPlayer().Place();

        string[] strBriefing = 
        {
            "[Info]"
            , "Info"
            , "<Slide>"
            , slide
            , ""
            , "<Caption>"
            , missionCaption
            , "<Slide>"
            , thin
            , ""
            , "<Caption>"
            , "(This description will change as your mission progresses)"
            , "Battle of Britain"
            , gameDateTime.ToString("D") //"July 10th 1940"
            , ""
            , homeBase.sector
            , homeBase.name +" (" +strBase +")"
            , "[Mission]"
            , "<Name>"
            , "Mission"
            , "<Description>"
            , "This is the first mission of Battle of Britain and it will change each time you fly unless you quit without taking off"
            , ""
            , "Victory conditions:"
            , "Luftwaffe strength lower than RAF at anytime."
            , "Luftwaffe strength ratio less than 3:1 by end of October."
            , "(no grande finale message or what ever so, this is not a campaign with a story unless you make one while flying)"
            , ""
            , "Battle of Britain"
            , gameDateTime.ToString("D") //"July 10th 1940"
            , ""
            , homeBase.sector
            , homeBase.name +" (" +strBase +")"
            , Designation(homeBase.aircraftType) //"Spitfire Mk Ia"
            , tailNoSerialNo
            , "check your callsign in <Player Plane>"
            , ""
            , "You will not get a briefing before each flight, just follow flight lead and listen to instructions."
            , "Mission description will be events from the previous day/days (randomly 1 to 3 days)."
            , "Overall Status will also be shown such as next date for reinforcement aircraft and airfields repair status and so on."
            , ""
            , "But for now, just press 'Fly' and follow your Red Flight leader with your section, Blue Flight will follow shortly."
            , "While landing after mission is not required it is recommended to keep flying after any combat to 'collect' kills as damaged aircrafts crash returning to base - it will help the 'campaign numbers'"
            , ""
            , "(When Satellite airfields are available, they will be listed down here)"
            , "<Slide>"
            , slide
            , "<Caption>"
            , ""
        };

        if (odenDebug) GamePlay.gpLogServer(null, "CreateIniFiles: {0}", new object[] { CampBriFile });
        using (StreamWriter sw = File.CreateText(CampBriFile))
        {
            foreach (string str in strBriefing)
                sw.WriteLine(str);
        };

        foreach (MissionMarker mm in MissionMarkers)
            switch (mm.key)
            {
                case fmDungeness:
                case fmDover:
                case fmRamsgate:
                case fmCanterbury:
                case fmRochester:
                case fmLondon:
                    mm.army = RAF;
                    break;
            };

        odenDebug = false;
        UpdateMissionFile();

        GamePlay.gpHUDLogCenter("Mission files have been restored, please return to the main menu");
    }

    private void ReadInitFile()
    {
        string[] strDynCamp = File.ReadAllLines(CampInitFile);

        int forEachIndex = 0;
        foreach (string str in strDynCamp)
        {
            //if(odenDebug) GamePlay.gpLogServer(null, "OdenDynCamp: {0}", new object[] { str });
            if (str.Length > 0)
            {
                string[] strs = str.Split(new char[] { ',' }, StringSplitOptions.None);
                switch (strs[0])
                {
                    case "CampaignDate":
                        gameDateTime = DateTime.Parse(strs[1]);
                        gameDateTime = gameDateTime.AddHours(5);
                        break;
                    case "ReinforcementDate":
                        ReinforcementDate = DateTime.Parse(strs[1]);
                        break;
                    case "highOctaneDate":
                        highOctaneDate = DateTime.Parse(strs[1]);
                        break;
                    case "Satellites":
                        Satellites.Clear();
                        foreach (string s in strs)
                            if (s != "Satellites" && s != "")
                                Satellites.Add(s);
                        if (Satellites.Count > 0)
                            defSatellites = string.Join(",", Satellites.ToArray());
                        break;
                    case "AirportDamage":
                        AirportsDamaged.Add(strs[1] + "," + strs[2]);
                        break;
                    case "CHRadarDamage":
                        CHRadarDamaged.Add(strs[1] + "," + strs[2]);
                        break;
                    case "PortDamage":
                        PortDamaged.Add(strs[1] + "," + strs[2]);
                        break;
                    case "FactoryDamage":
                        FactoryDamaged.Add(strs[1] + "," + strs[2]);
                        break;
                    case "GndStatDead":
                        GndStatDead.Add(strs[1]);
                        break;
                    case "GndGrpDead":
                        GndGrpDead.Add(strs[1]);
                        break;
                    case "AirGroup":
                        //AirGroup,RAF Hornchurch,BoB_RAF_F_54Sqn,12,Sounthend,Aircraft.SpitfireMkIa_100oct
                        //if (odenDebug) GamePlay.gpLogServer(null, "Airgroup status: {1} {0} at {3} flying {2} {4}", new object[] { strs[1], strs[2], strs[3], strs[4], strs[5] });
                        MissionAirbase ab = ParseAirGroup(strs);
                        MissionAirbases.Add(ab);
                        //if(odenDebug) GamePlay.gpLogServer(null, "MissionAirbase: {0} at {1}", new object[] { ab.name, ab.airportName });
                        break;
                    case "Event":
                        string[] strEvent = new string[9];
                        for (int i = 0; i < strEvent.Length; i++)
                            if (strs.Length > i) strEvent[i] = strs[i];
                            else strEvent[i] = "";

                        DynCampEvent e = new DynCampEvent(DateTime.Parse(strEvent[1])
                            , strEvent[2] //action
                            , strEvent[3] //type
                            , strEvent[4] //typeValue
                            , strEvent[5] //airportName
                            , strEvent[6] //param1, qty for add
                            , strEvent[7] //param2, planetype for add
                            , strEvent[8] //param3, sector for add
                            );
                        DynCampEvents.Add(e);
                        break;
                };
            };
        };

        //if(odenDebug) GamePlay.gpLogServer(null, "gameDateTime: {0}", new object[] { gameDateTime.ToString("D") });
        foreach (DynCampEvent e in DynCampEvents)
        {
            //if(odenDebug) GamePlay.gpLogServer(null, "DynCampEvent?: {0} {1} {2} {3}", new object[] { e.dateTime.ToString("D"), e.typeValue, e.param1, e.airportName });
            if (e.dateTime <= gameDateTime)
            {
                //if(odenDebug) GamePlay.gpLogServer(null, "DynCampEvent: {0} {1} {2} {3}", new object[] { e.dateTime.ToString("D"), e.typeValue, e.param1, e.airportName });
                switch (e.action)
                {
                    case "add":
                        if (e.type == "AirGroup")
                        {   //Event,1 September 1940,add,AirGroup,BoB_RAF_F_46Sqn,North_weald,12,Aircraft.HurricaneMkI_100oct,RAF North Weald
                            string[] strs = { "AirGroup", e.param3, e.typeValue, e.param1, e.airportName, e.param2 };
                            MissionAirbase ab = ParseAirGroup(strs);
                            MissionAirbases.Add(ab);
                            if (ab.sector.Contains(ab.airportName.Substring(0, 3))) mainStoryList.Add(ab.name + " moved to " + ab.airportName.Replace('_', ' '));
                            else mainStoryList.Add(ab.name + " moved to " + ab.airportName.Replace('_', ' ') + " " + ab.sector);

                        };
                        break;
                    case "move":
                        if (e.type == "AirGroup")
                        {   //Event,7 September 1940,move,AirGroup,BoB_RAF_F_111Sqn,Kenley
                            MissionAirbase xab = null;
                            List<int> removeAirbaseAt = new List<int>();
                            int forEachIdx = 0;
                            foreach (MissionAirbase mab in MissionAirbases)
                            {
                                if (mab.sqn.Contains(e.typeValue) && !mab.sqn.Contains(defSquadron))
                                {
                                    xab = mab;
                                    removeAirbaseAt.Add(forEachIdx);
                                    break;
                                };
                                forEachIdx++;
                            };
                            foreach (int i in removeAirbaseAt)
                                MissionAirbases.RemoveAt(i);
                            string[] strs = { "AirGroup", e.param3, e.typeValue, e.param1, e.airportName, e.param2 };
                            MissionAirbase ab = ParseAirGroup(strs);
                            MissionAirbases.Add(ab);
                            if (xab != null)
                                if (ab.sector.Contains(ab.airportName.Substring(0, 3))) mainStoryList.Add(ab.name + " " + xab.sector + " moved to " + ab.airportName.Replace('_', ' '));
                                else mainStoryList.Add(ab.name + " " + xab.sector + " moved to " + ab.airportName.Replace('_', ' ') + " " + ab.sector);
                            else
                                if (ab.sector.Contains(ab.airportName.Substring(0, 3))) mainStoryList.Add(ab.name + " moved to " + ab.airportName.Replace('_', ' '));
                                else mainStoryList.Add(ab.name + " moved to " + ab.airportName.Replace('_', ' ') + " " + ab.sector);

                        };
                        break;
                    case "remove":
                        if (e.type == "AirGroup")
                        {   //Event,7 September 1940,remove,AirGroup,BoB_RAF_F_601Sqn,Exeter
                            MissionAirbase xab = null;
                            List<int> removeAirbaseAt = new List<int>();
                            int forEachIdx = 0;
                            foreach (MissionAirbase mab in MissionAirbases)
                            {
                                if (mab.sqn.Contains(e.typeValue) && !mab.sqn.Contains(defSquadron))
                                {
                                    xab = mab;
                                    removeAirbaseAt.Add(forEachIdx);
                                    break;
                                };
                                forEachIdx++;
                            };
                            foreach (int i in removeAirbaseAt)
                                MissionAirbases.RemoveAt(i);
                            string[] strs = { "AirGroup", e.param3, e.typeValue, e.param1, e.airportName, e.param2 };
                            MissionAirbase ab = ParseAirGroup(strs);
                            if (xab != null) mainStoryList.Add(ab.name + " " + xab.sector + " moved away to " + ab.airportName.Replace('_', ' '));
                            else mainStoryList.Add(ab.name + " moved away to " + ab.airportName.Replace('_', ' '));
                        };
                        break;
                    case "type":
                        if (e.type == "AirGroup")
                        {   //Event,1 October 1940,type,AirGroup,BoB_LW_JG2_III,Le_Havre_Octeville,0,Aircraft.Bf-109E-3B
                            foreach (MissionAirbase mab in MissionAirbases)
                                if (mab.sqn.Contains(e.typeValue))
                                {
                                    mab.aircraftType = e.param2;
                                    break;
                                };
                            string[] strs = { "AirGroup", e.param3, e.typeValue, e.param1, e.airportName, e.param2 };
                            MissionAirbase ab = ParseAirGroup(strs);
                            mainStoryList.Add(ab.name + " recieved " + Designation(ab.aircraftType));
                        };
                        break;
                };
            };
        };

        DynCampEvents.RemoveAll(e => e.dateTime <= gameDateTime);

    }

    private MissionAirbase ParseAirGroup(string[] strs)
    {   // "AirGroup,RAF Biggin Hill,BoB_RAF_F_1Sqn,12,Hawkinge,Aircraft.HurricaneMkI_100oct"        
        AiAirport airport = null;
        foreach (AiAirport ap in GamePlay.gpAirports())
            if (ap.Name().Equals(strs[4]))
            {
                airport = ap;
                break;
            };
        string name = strs[2];
        if (name.Contains("RAF_"))
        {
            name = name.Substring(10); // BoB_RAF_F_264Sqn >> 264Sqn
            name = name.Insert(name.Length - 3, " ");
        };
        if (name.Contains("LW_"))
        {
            name = name.Substring(7); // BoB_LW_KG2_I >> KG2_I
            int pos = name.IndexOf('_');
            if (pos > -1 && (name.Contains("I") || name.Contains("Stab")))
            {
                string gruppe = name.Substring(pos + 1);
                name = gruppe + "./" + name.Substring(0, pos);
            };
        };
        if (name.Contains("RA_"))
            name = name.Substring(7); // BoB_RA_56St_20Gruppo_353Sq >> 56St_20Gruppo_353Sq
        name = name.Replace('_', ' ');
        int army = strs[2].Contains("RAF") ? RAF : LW;
        string country = "gb";
        if (strs[2].Contains("LW")) country = "de";
        if (strs[2].Contains("RA_")) country = "it";
        int maxPlanes = 0;
        if (!int.TryParse(strs[3], out maxPlanes))
            maxPlanes = 0;

        // 360000x:270000y is Belgium/Germany Map Entry Point - use when Base name not found (KG4 Schiphol etc)
        // 180000x:310000y is Essex/Suffolk Map Entry Point (RAF Bomber Squadrons)
        Point2d pt2d = new Point2d(360000, 270000);
        if (airport != null)
        {
            pt2d.x = airport.Pos().x;
            pt2d.y = airport.Pos().y;
        }
        else
        {
            if (country == "gb")
            {
                pt2d.x = 180000;
                pt2d.y = 310000;
            };
        };

        string type = "RAF_Fighter_1939";
        if (strs[2].Contains("RAF_B")) type = "RAF_Bomber_1940";
        if (strs[2].Contains("RA_")) type = "RegiaAeronautica_Bomber_1940"; // RA_43, RA_13
        if (strs[2].Contains("RA_5")) type = "RegiaAeronautica_Fighter_1940"; // RA_53
        if (strs[2].Contains("LW_JG")) type = "Luftwaffe_Fighter_1940";
        if (strs[2].Contains("LW_LG")) type = "Luftwaffe_Fighter_1940";
        if (strs[2].Contains("LW_LG1")) type = "Luftwaffe_Bomber_1940"; // Ju88
        if (strs[2].Contains("LW_LG1_IV")) type = "Luftwaffe_Stuka_1940";
        if (strs[2].Contains("LW_LG1_V")) type = "Luftwaffe_Zerstorer_1940";
        if (strs[2].Contains("LW_LG2_I")) type = "Luftwaffe_Fighter_1940"; // Bf109E-4, Calais-Marck
        if (strs[2].Contains("LW_LG2_II")) type = "Luftwaffe_Bomber_1940"; // Bf109E-4B, St Omer
        if (strs[2].Contains("LW_KG")) type = "Luftwaffe_Bomber_1940";
        if (strs[2].Contains("LW_ZG")) type = "Luftwaffe_Zerstorer_1940";
        if (strs[2].Contains("LW_StG")) type = "Luftwaffe_Stuka_1940";
        if (strs[2].Contains("LW_Auf")) type = "Luftwaffe_Recon_1940";
        if (strs[2].Contains("LW_ErprGr210")) type = "Luftwaffe_Zerstorer_1940";
        if (strs[2].Contains("LW_ErprGr210F")) type = "Luftwaffe_Fighter_1940";
        if (strs[2].Contains("_Stab"))
        {
            if (strs[2].Contains("LW_JG")) type = "Luftwaffe_Stabstaffel_1940";
            if (strs[2].Contains("LW_ZG")) type = "Luftwaffe_Zerstorer_Stabstaffel_1940";
            if (strs[2].Contains("LW_KG")) type = "Luftwaffe_Bomber_Stabstaffel_1940";
            if (strs[2].Contains("LW_StG")) type = "Luftwaffe_Stuka_Stabstaffel_1940";
        };
        string sqn = strs[2];
        string sector = strs[1];
        if (strs[2].Contains("BoB_RAF_F_")) sqn = sqn + "_Early";
        //if (odenDebug) GamePlay.gpLogServer(null, "ParseAirGroup: {0} {1} {2}", new object[] { name, sqn, type });
        MissionAirbase ab = new MissionAirbase(name
            , army
            , pt2d.x
            , pt2d.y
            , 0, maxPlanes, true, true
            , country, type, sqn, sector, strs[5], airport, strs[4]);
        return ab;
    }

    private void UpdateInitFile()
    {
        //if (odenDebug) GamePlay.gpLogServer(null, "UpdateInitFile: {0} {1}", new object[] { homeBase.airportName, homeBase.aircraftType });
        bool forceSwitchSqn = false;
        using (StreamWriter sw = File.CreateText(CampInitFile))
        {
            string[] strDynCamp = { "//Battle Of Britain"
                                  , "//Dynamic Mission by theOden"
                                  , "//This file keeps track of units etc. and may carefully be edited manually"
                                  , "//Entries can be in any order"
                                  , "//AirGroup,Sector,Squadron/Gruppe,Quantity,Airfieldname,AircraftType - all CliffsOfDover data"
                                  , "//AirportDamage,Airfieldname,Damage - all CliffsOfDover data (GamePlay.gpAirports())"
                                  , "//GndStatDead,StaticName - all CliffsOfDover data as per mis-file [Stationary]"
                                  , "//GndGrpDead,GroundGroupName - all CliffsOfDover data as per mis-file [Chiefs]"
                                  , "//Player Squadron must be edited in mission editor"
                                  , "//-"};
            foreach (string str in strDynCamp)
                sw.WriteLine(str);

            sw.WriteLine("CampaignDate," + gameDateTime.ToString("D"));
            //if (odenDebug) GamePlay.gpLogServer(null, "UpdateInitFile ReinforcementDate: {0}", new object[] { ReinforcementDate.ToString("D") });
            if (ReinforcementDate < gameDateTime)
            {
                foreach (MissionAirbase ab in MissionAirbases)
                    if (ab.army == RAF)
                        ab.maxPlanes += 1;
                ReinforcementDate = ReinforcementDate.AddDays(3);
                //if (odenDebug) GamePlay.gpLogServer(null, "UpdateInitFile ReinforcementDate: {0}", new object[] { ReinforcementDate.ToString("D") });
            };
            sw.WriteLine("ReinforcementDate," + ReinforcementDate.ToString("D"));
            sw.WriteLine("highOctaneDate," + highOctaneDate.ToString("D"));
            //sw.WriteLine("defSquadron," + defSquadron);
            if (Satellites.Count > 0)
                sw.WriteLine("Satellites," + string.Join(",", Satellites.ToArray()));
            else
                sw.WriteLine("Satellites,");

            // Spitfire MkII option
            if (gameDateTime.Month > 8)
            {
                if (homeBase.aircraftType.Contains("Spitfire"))
                    if (!homeBase.aircraftType.Contains("SpitfireMkIIa"))
                        if (rnd.Next(0, 99) > 95)
                        {
                            homeBase.aircraftType = "Aircraft.SpitfireMkIIa";
                            mainStoryList.Add(homeBase.name + " recieved " + Designation(homeBase.aircraftType));
                        };
            };
            //if(odenDebug) homeBase.aircraftType = "Aircraft.SpitfireMkIIa";

            int forEachIdx = 0;
            foreach (MissionAirbase ab in MissionAirbases)
            {
                //List<string> newPlaneSet = new List<string>();
                if (highOctaneDate > gameDateTime)
                {
                    switch (ab.aircraftType)
                    {
                        case "Aircraft.HurricaneMkI_dH5-20_100oct": ab.aircraftType = "Aircraft.HurricaneMkI_dH5-20"; break;
                        case "Aircraft.HurricaneMkI_100oct": ab.aircraftType = "Aircraft.HurricaneMkI"; break;
                        case "Aircraft.SpitfireMkI_100oct": ab.aircraftType = "Aircraft.SpitfireMkI"; break;
                        case "Aircraft.SpitfireMkIa_100oct": ab.aircraftType = "Aircraft.SpitfireMkIa"; break;
                    };
                }
                else
                {
                    switch (ab.aircraftType)
                    {
                        case "Aircraft.HurricaneMkI_dH5-20": ab.aircraftType = "Aircraft.HurricaneMkI_dH5-20_100oct"; break;
                        case "Aircraft.HurricaneMkI": ab.aircraftType = "Aircraft.HurricaneMkI_100oct"; break;
                        case "Aircraft.SpitfireMkI": ab.aircraftType = "Aircraft.SpitfireMkI_100oct"; break;
                        case "Aircraft.SpitfireMkIa": ab.aircraftType = "Aircraft.SpitfireMkIa_100oct"; break;
                    };
                };
                string sqn = ab.sqn;
                if (sqn.Contains("_Early"))
                    sqn = sqn.Remove(sqn.IndexOf("_Early"));

                bool update = false;
                if (ab.army == LW)
                {
                    if (ab.type.Contains("Fighter") && misLWDeadF > rnd.Next(0, 10)) update = true;
                    if (ab.type.Contains("Bomber") && misLWDeadB > rnd.Next(0, 10)) update = true;
                    if (ab.type.Contains("Zerstorer") && misLWDeadF > rnd.Next(0, 10)) update = true;
                    if (ab.type.Contains("Stuka") && misLWDeadS > rnd.Next(0, 10)) update = true;
                };
                if (ab.army == RAF)
                {
                    if (ab.type.Contains("Fighter") && misRAFDeadF > rnd.Next(0, 10)) update = true;
                    if (ab.type.Contains("Bomber") && misRAFDeadB > rnd.Next(0, 10)) update = true;
                };
                if (activatedGroups.Contains(ab.sqn)) update = false; // active in mission
                if (update) ab.maxPlanes--;

                if (ab.maxPlanes > 2 && ab.army == GamePlay.gpFrontArmy(ab.x, ab.y))
                    sw.WriteLine("AirGroup," + ab.sector + "," + sqn + "," + ab.maxPlanes.ToString() + "," + ab.airportName + "," + ab.aircraftType);
                else
                {
                    mainStoryList.Add(ab.name + " was withdrawn.");
                    if (ab.name == homeBase.name) forceSwitchSqn = true;
                };
                forEachIdx++;
            };

            if (bombsOnTarget > 0)
            {
                string story = "";
                if (targetAirport != null)
                {
                    string strDmg = AirportsDamaged.Find(x => x.Contains(targetAirport.Name()));
                    int pos = AirportsDamaged.IndexOf(strDmg);
                    double percent = 100 * (bombsOnTarget / (targetAirport.CoverageR() * targetAirport.CoverageR() / 100) / 2); // 50% is a 2500kg bombload on a 500m base (500*500/100)
                    if (percent > 100) percent = 100;
                    if (pos == -1)
                    {   // add new entry
                        AirportsDamaged.Add(targetAirport.Name() + "," + percent.ToString("F0"));
                        if (ATK == LW) story = "Luftwaffe bombed " + targetAirport.Name().Replace('_', ' ') + ", " + percent.ToString("F0") + "% damaged.";
                        else story = "RAF bombed " + targetAirport.Name().Replace('_', ' ') + ", " + percent.ToString("F0") + "% damaged.";
                    }
                    else
                    {   // update existing entry
                        int prevDmg = 0;
                        if (int.TryParse(strDmg.Substring(strDmg.IndexOf(",") + 1), out prevDmg))
                        {
                            AirportsDamaged[pos] = targetAirport.Name() + "," + (prevDmg + percent).ToString();
                            if (ATK == LW) story = "Luftwaffe bombed " + targetAirport.Name().Replace('_', ' ') + " again, now +" + percent.ToString("F0") + "% damaged to a total of " + (prevDmg + percent).ToString("F0") + "%";
                            else story = "RAF bombed " + targetAirport.Name().Replace('_', ' ') + " again, now +" + percent.ToString("F0") + "% damaged to a total of " + (prevDmg + percent).ToString("F0") + "%";
                        };
                    };
                };
                if (mainGSTarget != null)
                {
                    Point2d pt2d = new Point2d(mainGSTarget.pos.x, mainGSTarget.pos.y);
                    if (mainGSTarget.Title.Contains("EnglishRadar3"))
                    {
                        string areaName = ChainHomeRadarName(pt2d);
                        string strDmg = CHRadarDamaged.Find(x => x.Contains(mainGSTarget.Name));
                        int pos = CHRadarDamaged.IndexOf(strDmg);
                        double percent = 100 * (bombsOnTarget / 10000); // 100% is a 10000kg bombload total
                        if (percent > 100) percent = 100;
                        if (pos == -1)
                        {   // add new entry                            
                            CHRadarDamaged.Add(mainGSTarget.Name + "," + percent.ToString("F0"));
                            if (ATK == LW) story = "Luftwaffe bombed " + areaName + ", " + percent.ToString("F0") + "% damaged.";
                            else story = "RAF bombed " + areaName + ", " + percent.ToString("F0") + "% damaged.";
                        }
                        else
                        {   // update existing entry
                            int prevDmg = 0;
                            if (int.TryParse(strDmg.Substring(strDmg.IndexOf(",") + 1), out prevDmg))
                            {
                                if (prevDmg + percent > 100) percent = 100 - prevDmg;
                                CHRadarDamaged[pos] = mainGSTarget.Name + "," + (prevDmg + percent).ToString();
                                if (ATK == LW) story = "Luftwaffe bombed " + areaName + " again, now +" + percent.ToString("F0") + "% damaged to a total of " + (prevDmg + percent).ToString() + "%";
                                else story = "RAF bombed " + areaName + " again, now +" + percent.ToString("F0") + "% damaged to a total of " + (prevDmg + percent).ToString() + "%";
                            };
                        };
                    };
                    if (mainGSTarget.Title.Contains("PortCrane1_UK1"))
                    {
                        string areaName = PortDockName(pt2d);
                        string strDmg = PortDamaged.Find(x => x.Contains(mainGSTarget.Name));
                        int pos = PortDamaged.IndexOf(strDmg);
                        double percent = 100 * (bombsOnTarget / 10000); // 100% is a 10000kg bombload total
                        if (percent > 100) percent = 100;
                        if (pos == -1)
                        {   // add new entry
                            PortDamaged.Add(mainGSTarget.Name + "," + percent.ToString("F0"));
                            if (ATK == LW) story = "Luftwaffe bombed " + areaName + ", " + percent.ToString("F0") + "% damaged.";
                            else story = "RAF bombed " + areaName + ", " + percent.ToString("F0") + "% damaged.";
                            if (percent > 50) ReinforcementDate = ReinforcementDate.AddDays(1);
                            if (percent > 50) highOctaneDate = highOctaneDate.AddDays(1);
                        }
                        else
                        {   // update existing entry
                            int prevDmg = 0;
                            if (int.TryParse(strDmg.Substring(strDmg.IndexOf(",") + 1), out prevDmg))
                            {
                                if (prevDmg + percent > 100) percent = 100 - prevDmg;
                                PortDamaged[pos] = mainGSTarget.Name + "," + (prevDmg + percent).ToString();
                                if (ATK == LW) story = "Luftwaffe bombed " + areaName + " again, now +" + percent.ToString() + "% damaged to a total of " + (prevDmg + percent).ToString() + "%";
                                else story = "RAF bombed " + areaName + " again, now +" + percent.ToString() + "% damaged to a total of " + (prevDmg + percent).ToString() + "%";
                                if (percent > 50) ReinforcementDate = ReinforcementDate.AddDays(1);
                                if (percent > 50) highOctaneDate = highOctaneDate.AddDays(1);
                            };
                        };
                    };
                    if (mainGSTarget.Title.Contains("Oxygen_Facility"))
                    {
                        string areaName = "Industry Area";
                        string strDmg = FactoryDamaged.Find(x => x.Contains(mainGSTarget.Name));
                        int pos = FactoryDamaged.IndexOf(strDmg);
                        double percent = 100 * (bombsOnTarget / 10000); // 100% is a 10000kg bombload total
                        if (percent > 100) percent = 100;
                        if (pos == -1)
                        {   // add new entry                            
                            FactoryDamaged.Add(mainGSTarget.Name + "," + percent.ToString("F0"));
                            if (ATK == LW) story = "Luftwaffe bombed " + areaName + ", " + percent.ToString("F0") + "% damaged.";
                            else story = "RAF bombed " + areaName + ", " + percent.ToString("F0") + "% damaged.";
                            ReinforcementDate = ReinforcementDate.AddDays(7 * (percent / 100));
                        }
                        else
                        {   // update existing entry
                            int prevDmg = 0;
                            if (int.TryParse(strDmg.Substring(strDmg.IndexOf(",") + 1), out prevDmg))
                            {
                                if (prevDmg + percent > 100) percent = 100 - prevDmg;
                                FactoryDamaged[pos] = mainGSTarget.Name + "," + (prevDmg + percent).ToString("F0");
                                if (ATK == LW) story = "Luftwaffe bombed " + areaName + " again, now +" + percent.ToString("F0") + "% damaged to a total of " + (prevDmg + percent).ToString("F0") + "%";
                                else story = "RAF bombed " + areaName + " again, now +" + percent.ToString("F0") + "% damaged to a total of " + (prevDmg + percent).ToString("F0") + "%";
                                ReinforcementDate = ReinforcementDate.AddDays(7 * (percent / 100));
                            };
                        };
                    };
                };
                if (story.Length > 0)
                    mainStoryList.Add(story);
            };

            List<string> AirportsDamagedUpdate = new List<string>();
            foreach (string apdmg in AirportsDamaged)
            {
                string[] strs = apdmg.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                int dmg = 0;
                if (int.TryParse(strs[1], out dmg))
                {
                    dmg -= 10 * addDays;
                    if (dmg > 0)
                        AirportsDamagedUpdate.Add(strs[0] + "," + dmg.ToString());
                };
            };
            foreach (string str in AirportsDamagedUpdate)
                sw.WriteLine("AirportDamage," + str);
            AirportsDamaged = AirportsDamagedUpdate;

            List<string> CHRadarDamagedUpdate = new List<string>();
            foreach (string chdmg in CHRadarDamaged)
            {
                string[] strs = chdmg.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                int dmg = 0;
                if (int.TryParse(strs[1], out dmg))
                {
                    dmg -= rnd.Next(10, 33) * addDays;
                    if (dmg > 0)
                        CHRadarDamagedUpdate.Add(strs[0] + "," + dmg.ToString());
                };
                if (dmg < 1) GndStatRestore.Add(strs[0]);
            };
            foreach (string str in CHRadarDamagedUpdate)
                sw.WriteLine("CHRadarDamage," + str);
            CHRadarDamaged = CHRadarDamagedUpdate;

            List<string> PortDamagedUpdate = new List<string>();
            foreach (string pdmg in PortDamaged)
            {
                string[] strs = pdmg.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                int dmg = 0;
                if (int.TryParse(strs[1], out dmg))
                {
                    dmg -= 20 * addDays;
                    if (dmg > 0)
                        PortDamagedUpdate.Add(strs[0] + "," + dmg.ToString());
                };
                if (dmg < 1) GndStatRestore.Add(strs[0]);
            };
            foreach (string str in PortDamagedUpdate)
                sw.WriteLine("PortDamage," + str);
            PortDamaged = PortDamagedUpdate;

            List<string> FactoryDamagedUpdate = new List<string>();
            foreach (string fdmg in PortDamaged)
            {
                string[] strs = fdmg.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                int dmg = 0;
                if (int.TryParse(strs[1], out dmg))
                {
                    dmg -= 10 * addDays;
                    if (dmg > 0)
                        FactoryDamagedUpdate.Add(strs[0] + "," + dmg.ToString());
                };
                if (dmg < 1) GndStatRestore.Add(strs[0]);
            };
            foreach (string str in FactoryDamagedUpdate)
                sw.WriteLine("FactoryDamage," + str);
            FactoryDamaged = FactoryDamagedUpdate;

            foreach (string str in GndStatDead)
                if (!GndStatRestore.Contains(str))
                    sw.WriteLine("GndStatDead," + str);
            foreach (string str in GndGrpDead)
                sw.WriteLine("GndGrpDead," + str); // ships permanent
            foreach (DynCampEvent e in DynCampEvents)
                sw.WriteLine("Event," + e.dateTime.ToString("D") + "," + e.action + "," + e.type + "," + e.typeValue + "," + e.airportName + "," + e.param1 + "," + e.param2 + "," + e.param3);
        };
        //if (odenDebug) forceSwitchSqn = true;
        if (forceSwitchSqn)
        {
            if (odenDebug) GamePlay.gpLogServer(null, "forceSwitchSqn: {0} {1}", new object[] { homeBase.airportName, homeBase.aircraftType });
            List<MissionAirbase> mabs = SortedAirbases(homeBase.army, new Point2d(homeBase.x, homeBase.y), false, true); // F bases in battlearea
            oldSqn = homeBase.sqn;
            string name = homeBase.name;
            foreach (MissionAirbase mab in mabs)
            {
                if (mab.aircraftType == homeBase.aircraftType)
                {
                    homeBase = mab;
                    break;
                };
            };
            // try similar planetype
            if (name == homeBase.name)
                if (homeBase.army == LW)
                {
                    if (homeBase.country == "de")
                    {
                        if (homeBase.type.Contains("Fighter"))
                            foreach (MissionAirbase mab in mabs)
                                if (LWFighterSet.Contains(mab.aircraftType))
                                {
                                    homeBase = mab;
                                    break;
                                };
                        if (homeBase.type.Contains("Zerstorer"))
                            foreach (MissionAirbase mab in mabs)
                                if (LWFighterSet.Contains(mab.aircraftType))
                                {
                                    homeBase = mab;
                                    break;
                                };
                        if (homeBase.type.Contains("Bomber"))
                            foreach (MissionAirbase mab in mabs)
                                if (LWBomberSet.Contains(mab.aircraftType))
                                {
                                    homeBase = mab;
                                    break;
                                };
                    };
                    if (homeBase.country == "it")
                    {
                        if (homeBase.type.Contains("Fighter"))
                            foreach (MissionAirbase mab in mabs)
                                if (RAFighterSet.Contains(mab.aircraftType))
                                {
                                    homeBase = mab;
                                    break;
                                };
                        if (homeBase.type.Contains("Bomber"))
                            foreach (MissionAirbase mab in mabs)
                                if (RABomberSet.Contains(mab.aircraftType))
                                {
                                    homeBase = mab;
                                    break;
                                };
                    };
                }
                else
                {   // RAF
                    if (homeBase.country == "gb")
                    {
                        if (homeBase.type.Contains("Fighter"))
                            foreach (MissionAirbase mab in mabs)
                                if (RAFFighterSet.Contains(mab.aircraftType))
                                {
                                    homeBase = mab;
                                    break;
                                };
                        if (homeBase.type.Contains("Bomber"))
                            foreach (MissionAirbase mab in mabs)
                                if (RAFBomberSet.Contains(mab.aircraftType))
                                {
                                    homeBase = mab;
                                    break;
                                };
                    };
                };

            if (name != homeBase.name)
            {
                mainStoryList.Add("Pilots of " + name + " were transferred to " + homeBase.name);
                foreach (AiAirport ap in GamePlay.gpAirports())
                    if (ap.Name() == homeBase.airportName)
                    {
                        homeBase.MoveSquadronToAirport(ap);
                        break;
                    };
            };

            if (odenDebug) GamePlay.gpLogServer(null, "forceSwitchSqn: {0} {1}", new object[] { homeBase.airportName, homeBase.aircraftType });
        };

        GameOver = BattleOfBritainVictoryCheck();
    }

    private void UpdateBriefing()
    {
        string strBase = homeBase.airportName.Replace('_', ' ');
        if (strBase == "Sounthend")
            strBase = "Rochford";

        //if (odenDebug) GamePlay.gpLogServer(null, "UpdateBriefing: {0} {1}", new object[] { homeBase.airportName, homeBase.aircraftType });

        if (misConvoyKills == 1) mainStoryList.Add(misConvoyKills.ToString() + " Convoy Ship was sunk - 100 octane fuel delayed one day.");
        else if (misConvoyKills > 0) mainStoryList.Add(misConvoyKills.ToString() + " Convoy Ships were sunk - 100 octane fuel delayed for as many days.");

        if (misRAFDeadF > 0) mainStoryList.Add("RAF lost " + misRAFDeadF.ToString() + " fighters.");
        if (misLWDeadF + misLWDeadB + misLWDeadS > 0) mainStoryList.Add("Luftwaffe lost " + misLWDeadF.ToString() + " fighters, " + (misLWDeadB + misLWDeadS).ToString() + " bombers.");
        if (mainStoryList.Count == 0)
        {
            if (addDays == 1) mainStoryList.Add("The previous day was calm with no reports of hostilities.");
            else mainStoryList.Add("The previous " + addDays.ToString() + " days saw no enemy bombers over Britain.");
            mainStoryList.Add(" One could almost think the war was over but they say there is calm before the storm.");
        };
        List<string> campStatus = new List<string>();
        if (defPlayer.Contains("RAF"))
        {
            campStatus.Add("Next reinforcement: " + ReinforcementDate.ToString("D") + ".");
            if (highOctaneDate > gameDateTime) campStatus.Add("Next deliveries of 100 octane fuel: " + highOctaneDate.ToString("D"));
            campStatus.Add("");
        };
        if (MissionMarkers[idxDungeness].army == LW) campStatus.Add("Dungeness area is in German control.");
        if (MissionMarkers[idxDover].army == LW) campStatus.Add("Dover is in German control.");
        if (MissionMarkers[idxRamsgate].army == LW) campStatus.Add("Ramsgate is in German control.");
        if (MissionMarkers[idxCanterbury].army == LW) campStatus.Add("Canterbury is in German control.");
        if (MissionMarkers[idxRochester].army == LW) campStatus.Add("Rochester is in German control.");
        if (MissionMarkers[idxLondon].army == LW) campStatus.Add("London is in German control.");

        campStatus.Add("");
        foreach (string chdmg in CHRadarDamaged)
        {
            string[] strs = chdmg.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            GroundStationary gs = GroundStationaryByName(strs[0]);
            if (gs != null)
            {
                string areaName = ChainHomeRadarName(new Point2d(gs.pos.x, gs.pos.y));
                int dmg = 0;
                if (int.TryParse(strs[1], out dmg))
                    if (dmg > 0)
                        campStatus.Add(areaName + " Chain Home Radar Station is " + dmg.ToString() + "% damaged.");
            };
        };

        foreach (string pdmg in PortDamaged)
        {
            string[] strs = pdmg.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            GroundStationary gs = GroundStationaryByName(strs[0]);
            if (gs != null)
            {
                string areaName = PortDockName(new Point2d(gs.pos.x, gs.pos.y));
                int dmg = 0;
                if (int.TryParse(strs[1], out dmg))
                    if (dmg > 0)
                        campStatus.Add(areaName + " is " + dmg.ToString() + "% damaged.");
            };
        };

        foreach (string fdmg in FactoryDamaged)
        {
            string[] strs = fdmg.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            GroundStationary gs = GroundStationaryByName(strs[0]);
            if (gs != null)
            {
                string areaName = "Industry Area";
                int dmg = 0;
                if (int.TryParse(strs[1], out dmg))
                    if (dmg > 0)
                        campStatus.Add(areaName + " is " + dmg.ToString() + "% damaged.");
            };
        };

        foreach (string apdmg in AirportsDamaged)
        {
            string[] strs = apdmg.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            int dmg = 0;
            if (int.TryParse(strs[1], out dmg))
                if (dmg > 0)
                    campStatus.Add(strs[0].Replace('_', ' ') + " is " + dmg.ToString() + "% damaged.");
        };

        campStatus.Add("");
        int gbSqnsF = 0, gbSqnsB = 0, gbTotal = 0, deGrpsF = 0, deGrpsB = 0, deGrpsS = 0, deTotal = 0;
        foreach (MissionAirbase mab in MissionAirbases)
            if (mab.army == RAF)
            {
                if (mab.type.Contains("Fighter")) gbSqnsF += 1;
                if (mab.type.Contains("Bomber")) gbSqnsB += 1;
                gbTotal += mab.maxPlanes;
            }
            else
            {
                if (mab.type.Contains("Fighter")) deGrpsF += 1;
                if (mab.type.Contains("Bomber")) deGrpsB += 1;
                if (mab.type.Contains("Stuka")) deGrpsS += 1;
                deTotal += mab.maxPlanes;
            };
        campStatus.Add("RAF air assets: " + gbTotal.ToString() + " aircrafts in " + gbSqnsF.ToString() + " fighter- and " + gbSqnsB.ToString() + " bomber squadrons");
        campStatus.Add("Luftwaffe air assets: " + deTotal.ToString() + " aircrafts in " + deGrpsF.ToString() + " fighter-, " + gbSqnsB.ToString() + " bomber- and " + deGrpsS.ToString() + " Stuka Gruppen's");
        double ratio = deTotal / gbTotal;
        campStatus.Add("");
        campStatus.Add("Luftwaffe : RAF ratio is " + ratio.ToString("F1"));
        if (ratio >= 3) campStatus.Add("(Luftwaffe is more than 3:1 and will launch 'Sea Lion' in September)");
        else
            if (ratio >= 2) campStatus.Add("(Luftwaffe is more than 2:1 but will postpone 'Sea Lion' by end of October)");
            else
                if (ratio < 1) campStatus.Add("(RAF defeated Luftwaffe and will launch airfield strikes (Circus Ops in 1941))");
                else campStatus.Add("(Luftwaffe is less than 2:1 and will postpone 'Sea Lion' by end of October)");
        campStatus.Add("");

        int inds = Industries.Count;
        if (inds == 1) campStatus.Add("Only one industry area is still operational");
        else campStatus.Add(inds + " industrial areas are operational"); // if down to zero UK has another problem :)        

        if (ChainHomeRadars.Count > 0)
        {
            campStatus.Add("");
            campStatus.Add("Chain Home Radar Stations still operational (" + ChainHomeRadars.Count.ToString() + "):");
            foreach (GroundStationary gs in ChainHomeRadars)
                campStatus.Add(" -" + ChainHomeRadarName(new Point2d(gs.pos.x, gs.pos.y)));
        };
        if (PortCranes.Count > 0)
        {
            campStatus.Add("");
            campStatus.Add("Port Facilities still operational (" + PortCranes.Count.ToString() + "):");
            foreach (GroundStationary gs in PortCranes)
                campStatus.Add(" -" + PortDockName(new Point2d(gs.pos.x, gs.pos.y)));
        };

        string reinforcements = "";
        if (homeBase.maxPlanes < 13) reinforcements = ")";
        if (homeBase.maxPlanes == 13) reinforcements = " with one extra plane available)";
        if (homeBase.maxPlanes == 14) reinforcements = " with two extra planes available)";
        if (homeBase.maxPlanes == 15) reinforcements = " with three extra planes available)";
        if (homeBase.maxPlanes > 15) reinforcements = " with many planes available)";

        string recentEventsCaption = "Yesterday events:";
        if (addDays > 1) recentEventsCaption = "Recent events last " + addDays.ToString() + " days:";

        if (odenDebug) GamePlay.gpLogServer(null, "UpdateBriefing: {0} {1}", new object[] { homeBase.airportName, homeBase.aircraftType });
        int len = 6;
        if (homeBase.aircraftType.Length < 9 + 6)
            len = homeBase.aircraftType.Length - 9;
        string type = "DynMis40" + homeBase.aircraftType.Substring(9, len).ToLower();
        string slide = type + ".jpg";
        string thin = type + "_thin.jpg";

        string[] strBriefing =
        {
            "[Info]"
            , "Info"
            , "<Slide>"
            , slide
            , ""
            , "<Caption>"
            , missionCaption
            , "<Slide>"
            , thin
            , ""
            , "<Caption>"
            , "Battle of Britain"
            , gameDateTime.ToString("D") //"July 10th 1940"
            , ""
            , homeBase.sector //"RAF Hornchurch"
            , homeBase.name +" (" +strBase +")"
            , "[Mission]"
            , "<Name>"
            , "Mission"
            , "<Description>"
            , "Battle of Britain"
            , gameDateTime.ToString("D") //"July 10th 1940"
            , ""
            , homeBase.sector //"RAF Hornchurch"
            , homeBase.name +" (" +strBase +reinforcements
            , Designation(homeBase.aircraftType) //"Spitfire Mk Ia"
            , tailNoSerialNo
            , "check your callsign in <Player Plane>"
            , ""
            , recentEventsCaption
        };
        string strSat = "";
        int cnt = Satellites.Count;
        if (cnt > 0)
        {
            strSat = "(You may land at ";
            for (int pos = 0; pos < cnt - 1; pos++)
            {
                if (Satellites[pos] == "Sounthend") Satellites[pos] = "Rochford";
                if (pos == 0) strSat += Satellites[pos];
                else strSat += ", " + Satellites[pos];
            };
            strSat += " or " + Satellites[cnt - 1] + " to change airfield during the campaign)";
        };
        if (GameOver > 0) strSat = "Battle of Britain is over, but you may proceed flying if you choose so, but to restart: remove the DynMis-ini file in [" + DoverFolder + "] and start the mission once just to back out to main menu again getting all files restored.";

        string[] endBriefing = 
        {
            //"(You may land at Hornchurch, Rochford or Manston to change airfield during the campaign)"
            "<Slide>"
            , slide
            , "<Caption>"
            , ""
        };

        if (MissionMarkers[idxLondon].army == LW)
        {
            campStatus.Clear();
            campStatus.Add("London is in German control.");
            campStatus.Add("");
            campStatus.Add("");
            campStatus.Add("Mission Files will reset as you enter cockpit.");
            campStatus.Add("");
            campStatus.Add("When you see the reset-notification.");
            campStatus.Add("Exit out to Main Menu and start the mission again.");
            campStatus.Add("");
            campStatus.Add("You will be back at 10 July 1940 for a new adventure.");
            strSat = "";
        };

        using (StreamWriter sw = File.CreateText(CampBriFile))
        {
            foreach (string str in strBriefing)
                sw.WriteLine(str);
            foreach (string str in mainStoryList)
                sw.WriteLine(str);
            sw.WriteLine("");
            sw.WriteLine("Status:");
            foreach (string str in campStatus)
                sw.WriteLine(str);
            sw.WriteLine("");
            if (strSat.Length > 0)
                sw.WriteLine(strSat);
            foreach (string str in endBriefing)
                sw.WriteLine(str);
        };
    }

    private void UpdateMissionFile()
    {
        if (odenDebug) GamePlay.gpLogServer(null, "UpdateMissionFile: {0} {1}", new object[] { homeBase.airportName, homeBase.aircraftType });
        string[] strMisFile = File.ReadAllLines(CampMisFile);
        List<string> strMisList = new List<string>();

        bool setHomeBaseTakeOff = false;
        Point2d wp2d = new Point2d(homeBase.x, homeBase.y);
        if (homeBase.army == RAF) wp2d = calcPos(wp2d, 3 * 1609, 90 + rnd.Next(0, 90));
        else wp2d = calcPos(wp2d, 3 * 1609, 270 + rnd.Next(0, 90));
        int aging = (gameDateTime - BoBDateTime).Days;
        if (aging > 99) aging = 99;
        /*
        string firstSection0 = "  Flight0 1 2 3";
        string firstSection1 = "  Flight1 4 5 6";
        string otherSection0 = "  Flight0 7 8 9";
        string otherSection1 = "  Flight1 10 11 12";
        */
        string form = "VIC3";
        if (homeBase.type.Contains("Stuka")) form = "VIC";
        if (homeBase.type.Contains("Zerstorer")) form = "FINGERFOUR";
        if (homeBase.type.Contains("Fighter")) form = "FINGERFOUR";
        if (homeBase.type.Contains("Recon")) form = "LINEABREAST";
        if (homeBase.type.Contains("Bomber") && homeBase.sqn.Contains("RAF_")) form = "VIC";
        if (homeBase.type.Contains("Fighter") && homeBase.sqn.Contains("Early")) form = "VIC3";
        if (homeBase.type.Contains("Fighter") && homeBase.sqn.Contains("RA_")) form = "VIC3";

        int max = homeBase.maxPlanes;
        if (max > 12) max = 12;
        int Flight1Qty = max / 2;
        if (form == "FINGERFOUR") Flight1Qty = max * 2 / 3;
        int Flight2Qty = max - Flight1Qty;

        string sqn = homeBase.sqn;
        if (oldSqn != "") sqn = oldSqn;
        bool dynMisSectionFound = false;
        string slot = defPlayer.Substring(defPlayer.IndexOf(".") + 1); // 010 for 2nd flight 1st slot, 000 for 1st/1st, 003 for 1st/3rd, 110 for 2nd/1st of 2nd section
        // LW usually has 4 sections/staffeln(=12), Stab./1./2./3. in I/JG and Stab./4./5./6. in II/JG and so on. 100 is leader in Stab., 200 is leader in 1./JG and 300 is leader in 2./JG (all from I./JG)
        int iSect = int.Parse(slot.Substring(0, 1)); // [BoB_LW_JG26_II.13] block, used as flight (12-ship RAF Sqn is split in 2x6-ship flights of 2x3-ship sections)
        int iFlgt = int.Parse(slot.Substring(1, 1)); // actually section in flight (Flight0, Flight1 in mis-file)
        int iSlot = int.Parse(slot.Substring(2, 1)); // index of what tailNo is player, "Flight1 5 6 7 8" where player is #5 (or E) tail is slot/index 0 (and #8 is slot/index 3)

        // player slot is not available, move towards flightleader (player should be #1slot in 2nd section and fly as section leader until sqn depleted)
        // VIC3 3-ship sections
        slot = defPlayer.Substring(defPlayer.IndexOf(".") + 2);
        if (Flight1Qty < 4) slot = "02"; // 1st section (Flight0) in slot 3
        if (Flight1Qty < 3) slot = "01"; // 1st section (Flight0) in slot 2
        if (Flight1Qty < 2) slot = "00"; // 1st section (Flight0) in slot 1 (alone)
        if (form == "FINGERFOUR")
        {   // normal 4-ship sections (what we call flight today - flightleader/wingman/sectionleader/wingman)
            if (Flight1Qty < 5) slot = "03"; // 1st section (Flight0) in slot 4 (Schwarm of 2x2 Rotte)
            if (Flight1Qty < 4) slot = "02"; // 1st section (Flight0) in slot 3
            if (Flight1Qty < 3) slot = "01"; // 1st section (Flight0) in slot 2
            if (Flight1Qty < 2) slot = "00"; // 1st section (Flight0) in slot 1 (alone)
        };
        slot = iSect.ToString() + slot; // restore first index of what mis-file block player is in

        if (odenDebug) GamePlay.gpLogServer(null, "UpdateMissionFile: {0} {1}+{2}", new object[] { form, Flight1Qty, Flight2Qty });

        string form2 = form;
        if (homeBase.army == RAF && form == "VIC3" && max > 6) form2 = "VIC6";
        string[] FlightSection = FlightSections(homeBase.army, max, form, 0);

        if (odenDebug) GamePlay.gpLogServer(null, "FlightSections: '{0}' '{1}' '{2}' '{3}'", new object[] { FlightSection[0], FlightSection[1], FlightSection[2], FlightSection[3] });

        int qty0 = (FlightSection[0].Length + 1) / 2;
        int qty1 = (FlightSection[1].Length + 1) / 2;
        int qty2 = (FlightSection[2].Length + 1) / 2;
        int qty3 = (FlightSection[3].Length + 1) / 2;
        int flightsInFirst = 1;
        if (Flight1Qty > qty1) flightsInFirst++;
        if (Flight1Qty > qty1 + qty2) flightsInFirst++;
        if (Flight1Qty > qty1 + qty2 + qty3) flightsInFirst++;
        // FlightSection[0] and [1] is for first Flight, [2] and [3] for 2nd Flight sections called "Flight0" and "Flight1"
        string Flight0 = FlightSection[0];
        string Flight1 = FlightSection[1];
        string Flight2 = FlightSection[2];
        string Flight3 = FlightSection[3];
        if (flightsInFirst < 4) Flight3 = "";
        if (flightsInFirst < 3) Flight2 = "";
        if (flightsInFirst < 2) Flight1 = "";
        string Second0 = "";
        string Second1 = "";
        string Second2 = "";
        string Second3 = "";
        if (flightsInFirst == 1) { Second0 = FlightSection[1]; Second1 = FlightSection[2]; Second2 = FlightSection[3]; };
        if (flightsInFirst == 2) { Second0 = FlightSection[2]; Second1 = FlightSection[3]; };
        if (flightsInFirst == 3) Second0 = FlightSection[3];

        FlightSection = FlightSections(homeBase.army, Flight1Qty, form, iSect);
        string Flight1Name = homeBase.sqn + "." + FlightSection[4];
        FlightSection = FlightSections(homeBase.army, Flight2Qty, form, iSect + 1);
        string Flight2Name = homeBase.sqn + "." + FlightSection[4];

        bool isFirstFlight = false; // will flip immediately at AirGroups
        bool airStart = false; // AI fails pretty often to taxi/takeoff

        foreach (string str in strMisFile)
        {
            if (str.Length > 0)
            {
                string[] strs = str.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                if (strs.Length > 0)
                {
                    string newStr = str;
                    // keep squadron/gruppe up to date if switched ditto
                    if (!dynMisSectionFound)
                        if (str.Contains("_Way"))
                        {   //[BoB_LW_JG26_II.13_Way]
                            setHomeBaseTakeOff = true;
                            if (isFirstFlight) newStr = "[" + Flight1Name + "_Way]";
                            else newStr = "[" + Flight2Name + "_Way]";
                        }
                        else
                        {
                            if (str.Contains(sqn) && !str.Contains("player"))
                            {   //[BoB_LW_JG26_II.13] or BoB_LW_JG26_II.13 in [AirGroups]
                                isFirstFlight = !isFirstFlight;
                                if (str.Contains("["))
                                {
                                    if (isFirstFlight) newStr = "[" + Flight1Name + "]";
                                    else newStr = "[" + Flight2Name + "]";
                                }
                                else
                                    if (isFirstFlight) newStr = "  " + Flight1Name;
                                    else newStr = "  " + Flight2Name;
                            };
                        };

                    switch (strs[0])
                    {
                        case "MAP":
                            newStr = "  MAP Land$English_Channel_1940_Winter";
                            if (gameDateTime.Month > 4)
                                newStr = "  MAP Land$English_Channel_1940";
                            if (gameDateTime.Month > 9)
                                newStr = "  MAP Land$English_Channel_1940_Autumn";
                            if (gameDateTime.Month > 10)
                                newStr = "  MAP Land$English_Channel_1940_Winter";
                            break;
                        case "BattleArea":
                            if (homeBase.army == RAF) newStr = "  BattleArea 115000 175000 193080 128720 16090";
                            else newStr = "  BattleArea 100000 175000 180000 140000 20000";
                            break;
                        case "TIME":
                            newStr = "  TIME " + rnd.Next(7, 19).ToString() + "." + rnd.Next(0, 99).ToString();
                            break;
                        case "WeatherIndex":
                            newStr = "  WeatherIndex " + rnd.Next(0, 2).ToString();
                            break;
                        case "CloudsHeight":
                            newStr = "  CloudsHeight " + rnd.Next(5, 12).ToString() + "00";
                            break;
                        case "player":
                            newStr = "  player " + homeBase.sqn + "." + slot;
                            break;
                        case "Power":
                            newStr = "  Power " + rnd.Next(-7, 7).ToString("F3") + " " + rnd.Next(-7, 7).ToString("F3") + " 0.000";
                            break;
                        case "GustPower":
                            newStr = "  GustPower " + rnd.Next(1, 9).ToString();
                            break;
                        case "GustAngle":
                            newStr = "  GustAngle " + rnd.Next(-40, 40).ToString();
                            break;
                        case "Flight0":
                            newStr = "  Flight0 " + Flight0;
                            if (!isFirstFlight) newStr = "  Flight0 " + Second0;
                            break;
                        case "Flight1":
                            newStr = "  Flight1 " + Flight1;
                            if (!isFirstFlight) newStr = "  Flight1 " + Second1;
                            break;
                        case "Flight2":
                            newStr = "  Flight2 " + Flight2;
                            if (!isFirstFlight) newStr = "  Flight2 " + Second2;
                            break;
                        case "Flight3":
                            newStr = "  Flight3 " + Flight3;
                            if (!isFirstFlight) newStr = "  Flight3 " + Second3;
                            break;
                        case "Class":
                            newStr = "  Class " + homeBase.aircraftType;
                            break;
                        case "Aging":
                            newStr = "  Aging " + aging.ToString();
                            break;
                        case "TAKEOFF":
                            newStr = "  TAKEOFF " + homeBase.x.ToString("F2") + " " + homeBase.y.ToString("F2") + " 0 0";                            
                            break;
                        case "NORMFLY":
                            newStr = "  NORMFLY " + wp2d.x.ToString("F2") + " " + wp2d.y.ToString("F2") + " 914.00 288.00";
                            if (airStart) newStr = "  NORMFLY " + homeBase.x.ToString("F2") + " " + homeBase.y.ToString("F2") + " 914.00 288.00";
                            airStart = !airStart;
                            break;
                        case "[DYNMIS]":
                            dynMisSectionFound = true; // ignore remaining values regarding updates on homebase
                            break;
                        default:
                            if (strs[0].Contains("FrontMarker"))
                                switch (strs[0])
                                {
                                    case fmDungeness: newStr = "  " + strs[0] + " " + strs[1] + " " + strs[2] + " " + MissionMarkers[idxDungeness].army.ToString(); break;
                                    case fmDover: newStr = "  " + strs[0] + " " + strs[1] + " " + strs[2] + " " + MissionMarkers[idxDover].army.ToString(); break;
                                    case fmRamsgate: newStr = "  " + strs[0] + " " + strs[1] + " " + strs[2] + " " + MissionMarkers[idxRamsgate].army.ToString(); break;
                                    case fmCanterbury: newStr = "  " + strs[0] + " " + strs[1] + " " + strs[2] + " " + MissionMarkers[idxCanterbury].army.ToString(); break;
                                    case fmRochester: newStr = "  " + strs[0] + " " + strs[1] + " " + strs[2] + " " + MissionMarkers[idxRochester].army.ToString(); break;
                                    case fmLondon: newStr = "  " + strs[0] + " " + strs[1] + " " + strs[2] + " " + MissionMarkers[idxLondon].army.ToString(); break;
                                };
                            break;
                    };
                    if (!dynMisSectionFound)
                        if (strs[0].Contains("Skin"))
                            newStr = "  " + strs[0] + " " + DefaultSkins(homeBase.aircraftType, homeBase.sqn);

                    if (newStr.Length > 0)
                        strMisList.Add(newStr);
                };
            };
        };
        if (!dynMisSectionFound)
        {
            strMisList.Add("[DYNMIS]");
            strMisList.Add("  DynMisPlayer " + defPlayer);
            strMisList.Add("  DynMisSquadron " + defSquadron);
            strMisList.Add("  DynMisClass " + defClass);
            strMisList.Add("  DynMisCallSign " + defCallSign);
            strMisList.Add("  DynMisSkin " + defSkin);
            strMisList.Add("  DynMisSerial " + defSerial);
            strMisList.Add("  DynMisTail " + defTail);
        };

        using (StreamWriter sw = File.CreateText(CampMisFile))
            foreach (string str in strMisList)
                sw.WriteLine(str);
    }

    private void GameOverBriefing()
    {
        string strBase = homeBase.airportName.Replace('_', ' ');
        if (strBase == "Sounthend")
            strBase = "Rochford";

        List<string> campStatus = new List<string>();

        int gbSqnsF = 0, gbSqnsB = 0, gbTotal = 0, deGrpsF = 0, deGrpsB = 0, deGrpsS = 0, deTotal = 0;
        foreach (MissionAirbase mab in MissionAirbases)
            if (mab.army == RAF)
            {
                if (mab.type.Contains("Fighter")) gbSqnsF += 1;
                if (mab.type.Contains("Bomber")) gbSqnsB += 1;
                gbTotal += mab.maxPlanes;
            }
            else
            {
                if (mab.type.Contains("Fighter")) deGrpsF += 1;
                if (mab.type.Contains("Bomber")) deGrpsB += 1;
                if (mab.type.Contains("Stuka")) deGrpsS += 1;
                deTotal += mab.maxPlanes;
            };
        campStatus.Add("RAF air assets: " + gbTotal.ToString() + " aircrafts in " + gbSqnsF.ToString() + " fighter- and " + gbSqnsB.ToString() + " bomber squadrons");
        campStatus.Add("Luftwaffe air assets: " + deTotal.ToString() + " aircrafts in " + deGrpsF.ToString() + " fighter-, " + gbSqnsB.ToString() + " bomber- and " + deGrpsS.ToString() + " Stuka Gruppen's");
        campStatus.Add("");
        List<string> AirportsDamagedUpdate = new List<string>();
        foreach (string apdmg in AirportsDamaged)
        {
            string[] strs = apdmg.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            int dmg = int.Parse(strs[1]);
            dmg -= 25;
            if (dmg > 0)
            {
                if (dmg > 50) campStatus.Add(strs[0].Replace('_', ' ') + " is " + dmg.ToString() + "% damaged.");
                else campStatus.Add(strs[0].Replace('_', ' ') + " is " + dmg.ToString() + "% damaged but operational.");
                AirportsDamagedUpdate.Add(strs[0] + "," + dmg.ToString());
            };
        };
        AirportsDamaged = AirportsDamagedUpdate;
        if (AirportsDamagedUpdate.Count > 0)
            campStatus.Add("");

        int inds = Industries.Count;
        if (inds == 1) campStatus.Add("Only one industry area is still operational");
        else campStatus.Add(inds + " industrial areas are operational"); // if down to zero UK has another problem :)        

        if (ChainHomeRadars.Count > 0)
        {
            campStatus.Add("");
            campStatus.Add("Chain Home Radar Stations still operational:");
            foreach (GroundStationary gs in ChainHomeRadars)
                campStatus.Add(" -" + ChainHomeRadarName(new Point2d(gs.pos.x, gs.pos.y)));
        }
        else campStatus.Add("No Chain Home Radar Stations operational.");

        if (PortCranes.Count > 0)
        {
            campStatus.Add("");
            campStatus.Add("Port Facilities still operational:");
            foreach (GroundStationary gs in PortCranes)
                campStatus.Add(" -" + PortDockName(new Point2d(gs.pos.x, gs.pos.y)));
        }
        else campStatus.Add("No Port Facilities operational");

        int len = 6;
        if (homeBase.aircraftType.Length < 9 + 6)
            len = homeBase.aircraftType.Length - 9;
        string type = "DynMis40" + homeBase.aircraftType.Substring(9, len).ToLower();
        string slide = type + ".jpg";
        string thin = type + "_thin.jpg";

        string[] strBriefing =
        {
            "[Info]"
            , "Info"
            , "<Slide>"
            , slide
            , ""
            , "<Caption>"
            , missionCaption
            , "<Slide>"
            , thin
            , ""
            , "<Caption>"
            , "Battle of Britain has ended"
            , gameDateTime.ToString("D")
            , ""
            , "Press 'Fly' to show final status"
            , "[Mission]"
            , "<Name>"
            , "Mission"
            , "<Description>"
            , "Battle of Britain"
            , gameDateTime.ToString("D")
            , ""
            , "-Press 'Fly' to reset status. Then quit the mission and return to main menu to start a new dynamic mission-"
            , ""
        };
        string strSat = "";

        string[] endBriefing = 
        {
            ""
            ,"-Press 'Fly' to reset status. Then quit the mission and return to main menu to start a new dynamic mission-"
            ,"<Slide>"
            , slide
            , "<Caption>"
            , ""
        };

        using (StreamWriter sw = File.CreateText(CampBriFile))
        {
            foreach (string str in strBriefing)
                sw.WriteLine(str);
            foreach (string str in mainStoryList)
                sw.WriteLine(str);
            sw.WriteLine("");
            sw.WriteLine("Status:");
            foreach (string str in campStatus)
                sw.WriteLine(str);
            sw.WriteLine("");
            if (strSat.Length > 0)
                sw.WriteLine(strSat);
            foreach (string str in endBriefing)
                sw.WriteLine(str);
        };
    }

    internal List<AiActor> SortedConvoyShips(int army, Point3d pt3d, bool onlyBattleArea)
    {   // requires frontline
        List<AiActor> SortedConvoyShips = new List<AiActor>();
        string sn = "";
        foreach (AiGroundGroup gg in ConvoyShips)
            if (gg.GetItems().Length > 0)
            {
                AiActor a = gg.GetItems()[0];
                if (onlyBattleArea)
                    sn = GamePlay.gpSectorName(a.Pos().x, a.Pos().y);
                if (a.Army() == army && (sn.IndexOf(">") == -1 && sn.IndexOf("<") == -1))
                {
                    int forEachIndex = 0;
                    foreach (AiActor ship in SortedConvoyShips)
                    {
                        if (ship.Pos().distance(ref pt3d) > a.Pos().distance(ref pt3d))
                        {
                            SortedConvoyShips.Insert(forEachIndex, a);
                            break;
                        };
                        forEachIndex++;
                    };
                    if (!SortedConvoyShips.Contains(a)) SortedConvoyShips.Add(a); // add last
                };
            };
        return SortedConvoyShips;
    }
    private void CreateConvoyMission()
    {
        gameIsOn = true;
        List<AiActor> SortedShips = SortedConvoyShips(RAF, homePlate.Pos(), false);
        AiActor a = SortedShips[0];
        if (SortedShips.Count > 2) a = SortedShips[rnd.Next(0, 3)];

        target3d = a.Pos();
        Point2d pt2d = new Point2d(a.Pos().x, a.Pos().y);
        if (odenDebug) GamePlay.gpLogServer(null, "CreateConvoyMission: '{0}'", new object[] { GamePlay.gpSectorName(a.Pos().x, a.Pos().y) });
        ATK = a.Army() == RAF ? LW : RAF;
        convoyTarget = a;
        Point3d pt3d = a.Pos();
        targetName = (ATK == LW ? "British " : "German ") + "Convoy in " + GamePlay.gpSectorName(a.Pos().x, a.Pos().y);
        ISectionFile f = GamePlay.gpCreateSectionFile();
        if (!playerIsBomber)
        {
            CreateBMBMission(f, pt2d, 3, ATK, a, null, true);
            foreach (AiGroundGroup gg in ConvoyShips)
            {
                if (gg.Pos().distance(ref pt3d) < 16090)
                {
                    a = gg.GetItems()[0];
                    pt2d.x = a.Pos().x;
                    pt2d.y = a.Pos().y;
                    CreateBMBMission(f, pt2d, 3, ATK, a, null, false);
                };
            };
        };
        f.add("Trigger", "BombersAtTarget", "TPassThrough 5 " + ATK.ToString() + " " + pt3d.x.ToString() + " " + pt3d.y.ToString() + " 5000");
        GamePlay.gpPostMissionLoad(f);
        NextMissionNum++;
        RedirectCAP(ATK == LW ? RAF : LW, pt2d);
    }
    internal List<GroundStationary> SortedGroundStationaries(List<GroundStationary> GroundStationaries, int army, Point3d pt3d, bool onlyBattleArea)
    {   // requires frontline
        List<GroundStationary> SortedGroundStationaries = new List<GroundStationary>();
        string sn = "";
        foreach (GroundStationary gs in GroundStationaries)
        {
            if (onlyBattleArea)
                sn = GamePlay.gpSectorName(gs.pos.x, gs.pos.y);
            if (GamePlay.gpFrontArmy(gs.pos.x, gs.pos.y) == army && (sn.IndexOf(">") == -1 && sn.IndexOf("<") == -1))
            {
                int forEachIndex = 0;
                foreach (GroundStationary sgs in SortedGroundStationaries)
                {
                    if (sgs.pos.distance(ref pt3d) > gs.pos.distance(ref pt3d))
                    {
                        SortedGroundStationaries.Insert(forEachIndex, gs);
                        break;
                    };
                    forEachIndex++;
                };
                if (!SortedGroundStationaries.Contains(gs)) SortedGroundStationaries.Add(gs); // add last
            };
        };
        return SortedGroundStationaries;
    }
    private void CreateCHRadarMission()
    {
        gameIsOn = true;
        List<GroundStationary> SortedRadars = SortedGroundStationaries(ChainHomeRadars, RAF, homePlate.Pos(), false);
        GroundStationary gs = SortedRadars[0];
        if (SortedRadars.Count > 2) gs = SortedRadars[rnd.Next(0, 3)];

        target3d = gs.pos;
        mainGSTarget = gs;
        if (gs.country == "gb") ATK = LW;
        else ATK = RAF;
        Point2d pt2d = new Point2d(gs.pos.x, gs.pos.y);
        if (odenDebug) GamePlay.gpLogServer(null, "CreateCHRadarMission: '{0}'", new object[] { gs.Name });
        targetName = (ATK == LW ? "British " : "German ") + "radar station at " + ChainHomeRadarName(pt2d) + " in " + GamePlay.gpSectorName(pt2d.x, pt2d.y);
        ISectionFile f = GamePlay.gpCreateSectionFile();
        if (!playerIsBomber)
        {
            CreateBMBMission(f, pt2d, 3, ATK, null, gs, true);
            CreateBMBMission(f, pt2d, 3, ATK, null, gs, false); // never multiple escorts, they climb to same alt and risk collide
            if (rnd.Next(0, 9) > 6) CreateBMBMission(f, pt2d, 3, ATK, null, gs, false);
            if (rnd.Next(0, 9) > 6) CreateBMBMission(f, pt2d, 3, ATK, null, gs, false);
        };
        f.add("Trigger", "BombersAtTarget", "TPassThrough 5 " + ATK.ToString() + " " + pt2d.x.ToString() + " " + pt2d.y.ToString() + " 5000");
        GamePlay.gpPostMissionLoad(f);
        NextMissionNum++;
        RedirectCAP(ATK == LW ? RAF : LW, pt2d);
    }

    private void CreatePortMission()
    {
        gameIsOn = true;
        List<GroundStationary> SortedPortCranes = SortedGroundStationaries(PortCranes, RAF, homePlate.Pos(), false);
        GroundStationary gs = SortedPortCranes[0];
        if (SortedPortCranes.Count > 2) gs = SortedPortCranes[rnd.Next(0, 3)];

        target3d = gs.pos;
        mainGSTarget = gs;
        if (gs.country == "gb") ATK = LW;
        else ATK = RAF;
        Point2d pt2d = new Point2d(gs.pos.x, gs.pos.y);
        if (odenDebug) GamePlay.gpLogServer(null, "CreatePortMission: '{0}'", new object[] { gs.Name });
        targetName = (ATK == LW ? "British " : "German ") + PortDockName(pt2d) + " in " + GamePlay.gpSectorName(pt2d.x, pt2d.y);
        ISectionFile f = GamePlay.gpCreateSectionFile();
        if (!playerIsBomber)
        {
            CreateBMBMission(f, pt2d, 3, ATK, null, gs, true);
            CreateBMBMission(f, pt2d, 3, ATK, null, gs, false); // never multiple escorts, they climb to same alt and risk collide
            if (rnd.Next(0, 9) > 6) CreateBMBMission(f, pt2d, 3, ATK, null, gs, false);
            if (rnd.Next(0, 9) > 6) CreateBMBMission(f, pt2d, 3, ATK, null, gs, false);
        };
        f.add("Trigger", "BombersAtTarget", "TPassThrough 5 " + ATK.ToString() + " " + pt2d.x.ToString() + " " + pt2d.y.ToString() + " 5000");
        GamePlay.gpPostMissionLoad(f);
        NextMissionNum++;
        RedirectCAP(ATK == LW ? RAF : LW, pt2d);
    }

    private void CreateIndustryMission()
    {
        gameIsOn = true;
        List<GroundStationary> SortedIndustries = SortedGroundStationaries(Industries, RAF, homePlate.Pos(), false);
        GroundStationary gs = SortedIndustries[0];
        if (SortedIndustries.Count > 2) gs = SortedIndustries[rnd.Next(0, 3)];

        target3d = gs.pos;
        mainGSTarget = gs;
        if (gs.country == "gb") ATK = LW;
        else ATK = RAF;
        Point2d pt2d = new Point2d(gs.pos.x, gs.pos.y);
        if (odenDebug) GamePlay.gpLogServer(null, "CreateIndustryMission: '{0}'", new object[] { gs.Name });
        targetName = (ATK == LW ? "British " : "German ") + "industrial area in " + GamePlay.gpSectorName(pt2d.x, pt2d.y);
        ISectionFile f = GamePlay.gpCreateSectionFile();
        if (!playerIsBomber)
        {
            CreateBMBMission(f, pt2d, 3, ATK, null, gs, true);
            CreateBMBMission(f, pt2d, 3, ATK, null, gs, false); // never multiple escorts, they climb to same alt and risk collide
            if (rnd.Next(0, 9) > 6) CreateBMBMission(f, pt2d, 3, ATK, null, gs, false);
            if (rnd.Next(0, 9) > 6) CreateBMBMission(f, pt2d, 3, ATK, null, gs, false);
        };
        f.add("Trigger", "BombersAtTarget", "TPassThrough 5 " + ATK.ToString() + " " + pt2d.x.ToString() + " " + pt2d.y.ToString() + " 5000");
        GamePlay.gpPostMissionLoad(f);
        NextMissionNum++;
        RedirectCAP(ATK == LW ? RAF : LW, pt2d);
    }

    private void CreateAirbaseMission(int army, bool nearby)
    {   // only LW for now. ToDo: Blenheim dayraids
        gameIsOn = true;
        ATK = army;
        Point2d pt2d = new Point2d(homeBase.x, homeBase.y);
        Point3d pt3d = new Point3d(pt2d.x, pt2d.y, 0);
        target3d = pt3d;
        if (nearby)
        {
            if (army == LW) pt2d = new Point2d(310000, 50000);
            else pt2d = new Point2d(180000, 310000);
            pt3d = new Point3d(pt2d.x, pt2d.y, 0);
        };
        List<AiAirport> saps = SortedAirports(ATK == LW ? RAF : LW, pt2d, true);
        int idx = rnd.Next(1, saps.Count / 2) - 1;
        if (nearby) idx = rnd.Next(0, 3);

        targetAirport = saps[idx];
        if (odenDebug) GamePlay.gpLogServer(null, "CreateAirbaseMission: '{0}'", new object[] { targetAirport.Name() });
        Point2d tgt2d = new Point2d(targetAirport.Pos().x, targetAirport.Pos().y);
        targetName = (ATK == LW ? "RAF " : "Luftwaffe ") + targetAirport.Name() + " airbase in " + GamePlay.gpSectorName(tgt2d.x, tgt2d.y);
        ISectionFile f = GamePlay.gpCreateSectionFile();
        int qty = 3;
        if (army == RAF && gameDateTime.Year < 1941) qty = 5;
        bool circus = false;
        if (army == RAF && gameDateTime.Year > 1940) circus = true;

        if (!playerIsBomber)
        {
            CreateBMBMission(f, tgt2d, qty, ATK, null, null, true);
            CreateBMBMission(f, tgt2d, qty, ATK, null, null, circus);
            if (rnd.Next(0, 9) > 6) CreateBMBMission(f, tgt2d, qty, ATK, null, null, circus);
            if (rnd.Next(0, 9) > 6) CreateBMBMission(f, tgt2d, qty, ATK, null, null, circus);
        };
        f.add("Trigger", "BombersAtTarget", "TPassThrough 5 " + ATK.ToString() + " " + tgt2d.x.ToString() + " " + tgt2d.y.ToString() + " 5000");
        GamePlay.gpPostMissionLoad(f);
        NextMissionNum++;
        RedirectCAP(ATK == LW ? RAF : LW, tgt2d);
    }

    private void CreatePlayerBombMission(Point3d tgt3d, AiActor tgtActor)
    {   // append wps at end of current wps (usually just two from takeoff and join but future might add during flight like 2ndary target)
        AiAircraft pac = (AiAircraft)GamePlay.gpPlayer().Place();
        AiAirGroup pag = pac.AirGroup();
        AiActor leader = pag.GetItems()[0];
        AiAircraft lac = (AiAircraft)leader;
        AiWayPoint[] arrWP = pag.GetWay();
        if (odenDebug) GamePlay.gpLogServer(null, "arrWP: '{0}' at '{1}'", new object[] { arrWP.Length, pag.Pos().z });
        AiWayPoint[] arrNew = new AiWayPoint[4];
        int alt = rnd.Next(2200, 3800);
        double speed = 320f;
        Point2d inb2d = new Point2d(tgt3d.x, tgt3d.y);
        double dir = calcDir(new Point2d(lac.Pos().x, lac.Pos().y), inb2d);
        inb2d = calcPos(inb2d, -2500, dir);
        Point3d pt3d = new Point3d(inb2d.x, inb2d.y, alt);
        AiAirWayPoint inbAWP = new AiAirWayPoint(ref pt3d, speed);
        inbAWP.Action = AiAirWayPointType.NORMFLY;
        arrNew[0] = inbAWP;
        AiAirWayPoint tgtAWP = new AiAirWayPoint(ref tgt3d, speed);
        tgtAWP.Action = AiAirWayPointType.GATTACK_POINT;
        if (tgtActor != null)
        {
            tgtAWP.Target = tgtActor;
            tgtAWP.Action = AiAirWayPointType.GATTACK_TARG;
            if (lac.InternalTypeName().Contains("Ju-87")) tgtAWP.GAttackType = AiAirWayPointGAttackType.DIVE;
            else
            {
                tgtAWP.GAttackType = AiAirWayPointGAttackType.AUTO;
                alt = rnd.Next(500, 900);
            };
        };
        arrNew[1] = tgtAWP;
        if (odenDebug) GamePlay.gpLogServer(null, "tgtAWP: '{0}' at '{1}'", new object[] { tgtAWP.Action, alt });
        inb2d = new Point2d(homeBase.Pos().x, homeBase.Pos().y);
        inb2d = calcPos(inb2d, 2500, dir);
        pt3d = new Point3d(inb2d.x, inb2d.y, alt);
        inbAWP = new AiAirWayPoint(ref pt3d, speed);
        arrNew[2] = inbAWP;
        pt3d = new Point3d(homeBase.Pos().x, homeBase.Pos().y, 0);
        AiAirWayPoint lndAWP = new AiAirWayPoint(ref pt3d, speed);
        lndAWP.Action = AiAirWayPointType.LANDING;
        arrNew[3] = lndAWP;
        /* bite me 
        AiWayPoint[] arrNew = new AiWayPoint[arrWP.Length + 3];
        Array.Copy(arrWP, 0, arrNew, 0, arrWP.Length);
        arrNew[arrWP.Length] = tgtAWP;
        arrNew[arrWP.Length +1] = inbAWP;
        arrNew[arrWP.Length +2] = lndAWP;        
        pag.SetWay(arrNew);
        pag.setTask(AiAirGroupTask.TAKEOFF, pag);
        */
        pag.SetWay(arrNew);
        //foreach (AiAirWayPoint awp in arrNew)
        //    if (odenDebug) GamePlay.gpLogServer(null, "arrWP: '{0}' '{1}'", new object[] { arrNew.Length, awp.Action });

        int dist = (int)(lac.Pos().distance(ref tgt3d));
        dir = calcDir(inb2d, target2d);
        Point2d halfw2d = calcPos(inb2d, dist, dir);
        MissionAirbase home = homeBase;
        ISectionFile f = GamePlay.gpCreateSectionFile();
        string sqnF = CreateCAPMission(f, new Point2d(tgt3d.x, tgt3d.y), 12, pag.Army(), true);
        if (sqnF.Length > 0)
        {
            foreach (MissionAirbase mab in MissionAirbases)
                if (sqnF.Contains(mab.sqn))
                    home = mab;
            AddToFile(f, sqnF + "_Way", "TAKEOFF", home.x.ToString("F2") + " " + home.y.ToString("F2") + " 0 0");
            AddToFile(f, sqnF + "_Way", "ESCORT", halfw2d.x.ToString("F2") + " " + halfw2d.y.ToString("F2") + " " + alt.ToString("F2") + " " + speed.ToString("F2") + " " + pag.Name() + " 1");
            AddToFile(f, sqnF + "_Way", "ESCORT", tgt3d.x.ToString("F2") + " " + tgt3d.y.ToString("F2") + " " + alt.ToString("F2") + " " + speed.ToString("F2") + " " + pag.Name() + " 1");
            AddToFile(f, sqnF + "_Way", "ESCORT", halfw2d.x.ToString("F2") + " " + halfw2d.y.ToString("F2") + " " + alt.ToString("F2") + " " + speed.ToString("F2") + " " + pag.Name() + " 1");
            AddToFile(f, sqnF + "_Way", "LANDING", home.x.ToString("F2") + " " + home.y.ToString("F2") + " " + alt.ToString("F2") + " " + speed.ToString("F2"));
            if (odenDebug) GamePlay.gpLogServer(null, "Escort: '{0}'", new object[] { sqnF });
        };
        if (rnd.Next(0, 9) > 6)
        {   // fighter sweep
            sqnF = CreateCAPMission(f, new Point2d(tgt3d.x, tgt3d.y), 12, pag.Army(), true);
            if (odenDebug) GamePlay.gpLogServer(null, "Sweep {0}", new object[] { sqnF });
            if (sqnF.Length > 0)
            {
                sweepGroups.Add(sqnF);
                foreach (MissionAirbase mab in MissionAirbases)
                    if (sqnF.Contains(mab.sqn))
                        home = mab;
                if (rnd.Next(0, 99) < 10 && alt > 1000) alt = rnd.Next(700, 1500); // lure defence down low
                else alt = rnd.Next(2500, 4500);

                AddToFile(f, sqnF + "_Way", "TAKEOFF", home.x.ToString("F2") + " " + home.y.ToString("F2") + " 0 0");
                AddToFile(f, sqnF + "_Way", "AATTACK_FIGHTERS", halfw2d.x.ToString("F2") + " " + halfw2d.y.ToString("F2") + " " + alt.ToString("F2") + " " + speed.ToString("F2") + " " + pag.Name() + " 1");
                AddToFile(f, sqnF + "_Way", "AATTACK_FIGHTERS", tgt3d.x.ToString("F2") + " " + tgt3d.y.ToString("F2") + " " + alt.ToString("F2") + " " + speed.ToString("F2") + " " + pag.Name() + " 1");
                AddToFile(f, sqnF + "_Way", "AATTACK_FIGHTERS", halfw2d.x.ToString("F2") + " " + halfw2d.y.ToString("F2") + " " + alt.ToString("F2") + " " + speed.ToString("F2") + " " + pag.Name() + " 1");
                AddToFile(f, sqnF + "_Way", "LANDING", home.x.ToString("F2") + " " + home.y.ToString("F2") + " " + alt.ToString("F2") + " " + speed.ToString("F2"));
            };
        };
        GamePlay.gpPostMissionLoad(f);
        NextMissionNum++;

    }

    private void CreateLanding(int idx)
    {
        gameIsOn = true;
        idxTarget = idx;
        MissionMarker mm = MissionMarkers[idx];
        int chiefs = 0;
        foreach (int i in GamePlay.gpArmies())
            chiefs += GamePlay.gpGroundGroups(i).Length;

        ATK = LW;
        ISectionFile f = GamePlay.gpCreateSectionFile();
        string value;
        for (int i = chiefs; i < chiefs + 22; i++)
        {
            if (rnd.Next(1, 10) > 1) value = "Ship.MotorBarge_Large1 de" + " /sleep 0/skill 2/slowfire 1";
            else value = "Ship.Minensuchboote de" + " /sleep 0/skill 2/slowfire 1";
            f.add("Chiefs", i.ToString() + "_Chief", value);
            //GamePlay.gpLogServer(null, "CreateBarges: {0} {1} {2}", new object[] { sect, key, value });
        };
        double dir = 120 + rnd.Next(0, 60);
        if (idx == idxDungeness) dir = 90 + rnd.Next(0, 60);
        Point2d mrk2d = new Point2d(mm.x, mm.y);
        Point2d fleet2d = calcPos(mrk2d, 10000, dir); // 10km should take 30mins
        target2d = fleet2d;

        groundAssaultFrom = "vb_East_of";
        if (dir > 135) groundAssaultFrom = "vb_South_of";

        for (int i = chiefs; i < chiefs + 22; i++)
        {   // start and stop wp for each ship
            Point2d nearbyPoint2d = calcPos(fleet2d, (i - chiefs) * 300, dir + 90);
            value = (nearbyPoint2d.x + rnd.Next(0, 80)).ToString() + " " +
                    (nearbyPoint2d.y + rnd.Next(0, 80)).ToString() + " 38.40  0 2 5.56"; // 5.56 == 20 kmh
            f.add(i.ToString() + "_Chief_Road", "", value);

            nearbyPoint2d = calcPos(mrk2d, (i - chiefs) * 50, dir + 90);
            value = (nearbyPoint2d.x).ToString() + ' ' +
                    (nearbyPoint2d.y).ToString() + " 38.40";
            f.add(i.ToString() + "_Chief_Road", "", value);
        };

        CreateCAPMission(f, fleet2d, 12, LW, false);
        CreateBMBMission(f, mrk2d, 9, LW, null, null, false);
        if (GamePlay.gpIsServerSingle())
        {
            if (rnd.Next(1, 5) > 2) CreateCAPMission(f, mrk2d, 12, LW, false);
            if (rnd.Next(1, 5) > 2) CreateBMBMission(f, mrk2d, 9, LW, null, null, false);
        }
        else
        {
            if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 3) CreateCAPMission(f, mrk2d, 12, LW, false);
            if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 7) CreateCAPMission(f, fleet2d, 12, LW, false);
            if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 11) CreateCAPMission(f, mrk2d, 12, LW, false);

            if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 3) CreateBMBMission(f, mrk2d, 3, LW, null, null, false);
            if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 7) CreateBMBMission(f, mrk2d, 3, LW, null, null, false);
            if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 11) CreateBMBMission(f, mrk2d, 3, LW, null, null, false);
        };
        //f.add("Trigger", "PhaseSequenceOver", "TPassThrough 4 2 " + mrk2d.x.ToString() + " " + mrk2d.y.ToString() + " 1609");
        f.add("Trigger", mm.key, "TPassThrough 4 2 " + mrk2d.x.ToString() + " " + mrk2d.y.ToString() + " 1609");
        if (odenDebug) GamePlay.gpLogServer(null, "gpPostMissionLoad: {0}", new object[] { "SeaLion" });
        targetName = "Operation Sea Lion in " + GamePlay.gpSectorName(mrk2d.x, mrk2d.y);
        GamePlay.gpPostMissionLoad(f);
        NextMissionNum++;
    }

    private void CreateColumns(int idxFr, int idxTo)
    {
        gameIsOn = true;
        idxTarget = idxTo;
        MissionMarker mmFr = MissionMarkers[idxFr];
        MissionMarker mmTo = MissionMarkers[idxTo];
        ATK = mmFr.army;
        Point2d inb2d = new Point2d(mmFr.x, mmFr.y);
        Point2d tgt2d = new Point2d(mmTo.x, mmTo.y);
        double dir = calcDir(tgt2d, inb2d); // inbound dir
        inb2d = calcPos(tgt2d, 10000, dir); // 10km should take 30mins                

        target2d = tgt2d;
        groundAssaultFrom = "vb_North_of";
        if (dir > 134 && dir < 225) groundAssaultFrom = "vb_South_of";
        if (dir > 45 && dir < 135) groundAssaultFrom = "vb_East_of";
        if (dir > 224 && dir < 315) groundAssaultFrom = "vb_West_of";
        targetName = (ATK == LW ? "British " : "German ") + "Army columns heading for " + GamePlay.gpSectorName(tgt2d.x, tgt2d.y);

        ISectionFile f = GamePlay.gpCreateSectionFile();
        for (int cnt = 0; cnt < rnd.Next(1, 5); cnt++)
        {
            string TypeAndCountry = BlueTankList[rnd.Next(1, BlueTankList.Count) - 1];
            if (mmFr.army == RAF) TypeAndCountry = RedTankList[rnd.Next(1, RedTankList.Count) - 1];
            // " Armor.Pz_35t de"/num_units 4"
            int qty = 4;
            columnUnits += qty;
            f.add("Chiefs", cnt.ToString() + "_ChiefTankGr", TypeAndCountry + " /num_units " + qty.ToString());
            f.add(cnt.ToString() + "_ChiefTankGr_Road", "", inb2d.x.ToString("F2") + " " + inb2d.y.ToString("F2") + " 40 0 2 5.56");
            inb2d = calcPos(inb2d, 300, dir);
            gc = new GroundColumn(cnt.ToString() + "_ChiefTankGr", inb2d, tgt2d, GamePlay.gpFindPath(inb2d, 15f, tgt2d, 15f, maddox.game.PathType.GROUND, mmFr.army));
            GroundColumns.Add(gc);
            if (odenDebug) GamePlay.gpLogServer(null, "CreateColumns: {0}", new object[] { TypeAndCountry });
        };
        for (int cnt = 0; cnt < rnd.Next(3, 9); cnt++)
        {
            string TypeAndCountry = BlueVehicleList[rnd.Next(1, BlueVehicleList.Count) - 1];
            if (mmFr.army == RAF) TypeAndCountry = RedVehicleList[rnd.Next(1, RedVehicleList.Count) - 1];
            int qty = 3;
            columnUnits += qty;
            f.add("Chiefs", cnt.ToString() + "_ChiefVehGr", TypeAndCountry + " /num_units " + qty.ToString());
            f.add(cnt.ToString() + "_ChiefVehGr_Road", "", inb2d.x.ToString("F2") + " " + inb2d.y.ToString("F2") + " 40 0 2 5.56");
            inb2d = calcPos(inb2d, 300, dir);
            gc = new GroundColumn(cnt.ToString() + "_ChiefVehGr", inb2d, tgt2d, GamePlay.gpFindPath(inb2d, 15f, tgt2d, 15f, maddox.game.PathType.GROUND, mmFr.army));
            GroundColumns.Add(gc);
            if (odenDebug) GamePlay.gpLogServer(null, "CreateColumns: {0}", new object[] { TypeAndCountry });
        };
        // some weak defence
        for (int cnt = 0; cnt < rnd.Next(2, 3); cnt++)
        {
            Point2d def2d = calcPos(tgt2d, -3000, dir + 30 - rnd.Next(0, 60));
            string TypeAndCountry = RedTankList[rnd.Next(1, RedTankList.Count) - 1];
            if (mmFr.army == RAF) TypeAndCountry = BlueTankList[rnd.Next(1, BlueTankList.Count) - 1];
            int qty = 4;
            f.add("Chiefs", cnt.ToString() + "_ChiefDefGr", TypeAndCountry + " /num_units " + qty.ToString());
            f.add(cnt.ToString() + "_ChiefDefGr_Road", "", def2d.x.ToString("F2") + " " + def2d.y.ToString("F2") + " 40 0 2 5.56");
            def2d = calcPos(def2d, -200, dir + 30 - rnd.Next(0, 60));
            f.add("0_ChiefDefGr_Road", "", def2d.x.ToString("F2") + " " + def2d.y.ToString("F2") + " 40");
            if (odenDebug) GamePlay.gpLogServer(null, "CreateColumns: {0}", new object[] { TypeAndCountry });
        };
        f.add("Trigger", mmTo.key, "TPassThrough 3 " + mmFr.army.ToString() + " " + tgt2d.x.ToString() + " " + tgt2d.y.ToString() + " 1609");
        GamePlay.gpPostMissionLoad(f);
        NextMissionNum++;

        ISectionFile f2 = GamePlay.gpCreateSectionFile();
        //CreateBMBMission(f2, tgt2d, 9, mmFr.army, null, null, true);
        CreateCAPMission(f2, tgt2d, 12, mmFr.army, false);
        if (GamePlay.gpIsServerSingle())
        {
            if (rnd.Next(1, 5) > 2) CreateCAPMission(f2, tgt2d, 12, mmFr.army, false);
            //if (rnd.Next(1, 5) > 2) CreateBMBMission(f2, tgt2d, 9, mmFr.army, null, null, false);
        }
        else
        {
            if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 3) CreateCAPMission(f2, tgt2d, 12, mmFr.army, false);
            if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 7) CreateCAPMission(f2, tgt2d, 12, mmFr.army, false);
            if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 11) CreateCAPMission(f2, tgt2d, 12, mmFr.army, false);

            //if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 3) CreateBMBMission(f2, tgt2d, 3, mmFr.army, null, null, false);
            //if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 7) CreateBMBMission(f2, tgt2d, 3, mmFr.army, null, null, false);
            //if (rnd.Next(1, 5) > 2 && GamePlay.gpRemotePlayers().Length < 11) CreateBMBMission(f2, tgt2d, 3, mmFr.army, null, null, false);
        };
        GamePlay.gpPostMissionLoad(f2);
        NextMissionNum++;
    }

    private string SatelliteStations(string sector)
    {
        switch (sector)
        {
            case "RAF Biggin Hill": return "Biggin_Hill,Gravesend,Manston,Hawkinge";
            case "RAF Kenley": return "Biggin_Hill,Gravesend,Manston,Hawkinge";
            case "RAF Hornchurch": return "Hornchurch,Rochford,Manston,Hawkinge";
            case "RAF Tangmere": return "Tangmere,Westhampnett";
            case "RAF North Weald": return "North_weald,Boscombe_Down"; // Warmwell, Exeter, St. Eval
            case "RAF Debden": return "Debden,Martlesham";
            case "RAF Middle Wallop": return "Andover,Westhampnett";
            case "RAF Northolt": return "Northolt,Hendon";
            case "RAF Duxford": return "Duxford,Fowlmere";
            case "RAF Wittering": return "Wittering,Coltishall,Collyweston";
        };
        return "";
    }

    private string DefaultSkins(string type, string sqn)
    {   // skins relate to planetype
        if (sqn.Contains("_JG") && type.Contains("109"))
        {
            switch (sqn.Substring(sqn.IndexOf("I")))
            {   // "BoB_LW_JG26_I" > return "JG26_1.jpg";
                case "I": return sqn.Substring(7, sqn.LastIndexOf("_") - 6) + "1.jpg"; // staffel
                case "II": return sqn.Substring(7, sqn.LastIndexOf("_") - 6) + "4.jpg";
                case "III": return sqn.Substring(7, sqn.LastIndexOf("_") - 6) + "7.jpg";
            };
        };
        if (sqn.Contains("_ZG26") && type.Contains("110")) return "ZG26.jpg";
        if (sqn.Contains("_ZG76_II") && type.Contains("110")) return "ZG76_II.jpg"; // _III will also catch this one, but yeah
        if (sqn.Contains("ErprGr210") && type.Contains("109")) return "Erpro210.jpg";
        if (sqn.Contains("_LG2_I") && type.Contains("109")) return "LG2_1.jpg"; // staffel
        if (sqn.Contains("_LG2_II") && type.Contains("109")) return "LG2_4.jpg";

        if (type.Contains("Spitfire")) // spit MkI 100oct will fail this, all others have this skin - future Battle of France problem, almost (no 100oct by then?)
            if (gameDateTime.Year > 1940)
                return "Standard_41A.jpg";
        // WIP Black&White Hurri

        return "";
    }

    private string Designation(string className)
    {
        switch (className)
        {
            case "Aircraft.HurricaneMkI_dH5-20": return "Hurricane Mk I";
            case "Aircraft.HurricaneMkI_dH5-20_100oct": return "Hurricane Mk I";
            case "Aircraft.HurricaneMkI": return "Hurricane Mk I";
            case "Aircraft.HurricaneMkI_100oct": return "Hurricane Mk I";
            case "Aircraft.HurricaneMkI_FB": return "Hurricane Mk I";
            case "Aircraft.SpitfireMkI": return "Spitfire Mk I";
            case "Aircraft.SpitfireMkI_100oct": return "Spitfire Mk I";
            case "Aircraft.SpitfireMkIa": return "Spitfire Mk Ia";
            case "Aircraft.SpitfireMkIa_100oct": return "Spitfire Mk Ia";
            case "Aircraft.SpitfireMkIIa": return "Spitfire Mk IIa";
            case "Aircraft.BlenheimMkI": return "Blenheim Mk I";
            case "Aircraft.BlenheimMkIV": return "Blenheim Mk IV";
            case "Aircraft.BlenheimMkIV_Late": return "Blenheim Mk IV";
            case "Aircraft.BlenheimMkIVF": return "Blenheim Mk IVF";
            case "Aircraft.BlenheimMkIVF_Late": return "Blenheim Mk IVF";
            case "Aircraft.WellingtonMkIc": return "Wellington Mk Ic";
            case "Aircraft.BeaufighterMkIF": return "Beaufighter Mk IF";
            case "Aircraft.DefiantMkI": return "Defiant Mk I";
            case "Aircraft.GladiatorMkII": return "Gladiator MkII";
            case "Aircraft.DH82A": return "Tiger Moth";
            case "Aircraft.Bf-109E-1": return "Bf 109E-1";
            case "Aircraft.Bf-109E-1B": return "Bf 109E-1/B";
            case "Aircraft.Bf-109E-3": return "Bf 109E-3";
            case "Aircraft.Bf-109E-3B": return "Bf 109E-3/B";
            case "Aircraft.Bf-109E-4": return "Bf 109E-4";
            case "Aircraft.Bf-109E-4Late": return "Bf 109E-4";
            case "Aircraft.Bf-109E-4B": return "Bf 109E-4/B";
            case "Aircraft.Bf-109E-4B_Late": return "Bf 109E-4/B";
            case "Aircraft.Bf-109E-4N": return "Bf 109E-4/N";
            case "Aircraft.Bf-109E-4N_Late": return "Bf 109E-4/N";
            case "Aircraft.Bf-110C-2": return "Bf 110C-2";
            case "Aircraft.Bf-110C-4": return "Bf 110C-4";
            case "Aircraft.Bf-110C-4N": return "Bf 110C-4/N";
            case "Aircraft.Bf-110C-4Late": return "Bf 110C-4";
            case "Aircraft.Bf-110C-6": return "Bf 110C-6";
            case "Aircraft.Bf-110C-7": return "Bf 110C-7";
            case "Aircraft.Bf-110C-7Late": return "Bf 110C-7";
            case "Aircraft.Ju-87B-2": return "Ju 87B-2";
            case "Aircraft.Do-17Z-1": return "Do 17Z-1";
            case "Aircraft.Do-17Z-2": return "Do 17Z-2";
            case "Aircraft.Do-215B-1": return "Do 215B-1";
            case "Aircraft.Ju-88A-1": return "Ju 88A-1";
            case "Aircraft.He-111P-2": return "He 111P-2";
            case "Aircraft.He-111H-2": return "He 111H-2";
            case "Aircraft.CR42": return "CR.42 Falco";
            case "Aircraft.G50": return "G.50 Freccia";
            case "Aircraft.BR-20M": return "BR.20M Cicogna";
            case "Aircraft.FW-200C-1": return "Fw 200C-1 Condor";
            case "ShipUnit.Tanker_Medium1": return "Cargo Ship";
            case "ShipUnit.MotorBarge_Large1": return "Invasion Barge";
            case "ShipUnit.Minensuchboote": return "Minesweeper";
        };

        return className;
    }

    private string bobDesignation(string internalTypeName)
    {
        return Designation(internalTypeName.Substring(4));
    }

    private GroundStationary GroundStationaryByName(string name)
    {
        foreach (GroundStationary gs in GamePlay.gpGroundStationarys())
            if (gs.Name == name)
                return gs;
        return null;
    }

    private string ChainHomeRadarName(Point2d pt2d)
    {
        AiAirport ap = FindAirport(RAF, pt2d, 0);
        if (ap != null)
        {
            switch (ap.Name())
            {
                case "Ford": return "Poling"; break;
                case "Wilmington": return "Pevensey (Eastbourne)"; break;
                case "Littlestone": return "Rye";
                    //if (pt2d.y > ap.Pos().y) return "Newchurch"; // 1941
                    //else return "Rye";
                    break;
                case "Hawkinge": return "Dover"; break;
                case "Eastchurch": return "Dunkirk (Canterbury West)"; break;
                case "Rochford": return "Canewdon (Rochford)"; break;
                case "Canterbury": return "Wingham (Canterbury East)"; break;
                case "Sandown": return "Ventnor (Isle of Wight)"; break;
            };
            return ap.Name();
        };
        return "N/A";
    }

    private string PortDockName(Point2d pt2d)
    {
        AiAirport ap = FindAirport(RAF, pt2d, 0);
        if (ap != null)
        {
            switch (ap.Name())
            {
                case "Wilmington": return "Hastings Port"; break;
                case "Hawkinge":
                    if (pt2d.y > ap.Pos().y) return "Dover Port";
                    else return "Folkstone Port";
                    break;
                case "Rochford": return "Rochford Port"; break;
                case "Gravesend": return "Tilbury Docks"; break;
            };
            return ap.Name();
        };
        return "N/A";
    }

    private string MissionMarkerName(string key)
    {
        switch (key)
        {
            case fmDungeness: return "Dungeness"; break;
            case fmDover: return "Dover"; break;
            case fmRamsgate: return "Ramsgate"; break;
            case fmCanterbury: return "Canterbury"; break;
            case fmRochester: return "Rochester"; break;
            case fmLondon: return "London"; break;
        };
        return "";
    }

    private string[] FlightSections(int army, int size, string form, int sqnStaffel)
    {   // army defines max, form max per section/flight and iteration used if asking for 3 planes 3 times to be in same sqn/staffel
        string[] Flight = new string[5] { "1", "", "", "", "01" }; // last is 21 in BoB_LW_KG2_I.21
        string value = "";
        // Bomber max 3 +VIC3
        // Zerstörer max 4 +FF
        // Fighter max 4 +FF
        // Recon max 1 +LINEABREAST
        // Transp. max 3 +VIC
        // UK Bomber max 10 +VIC
        // UK Fighter max 6 +VIC3
        // UK Fighter max 4 +FF
        int max = 12;
        if (army == RAF && form == "VIC") max = 40;
        if (army != RAF && form == "VIC") max = 9;
        if (army != RAF && form == "VIC3") max = 9;
        if (size > max) size = max;

        switch (form)
        {
            case "VIC":
                if (army == RAF)
                {   // max 10 but yeah (can go 4x10 in editor)
                    if (size == 2) Flight[0] = "1 2"; // 2-ship flight
                    if (size == 3) Flight[0] = "1 2 3"; // 3-ship flight (single flight)
                    if (size == 4) Flight[0] = "1 2 3 4"; // 4-ship flight (schwarm x3 of staffel)
                    if (size > 4) Flight[0] = "1 2 3 4 5"; // 5-ship flight
                    if (size == 6) Flight[1] = "6";
                    if (size == 7) Flight[1] = "6 7";
                    if (size == 8) Flight[1] = "6 7 8";
                    if (size == 9) Flight[1] = "6 7 8 9";
                    if (size > 9) Flight[1] = "6 7 8 9 10";
                    if (size == 11) Flight[2] = "11";
                    if (size == 12) Flight[2] = "11 12";
                    if (size == 13) Flight[2] = "11 12 13";
                    if (size == 14) Flight[2] = "11 12 13 14";
                    if (size > 14) Flight[2] = "11 12 13 14 15";
                    if (size == 16) Flight[3] = "16";
                    if (size == 17) Flight[3] = "16 17";
                    if (size == 18) Flight[3] = "16 17 18";
                    if (size == 19) Flight[3] = "16 17 18 19";
                    if (size > 19) Flight[3] = "16 17 18 19 20";
                }
                else
                {   // Stuka can go 8 in editor but seems to work as normal staffel of 3x3 kette
                    if (size == 2) Flight[0] = "1 2"; // 2-ship flight
                    if (size > 2) Flight[0] = "1 2 3"; // 3-ship flight
                    if (size == 4) Flight[1] = "4";
                    if (size == 5) Flight[1] = "4 5";
                    if (size > 5) Flight[1] = "4 5 6"; // 2x3-ship staffel
                    if (size == 7) Flight[2] = "7";
                    if (size == 8) Flight[2] = "7 8";
                    if (size > 8) Flight[2] = "7 8 9"; // 3x3-ship staffel = max
                };
                break;
            case "VIC3":
                if (size == 2) Flight[0] = "1 2"; // 2-ship flight
                if (size > 2) Flight[0] = "1 2 3"; // 3-ship flight
                if (size == 4) Flight[1] = "4";
                if (size == 5) Flight[1] = "4 5";
                if (size > 5) Flight[1] = "4 5 6"; // 2x3-ship staffel
                if (size == 7) Flight[2] = "7";
                if (size == 8) Flight[2] = "7 8";
                if (size > 8) Flight[2] = "7 8 9"; // 3x3-ship staffel = max
                if (size == 10) Flight[3] = "10";
                if (size == 11) Flight[3] = "10 11";
                if (size > 11) Flight[3] = "10 11 12";
                break;
            case "VIC6": // RAF 12 must split Flight[0], [1] and [2],[3] in two seperate blocks as VIC3 or fly 2x6 manipulated as VIC6
                if (size == 2) Flight[0] = "1 2";
                if (size == 3) Flight[0] = "1 2 3";
                if (size == 4) Flight[0] = "1 2 3 4";
                if (size == 5) Flight[0] = "1 2 3 4 5";
                if (size > 5) Flight[0] = "1 2 3 4 5 6"; // 2x6-ship flight in x12 squadron
                if (size == 7) Flight[1] = "7";
                if (size == 8) Flight[1] = "7 8";
                if (size == 9) Flight[1] = "7 8 9";
                if (size == 10) Flight[1] = "7 8 9 10";
                if (size == 11) Flight[1] = "7 8 9 10 11";
                if (size > 11) Flight[1] = "7 8 9 10 11 12";
                break;
            case "FINGERFOUR":
                if (size == 2) Flight[0] = "1 2"; // 2-ship flight
                if (size == 3) Flight[0] = "1 2 3"; // 3-ship flight (single flight)
                if (size > 3) Flight[0] = "1 2 3 4"; // 4-ship flight (schwarm x3 of staffel)
                if (size == 5) Flight[1] = "5";
                if (size == 6) Flight[1] = "5 6";
                if (size == 7) Flight[1] = "5 6 7";
                if (size > 7) Flight[1] = "5 6 7 8";
                if (size == 9) Flight[2] = "9";
                if (size == 10) Flight[2] = "9 10";
                if (size == 11) Flight[2] = "9 10 11";
                if (size > 11) Flight[2] = "9 10 11 12";
                break;
            default:
                break;
        };

        double sections = 0;
        if (Flight[0].Length > 0) sections += Math.Pow(2, 0);
        if (Flight[1].Length > 0) sections += Math.Pow(2, 1);
        if (Flight[2].Length > 0) sections += Math.Pow(2, 2);
        if (Flight[3].Length > 0) sections += Math.Pow(2, 3); // BoB_LW_AufklGr10.08, 1 plane in 4th section
        // BoB_LW_AufklGr_14.0F, 1 plane in each of the 4 sections = 1 +2 +4 +8 = 15 = 0xF
        sections.ToString();
        Flight[4] = sqnStaffel.ToString() + String.Format("{0:X}", (int)sections);

        return Flight;

    }

    // flyables
    private static List<string> RAFFighterSet = new List<string>
    {	"Aircraft.HurricaneMkI_dH5-20",
        "Aircraft.HurricaneMkI_dH5-20_100oct",
		"Aircraft.SpitfireMkI",
        "Aircraft.SpitfireMkI_100oct",
    	"Aircraft.HurricaneMkI",
    	"Aircraft.HurricaneMkI_100oct",
		"Aircraft.SpitfireMkIa",
        "Aircraft.SpitfireMkIa_100oct",
		"Aircraft.SpitfireMkIIa",
        "Aircraft.BeaufighterMkIF",
    };
    private static List<string> RAFBomberSet = new List<string> { "Aircraft.BlenheimMkIV" };

    private static List<string> LWFighterSet = new List<string>
    {
        "Aircraft.Bf-109E-1"
        ,"Aircraft.Bf-109E-1B"
        ,"Aircraft.Bf-110C-2"
		,"Aircraft.Bf-109E-3"
		,"Aircraft.Bf-109E-3B"
		,"Aircraft.Bf-110C-4"
		,"Aircraft.Bf-109E-4"
		,"Aircraft.Bf-109E-4B"
		,"Aircraft.Bf-110C-7"
        ,"Aircraft.Bf-110C-7Late"
    };
    private static List<string> LWBomberSet = new List<string> { "Aircraft.Ju-87B-2", "Aircraft.Ju-88A-1", "Aircraft.He-111P-2", "Aircraft.He-111H-2" };

    private static List<string> RAFighterSet = new List<string> { "Aircraft.G50" };
    private static List<string> RABomberSet = new List<string> { "Aircraft.BR-20M" };

    private static List<string> RedTankList = new List<string>
    {   "Armor.Guy_Mk_I gb"		// .50 Vickers MG, 5t/15mm 4x4, 1939
        ,"Armor.Vickers_Mk_VIB gb"	// .50 Vickers MG, 5t/14mm Track, 1936
        ,"Armor.Cruiser_Mk_IV gb"	// QF 2pdr (40mm) Gun, 14t/30mm, 1940
        ,"Armor.Valentine_I gb"	// QF 2pdr (40mm) Gun, 16t/65mm, 1940
        ,"Armor.Matilda_2A gb"		// 2 pdr (40mm) Gun, 25t/78mm, 1939
    };
    private static List<string> BlueTankList = new List<string>
    {     //" Armor.7TP de"	// only 20 used
          //," Armor.Hotchkiss_H39 de" // Panzerkampfwagen 38H 735(f) not frontline use
          //," Armor.Krupp_L2H43_Protze_Flak30 de"
          " Armor.SdKfz_222 de"	// 20mm KwK30, 4t/14mm 4x4, 1935
          ," Armor.Pz_35t de"		// 37mm KwK34, 10t/25mm, 1936
          ," Armor.SdKfz_231_6Rad de" // 20mm KwK30, 8t/15mm 6x4, 1932
          ," Armor.Pz_38t de"		// 37mm KwK38, 10t/30mm, 1939
          ," Armor.Pz_IIC de"		// 20mm KwK30, 7t/13mm, 1936
          ," Armor.StuG_IIIB de"	// 75mm StuK37, 24t/50mm, 1940
          ," Armor.Pz_IIIF de"		// 37mm KwK36, 23t/70mm, 1939
          ," Armor.Pz_IVD de"		// 75mm KwK37, 25t/80mm, 1939
    };
    private static List<string> RedVehicleList = new List<string>
    {   "Vehicle.Scammell_Pioneer_R100 gb"
        ,"Vehicle.AEC_Matador gb"
        ,"Vehicle.Morris_CS8_tent gb"
        ,"Vehicle.Morris_CS8_tank gb"
        ,"Vehicle.Morris_CS8 gb"
        ,"Vehicle.Morris_CDSW gb"
        ,"Vehicle.Guy_Quad-Ant gb"
        ,"Vehicle.Ford_V8_E917_tent gb"
        ,"Vehicle.Ford_V8_E917_open gb"
        ,"Vehicle.Bedford_OY_tent gb"
        ,"Vehicle.Bedford_OY_open gb"
        ,"Vehicle.Bedford_MW_tent gb"
        ,"Vehicle.Bedford_MW_tank gb"
        ,"Vehicle.Bedford_MW_open gb"
        ,"Vehicle.Austin_K2_Ambulance gb"
        ,"Vehicle.Austin_10_Tilly gb"
    };

    private static List<string> BlueVehicleList = new List<string>
    {   "Vehicle.Ford_G917 de"
        ,"Vehicle.SdKfz_9 de"
        ,"Vehicle.SdKfz_7 de"
        ,"Vehicle.SdKfz_252 de"
        ,"Vehicle.SdKfz_11 de"
        ,"Vehicle.SdKfz_10_5_Transport de"
        ,"Vehicle.SdKfz_10_4_Transport de"
        ,"Vehicle.SdKfz_10_1 de"
        //,"Vehicle.SdKfz_9_Trailer de"
        ,"Vehicle.Opel_Blitz_tent de"
        ,"Vehicle.Opel_Blitz_radio_transport de"
        ,"Vehicle.Opel_Blitz_med-tent de"
        ,"Vehicle.Opel_Blitz_fuel de"
        ,"Vehicle.Opel_Blitz_cargo_med de"
        ,"Vehicle.Opel_Blitz_cargo de"
        ,"Vehicle.Kubelwagen de"
        ,"Vehicle.Krupp_L2H43_Protze de"
        ,"Vehicle.Horch_108_Typ_1A_open de"
        ,"Vehicle.Horch_108_Typ_1A de"
    };
    private static List<string> RedColumnList = new List<string>
    {   "Vehicle.GB_HAAA_Transport_Column gb"
        ,"Vehicle.GB_Fuel_Column gb"
        ,"Vehicle.GB_Tank_Transport_Column gb"
    };

    private static List<string> BlueColumnList = new List<string>
    {   " Vehicle.GER_HAAA_Transport_Column de"
        ," Vehicle.GER_Bofors_Column de"
        ," Vehicle.GER_Fuel_Column de"
        ," Vehicle.GER_Tank_Transport_Column de"
    };
}
